unit grdlic;

interface
{$IFDEF FPC} {Free Pascal}
	uses
		SysUtils;
		{$L grdlic_freepascal.obj}
{$ELSE}
	{$IF CompilerVersion < 23.0} {Delphi 6 to Delphi XE}
	uses
		SysUtils, Windows;
		{$L grdlic_borland_delphi.obj}
	{$ELSE}                      {Delphi XE2 and above}
	  {$LEGACYIFEND ON}
	uses
		System.SysUtils, Winapi.Windows;
		{$IFDEF CPUX86}
			{$L grdlic_delphi.obj}  {it is also possible to use grdlic_borland_delphi.obj}
		{$ELSE}
			{$L grdlic_msvc.obj}
		{$ENDIF}
	{$IFEND}
{$ENDIF}

const SlkVersion = 3.22;

const
  GRD_INVALID_HANDLE_VALUE   = $FFFFFFFF;

// GrdStatus
  GRD_OK                                          = 0;        { Successful operation status }
  GRD_INVALID_HANDLE                              = 1;        { Invalid handle }
  GRD_FEATURE_NOT_FOUND                           = 2;        { Feature not found }
  GRD_FEATURE_RESOURCE_EXHAUST                    = 3;        { Feature network resource exhausted }
  GRD_INVALID_FEATURE                             = 4;        { Invalid feature data }
  GRD_INVALID_SERIAL_NUMBER                       = 5;        { Serial number is invalid }
  GRD_INVALID_LICENSE                             = 6;        { Can't read DL license file because of hardware mismatched or part of file header is corrupted }
  GRD_LICENSE_NOT_ACTIVATED                       = 7;        { License with this licenseId has not been activated yet or reactivation required }
  GRD_LICENSE_ALREADY_ACTIVATED                   = 8;        { License already activated }
  GRD_GET_HARDWARE_ID_ERROR                       = 9;        { Can not get hardware id }
  GRD_NOT_ENOUGH_HARWARE_TO_BIND                  = 10;       { Insufficient number of computer components to activate }
  GRD_REQUEST_FAILED                              = 12;       { Unable to send request }
  GRD_GET_RESPONSE_FAILED                         = 13;       { Unable to receive response }
  GRD_SERVER_NOT_FOUND                            = 14;       { Unable to find server }
  GRD_INCORRECT_RESPONSE_DATA                     = 15;       { Incorrect response format or data not valid }
  GRD_FILE_CREATION_ERROR                         = 16;       { Error during creation of file (e.g. license file) }
  GRD_SERIAL_NUMBER_RESOURCE_EXHAUSTED            = 17;       { Serial number resource exhausted }
  GRD_NO_UPDATES_AVAILABLE                        = 18;       { No updates available for license }
  GRD_ACTIVATION_NOT_AVAILABLE                    = 19;       { Activation not enabled by software vendor reasons }
  GRD_NO_CUSTOMER_SPECIFIED                       = 20;       { No customer info specified }
  GRD_CAN_NOT_ACTIVATE_NON_DEMO_LICENSE           = 21;       { Can't activate non demo license }
  GRD_OUTDATED_API_VERSION                        = 22;       { API version too low }
  GRD_INVALID_PARAMETER                           = 23;       { Invalid function argument }
  GRD_BUFFER_TOO_SMALL                            = 24;       { Buffer is not enough to perform operation }
  GRD_INTERNAL_ERROR                              = 25;       { Internal error }
  GRD_LICENSE_IS_BANNED                           = 26;       { Software vendor has banned serial number and license }
  GRD_NO_LICENSE_AVAILABLE                        = 27;       { No license available (e.g. order access time is over }
  GRD_FEATURE_RUNCOUNTER_EXHAUST                  = 28;       { Feature execution counter is exhausted }
  GRD_FEATURE_EXPIRED                             = 29;       { Feature life time is expired }
  GRD_INVALID_PUBLIC_KEY                          = 30;       { Invalid public key }
  GRD_NO_SERVICE                                  = 31;       { Service is not supported }
  GRD_UNABLE_SEND_REQUEST_TO_ADMIN_RUNTIME        = 32;       { Admin runtime is not installed or outdated }
  GRD_DONGLE_NOT_FOUND                            = 33;       { Guardant dongle not found }
  GRD_NO_RESULTS_FOUND                            = 34;       { No any results found }
  GRD_INVALID_DIGEST                              = 35;       { Invalid message or digest }
  GRD_FEATURE_INACTIVE                            = 36;       { The period of using feature has not yet arrived or the feature is not active  }
  GRD_MEMORY_OUT_OF_RANGE                         = 37;       { Access to memory beyond its range }
  GRD_ACCESS_DENIED                               = 38;       { Access denied }
  GRD_NUMBER_ATTEMPTS_EXHAUSTED                   = 39;       { The number of password attempts has been exhausted }
  GRD_SERIAL_NUMBER_NOT_FOUND                     = 40;       { Such serial number not found on server }
  GRD_INVALID_MEMORY                              = 41;       { Invalid memory ID specified or item does not exist }
  GRD_VIRTUAL_ENVIRONMENT_DETECTED                = 42;       { Running in virtual environment detected }
  GRD_REMOTE_DESKTOP_DETECTED                     = 43;       { Detected running in remote desktop mode }
  GRD_OUTDATED_GCC_VERSION                        = 44;       { Outdated Gurdant Control Center version }
  GRD_REHOST_IS_NOT_ALLOWED                       = 45;       { Rehost is not allowed }
  GRD_STORAGE_CORRUPTED                           = 46;       { DL storage corrupted }
  GRD_STORAGE_ID_MISMATCH                         = 47;       { DL storage ID mismatch (input is not intended for current storage) }
  GRD_INVALID_UPDATE_OR_ALREADY_INSTALLED         = 48;       { Invalid update for DL received or update has been installed before }
  GRD_INVALID_REHOST_DATA                         = 49;       { Invalid rehost dat? sent to server }
  GRD_HARDWARE_ID_MISMATCH                        = 50;       { Hardware ID mismatch }
  GRD_REACTIVATION_IS_NOT_ALLOWED                 = 51;       { Reactivation is not allowed because of license contains write memory or features with counter }
  GRD_RECIPIENT_HAS_THE_SAME_LICENSE              = 52;       { Recipient has the same license activated as the license trying to rehost to it }
  GRD_CAN_NOT_DETACH_SOME_FEATURES                = 53;       { Detach failed for some of chosen features }
  GRD_CANCEL_PREVIOUS_DETACH_REQUIRED             = 54;       { Required cancel previous detach before making new one }
  GRD_DETACHED_LICENSE_EXPIRED                    = 55;       { Detached license already exprired }
  GRD_DETACHED_LICENSE_NOT_FOUND                  = 56;       { Detached license not found }
  GRD_DETACH_FORBIDDEN_BY_GCC                     = 57;       { Detach forbidden by Gurdant Control Center }
  GRD_TRIAL_LICENSE_EXPIRED                       = 58;       { Trial license expired }
  GRD_ORERATION_IS_NOT_ALLOWED                    = 59;       { Operation is not allowed }
  GRD_RTC_ERROR                                   = 60;       { Timer error }

  // GrdRemoteMode
  GRD_LOCAL_MODE_ON  = $01;
  GRD_REMOTE_MODE_ON = $02;

  // GrdDongleModel
  GRD_DONGLE_MODEL_SIGN = $80;
  GRD_DONGLE_MODEL_CODE = $100;
  GRD_DONGLE_MODEL_DL   = $400;

  // GrdDriverType
  GRD_DRIVER_TYPE_NONE   = 0;
  GRD_DRIVER_TYPE_HID    = 1;
  GRD_DRIVER_TYPE_WINUSB = 2;
  GRD_DRIVER_TYPE_VENDOR = 3;

  // GrdFirmwareState
  GRD_FIRMWARE_STATE_OK                    = 0;
  GRD_FIRMWARE_STATE_CRC_ERROR             = 1;
  GRD_REMOTE_UPDATE_MODE_ON                = 2;
  GRD_FIRMWARE_STATE_TIMER_FREQUENCY_ERROR = 4;
  GRD_FIRMWARE_STATE_TIMER_INTEGRITY_ERROR = 8;
  GRD_FIRMWARE_STATE_TIMER_BATTERY_ERROR   = 16;
  GRD_FIRMWARE_STATE_TIMER_COMMINICATION_ERROR = 32;
  GRD_FIRMWARE_STATE_TIMER_STOPPED = 64;
  GRD_FIRMWARE_STATE_VTC_RUNNING = 128;

  // GrdConsumptionMode
  GRD_CONSUMPTION_MODE_PERSTATION = $00;
  GRD_CONSUMPTION_MODE_PERLOGIN   = $01;
  GRD_CONSUMPTION_MODE_PERPROCESS = $02;

  // GrdEcc160
  GRD_ECC160_MESSAGE_SIZE    = 20;
  GRD_ECC160_PUBLIC_KEY_SIZE = 40;
  GRD_ECC160_DIGEST_SIZE     = 40;

  // GrdFeatureEncryptMode
  GRD_EM_ECB                = $00;  { Electronic Code Book encryption mode (set by default) }
  GRD_EM_CBC                = $01;  { Cipher Block Chaining encryption mode }
  GRD_EM_CFB                = $02;  { Cipher Feed Back encryption mode. Is allowed only if GRD_SOFTWARE_ACCELERATION flag is provided }
  GRD_EM_OFB                = $04;  { Output Feed Back encryption mode. Is allowed only if GRD_SOFTWARE_ACCELERATION flag is provided }
  GRD_NO_COUNTER_DECREMENT  = $100; { Execute encryption/decryption without counter decrement (used another cryptographic key) }
  GRD_SOFTWARE_ACCELERATION = $200; { Software acceleration for encryption/decryption of a large amount of data }

  // GrdFeatureLicenseType
  GRD_PERPETUAL             = $01;  { License without restrictions }
  GRD_TIME_PERIOD           = $02;  { License with a period of time }
  GRD_EXPIRATION_DATE       = $03;  { License with expiration date }
  GRD_NUMBER_OF_DAYS        = $04;  { License with the expiration of a given number of days }
  GRD_EXECUTIONS            = $05;  { License with a limited number of launches }

  // GrdFeatureFlags
  FEATURE_VM_FORBIDDEN      = $04;    { Work on VM is prohibited }
  FEATURE_VTC_FORBIDDEN     = $20;    { Virtual timer clock is disabled }
  FEATURE_RDP_FORBIDDEN     = $100;   { Work on RD is prohibited }
  FEATURE_IS_DETACHABLE     = $0800;  { Marked as detachable }
  FEATURE_DETACH_ALLOWED    = $1000;  { Detaching allowed }

  // GrdMemoryWriteMode
  GRD_WRITE_MODE_MOV = $00; { Destination becomes source }
  GRD_WRITE_MODE_XOR = $01; { Destination becomes source XORed with destination }

  // for getting formatted error message
  GRD_LANG_EN = 0;
  GRD_LANG_RU = 1;

type
  grd_uint64 = UInt64;
  grd_int64  = Int64;

  grd_uint32 = Cardinal;
  grd_int32  = Integer;

  grd_uint16 = Word;
  grd_int16  = SmallInt;

  grd_uint8  = Byte;
  grd_int8   = ShortInt;

  GrdHandle  = grd_uint32;
  PGrdHandle = ^GrdHandle;
  Pgrd_uint32 = ^grd_uint32;

  grd_time_t = Integer;

  PGrdVendorCodes = ^TGrdVendorCodes;
  TGrdVendorCodes = packed record
    publicCode:       grd_uint32;
    privateReadCode:  grd_uint32;
    privateWriteCode: grd_uint32;
  end;

  PGrdCustomerInfo = ^TGrdCustomerInfo;
  TGrdCustomerInfo = packed record
    firstName:   array[0..64-1] of AnsiChar;
    lastName:    array[0..64-1] of AnsiChar;
    email:       array[0..128-1] of AnsiChar;
    companyName: array[0..128-1] of AnsiChar;
    phone:       array[0..32-1] of AnsiChar;
  end;

  PGrdAesContext = ^TGrdAesContext;
  TGrdAesContext = packed record
    iv: array[0..16 - 1] of grd_uint8; // initialization vector
    inf: array[0..8 - 1] of grd_uint8; // additional encryption info
  end;

  PGrdProductFeaturePair = ^TGrdProductFeaturePair;
  TGrdProductFeaturePair = packed record
    productNumber: grd_uint32;
    featureNumber: grd_uint32;
  end;

// ========================================================================== //
// =========================== Feature API ================================== //
// ========================================================================== //

function GrdGetApiVersion(majorVersion: Pointer; minorVersion: Pointer): Integer; stdcall; external;
function GrdGetErrorMessage(errorCode: Integer; lang: Integer; msgBuf: Pointer; msgBufSize: grd_uint32): Integer; stdcall; external;
function GrdGetLicenseInfo(visibilityJson: PAnsiChar; const codes: PGrdVendorCodes; licenseInfoJsonArray: PPointer): Integer; stdcall; external;
function GrdGetSessionInfo(visibilityJson: PAnsiChar; const codes: PGrdVendorCodes; sessionInfoJsonArray: PPointer): Integer; stdcall; external;
function GrdGetHostInfo(visibilityJson: PAnsiChar; hostInfoJsonArray: PPointer): Integer; stdcall; external;
function GrdFeatureLogin(featureId: grd_uint32; const codes: PGrdVendorCodes; visibilityJson: PAnsiChar; handle: PGrdHandle): Integer; stdcall; external;
function GrdFeatureLogout(handle: GrdHandle): Integer; stdcall;  external;
function GrdFeatureCheck(handle: GrdHandle; publicKeySize: grd_uint32; const publicKey: Pointer): Integer; stdcall; external;
function GrdFeatureGetInfo(handle: GrdHandle; outLicenseInfoJson: PPointer): Integer; stdcall; external;
function GrdFeatureEncrypt(handle: GrdHandle; dataSize: grd_uint32; data: Pointer; mode: grd_uint32; context: Pointer; contextSize: grd_uint32): Integer; stdcall; external;
function GrdFeatureDecrypt(handle: GrdHandle; dataSize: grd_uint32; data: Pointer; mode: grd_uint32; context: Pointer; contextSize: grd_uint32): Integer; stdcall; external;
function GrdFeatureCryptAndCount(handle: GrdHandle; dataSize: grd_uint32; data: Pointer; mode: grd_uint32; context: Pointer; contextSize: grd_uint32;
                                    decryption: Integer; n: grd_uint32; flags: grd_uint32): Integer; stdcall; external;
function GrdFeatureSign(handle: GrdHandle; messageSize: grd_uint32; message: Pointer; digestSize: grd_uint32; digest: Pointer): Integer; stdcall; external;
function GrdVerifyDigest(publicKeySize: grd_uint32; publicKey: Pointer; messageSize: grd_uint32; message: Pointer; digestSize: grd_uint32; digest: Pointer): Integer; stdcall; external;
function GrdFeatureGetTimeLimit(handle: GrdHandle; remainingTime: Pointer): Integer; stdcall; external;
function GrdGetRealTime(handle: GrdHandle; realTime: Pointer): Integer; stdcall; external;
function GrdFeatureGetMaxConcurrentResource(handle: GrdHandle; maxResource: Pgrd_uint32): Integer; stdcall; external;
function GrdFeatureGetRunCounter(handle: GrdHandle; runCounter: Pgrd_uint32): Integer; stdcall; external;

function GrdMemoryGetSize(handle: GrdHandle; memoryId: grd_uint32; size: pgrd_uint32): Integer; stdcall; external;
function GrdMemoryRead(handle: GrdHandle; memoryId: grd_uint32; password: grd_uint32; dataSize: grd_uint32; offset: grd_uint32; data: Pointer): Integer; stdcall; external;
function GrdMemoryWrite(handle: GrdHandle; memoryId: grd_uint32; password: grd_uint32; mode: grd_uint32; dataSize: grd_uint32; offset: grd_uint32; const data: Pointer): Integer; stdcall; external;
function GrdLedBlink(dongleId: grd_uint32): Integer; stdcall; external;
function GrdSetDriverType(const codes: PGrdVendorCodes; dongleId: grd_uint32; driverType: grd_uint32): Integer; stdcall; external;

// ========================================================================== //
// ========================= Licensing API ================================== //
// ========================================================================== //

function GrdGetHostFingerprint(hostFingerprintJson: PPointer; visibilityJson: PAnsiChar): Integer; stdcall; external;
function GrdLicenseCheckSerialNumberFormat(serialNumber: PAnsiChar): Integer; stdcall; external;
function GrdLicenseActivate(serialNumber: PAnsiChar; host: PAnsiChar; port: grd_uint32; customerInfo: PGrdCustomerInfo; outLicenseId: Pgrd_uint32): Integer; stdcall; external;
function GrdLicenseUpdate(licenseId: grd_uint32; host: PAnsiChar; port: grd_uint32): Integer; stdcall;  external;
function GrdLicenseCheckUpdateIsAvailable(licenseId: grd_uint32; host: PAnsiChar; port: grd_uint32): Integer; stdcall; external;
function GrdLicenseRemove(licenseId: grd_uint32): Integer; stdcall; external;
function GrdLicenseCheckIsNotBanned(licenseId: grd_uint32; host: PAnsiChar; port: grd_uint32): Integer; stdcall; external;
function GrdLicenseCreateActivationRequest(outRequest: PPointer; outRequestSize: Pgrd_uint32): Integer; stdcall; external;
function GrdLicenseSendActivationRequest(serialNumber: PAnsiChar; customerInfo: PGrdCustomerInfo; request: Pointer; requestSize: grd_uint32; host: PAnsiChar; port: grd_uint32; outResponse: PPointer; outResponseSize: Pgrd_uint32): Integer; stdcall; external;
function GrdLicenseCreateUpdateRequest(licenseId: grd_uint32; outRequest: PPointer; outRequestSize: Pgrd_uint32): Integer; stdcall; external;
function GrdLicenseSendUpdateRequest(request: Pointer; requestSize: grd_uint32; host: PAnsiChar; port: grd_uint32; outResponse: PPointer; outResponseSize: Pgrd_uint32): Integer; stdcall; external;
function GrdLicenseExtractForRehost(licenseId: grd_uint32; outDonorLicense: PPointer; outDonorLicenseSize: Pgrd_uint32): Integer; stdcall; external;
function GrdLicenseRehost(host: PAnsiChar; port: grd_uint32; donorLicense: Pointer; donorLicenseSize: grd_uint32): Integer; stdcall; external;
function GrdLicenseSendRehostRequest(host: PAnsiChar; port: grd_uint32; donorLicense: Pointer; donorLicenseSize: grd_uint32; recipientHardwareFingerprint: Pointer; recipientLicense: PPointer; recipientLicenseSize: Pgrd_uint32): Integer; stdcall; external;
function GrdLicenseDetach(visibilityJson: PAnsiChar; featuresList: PGrdProductFeaturePair; featuresCount: grd_uint32; durationInSec: grd_time_t; concurrentResourceToConsume: grd_uint16; recipientHardwareFingerprint: PAnsiChar; recipientLicense: PPointer; recipientLicenseSize: Pgrd_uint32): Integer; stdcall; external;
function GrdLicenseReturnToPool(licenseId: grd_uint32; licenseToReturn: PPointer; licenseToReturnSize: Pgrd_uint32): Integer; stdcall; external;
function GrdLicenseInstall(response: Pointer; responseSize: grd_uint32): Integer; stdcall;  external;
function GrdLicenseInstallRemotely(visibilityJson: PAnsiChar; response: Pointer; responseSize: grd_uint32): Integer; stdcall;  external;
function GrdDongleSetLicense(dongleId: grd_uint32; serialNumber: PAnsiChar; driverType: grd_uint32; host: PAnsiChar; port: grd_uint32): Integer; stdcall;  external;
function GrdFree(buffer: Pointer): Integer; stdcall; external;

{$IFDEF FPC}
  {$IFDEF CPUX64}
   var
      ImageBase : Int64 public name 'ImageBase';
  {$ENDIF}
{$ENDIF}

implementation

end.
