unit grdlic_delphi;

interface

uses
  System.Classes, System.SysUtils, System.DateUtils, System.Generics.Collections, System.JSON, grdlic;

type
  TDongleModel = (dmSign = GRD_DONGLE_MODEL_SIGN, dmCode = GRD_DONGLE_MODEL_CODE, dmDl = GRD_DONGLE_MODEL_DL, dmAll = dmSign+dmCode+dmDl );
  TRemoteMode = (rmLocal = GRD_LOCAL_MODE_ON, rmRemote = GRD_REMOTE_MODE_ON, rmAll = GRD_LOCAL_MODE_ON + GRD_REMOTE_MODE_ON);

  PGrdTime = ^TGrdTime;
  TGrdTime = packed record
    wYear:         Word;
    wMonth:        Word;
    wDayOfWeek:    Word;
    wDay:          Word;
    wHour:         Word;
    wMinute:       Word;
    wSecond:       Word;
    wMilliseconds: Word;
  end;

  PCustomerInfo = ^TCustomerInfo;
  TCustomerInfo = record
    FirstName:   string;
    LastName:    string;
    Email:       string;
    CompanyName: string;
    Phone:       string;
    private
      function GrdCustomerInfo(): TGrdCustomerInfo;
  end;

  TProductFeaturePair = record
    productNumber: Cardinal;
    featureNumber: Cardinal;
  end;
  TProductFeaturePairs = array of TProductFeaturePair;

  TVisibility = record
    LicenseId:     Cardinal;
    RemoteMode:    TRemoteMode;
    DongleModel:   TDongleModel;
    ProductNumber: Cardinal;
    NetworkResourceToConsume: Cardinal;
    HostName: AnsiString;
    BroadcastSearch: Boolean;
    private
      function JsonVisibility(): AnsiString;
  end;

  TFeatureFlagsAsFileds = record
    expired: Boolean;
    vmForbidden: Boolean;
    rdpForbidden: Boolean;
    isDetachable: Boolean;
    isDetachAllowed: Boolean;
  end;

  TFeature = packed record
    number: Word;
    flags: Cardinal;
    remoteMode: Cardinal;
    consumptionMode: Cardinal;
    validFromDate: Cardinal;
    validUpToDate: Cardinal;
    restOfLifeTime: Cardinal;
    maxRunCounter: Cardinal;
    maxConcurrentResource: Cardinal;
    floatingResource: Cardinal;
    reservedResource: Cardinal;
    detachedResource: Cardinal;
    sessionsCount: Integer;
    currentRunCounterValue: Cardinal;
    name: String;
    licenseType: Integer;
    flagsAsFields: TFeatureFlagsAsFileds;
  end;
  TFeatures = array of TFeature;

  PProduct = ^TProduct;
  TProduct = packed record
    number: Cardinal;
    modification: Cardinal;
    flags: Cardinal;
    featuresCount: Cardinal;
    name: string;
    features: array of TFeature;
  end;
  TProducts = array of TProduct;

  TDongleInfo = packed record
    dongleId: Cardinal;
    detachedFromDongleId: Cardinal;
    detachedFromHostName: String;
    dongleModel: Cardinal;
    typeFlags: Cardinal;
    netResource: Word;
    publicCode: Cardinal;
    driverType: Cardinal;
    dongleMemorySize: Cardinal;
    firmwareState: Cardinal;
    firmwareVersion: Cardinal;
    firmwareFeatures: Cardinal;
    hwMcuVersion: Cardinal;
    containerVersion: Cardinal;
    containerActivationTime: Cardinal;
    containerDeathTime: Cardinal;
    detachDeathTime: Cardinal;
  end;

  TFlagsAsFields = record
    bindHardware: Boolean;
    bindOs: Boolean;
    isDemoLicense: Boolean;
    isRehostAllowed: Boolean;
    isDetachable: Boolean;
    isDetachAllowed: Boolean;
    isDetached: Boolean;
    isTrial: Boolean;
    isTrialLicenseExpired: Boolean;
    timerError: Boolean;
    lowBattery: Boolean;
    externalMicroSd: Boolean;
    driverless: Boolean;
    expired: Boolean;
    inaccessible: Boolean;
    remote: Boolean;
	isVtcEnabled: Boolean;
  end;

  PLicenseInfo = ^TLicenseInfo;
  TLicenseInfo = packed record
    currentUpdate: Cardinal;
    flags: Cardinal;
    freeMemory: Cardinal;
    licenseId: Cardinal;
    vendorPublicCode: Cardinal;
    productsCount: Cardinal;
    vendorCompanyName: string;
    customerFirstName: string;
    customerLastName: string;
    customerEmail: string;
    customerCompanyName: string;
    products: TProducts;
  end;

  TUser = record
    name: String;
    id: UInt64;
  end;

  THost = record
    fqdn: String;
    hostId: Uint64;
    ip: String;
    name: String;
  end;

  TLm = packed record
    hostName: String;
    hostIp: String;
    hostPort: Cardinal;
    lmsId: String;
    versionValue: Cardinal;
    version: String;
    uptimeEng: String;
    uptimeRus: String;
    startTime: Cardinal;
    currentTime: Cardinal;
    currentTimeString: String;
    osArchitecture: String;
    processorCount: Cardinal;
    osName: String;
    protocols: String;
    remote: Boolean;
  end;
  TLms = array of TLm;

  TStorageInfo = packed record
    detachedIds: array of Cardinal;
  end;

  PLicense = ^ TLicense;
  TLicense = packed record
    isBroken: Integer;
    storageInfo: TStorageInfo;
    dongleInfo: TDongleInfo;
    flagsAsFields: TFlagsAsFields;
    sessionsCount: Integer;
    licenseInfo: TLicenseInfo;
    productsCount: Integer;
    lm: TLm;
  end;
  TLicenses = array of TLicense;



  TSessionLicense = packed record
    isBroken: Integer;
    //storageInfo: TStorageInfo;
    dongleInfo: TDongleInfo;
    flagsAsFields: TFlagsAsFields;
    sessionsCount: Integer;
    productsCount: Integer;
    lm: TLm;
  end;

  TSession = packed record
    //  SLK 3.21 //
    featureNumber: Cardinal;
    host: THost;
    issueTime: Cardinal;
    issueTimeText: String;
    licenseId: Cardinal;
    processId: Cardinal;
    processName: String;
    processPath: String;
    productNumber: Cardinal;
    sessionId: Cardinal;
    sessionIdText: String;
    user: TUser;
    // -- SLK 3.21 -- //

    product: TProduct;
    feature: TFeature;
    license: TSessionLicense;

  end;
  TSessions = array of TSession;

  TSessionList = packed record
    sessionCount: Integer;
    sessions: TSessions;
  end;

  TLicenseList = packed record
    licenseCount: Integer;
    licenses: TLicenses;
    sessionCount: Integer;
    sessions: TSessions;
    lms: TLms;
  end;

  TVendorCodes = packed record
    PublicCode: Cardinal;
    PrivateReadCode: Cardinal;
    PrivateWriteCode: Cardinal;
    private
      function GrdVendorCodes(): TGrdVendorCodes;
  end;

  TApiVersion = record
    Major: Cardinal;
    Minor: Cardinal;
  end;

  TGetJsonDataEvent = procedure(Sender: TObject; const AParam: PAnsiChar) of object;

  TLicenseManager = class(TObject)
  private
    FVisibility: TVisibility;
    FCodes: TVendorCodes;
    FLastError: Integer;
    FHost: AnsiString;
    FOnGetJsonData: TGetJsonDataEvent;
    FPort: Cardinal;

    function ParseLicenses(RawJsonData: PAnsiChar): TLicenseList;
    function ParseSessions(RawJsonData: PAnsiChar): TSessionList;
    function GetErrorDescription: string;
    function GetHost: string;
    procedure SetHost(const Value: string);
    function GetApiVersion: TApiVersion;
    procedure SendJsonData(Value: PAnsiChar);
  public
    class function CheckSerialNumberFormat(Serial: string): Integer; static;
    class function ErrorMessage(AErrorCode: Integer): string;

  public
    constructor Create(); overload;
    constructor Create(Visibility: TVisibility); overload;

    property Host: string read GetHost write SetHost;
    property Port: Cardinal read FPort write FPort;
    property Visibility: TVisibility read FVisibility write FVisibility;
    property Codes: TVendorCodes read FCodes write FCodes;

    function LicenseActivate(SerialNumber: string; CustomerInfo: PCustomerInfo = nil): Cardinal;
    function LicenseUpdate(LicenseId: Cardinal): Integer;
    function LicenseCheckUpdateIsAvailable(LicenseId: Cardinal): Integer;
    function LicenseRemove(LicenseId: Cardinal): Integer;
    function LicenseCheckIsNotBanned(LicenseId: Cardinal): Integer;

    function GetLicenseInfo(): TLicenseList;
    function GetSessionInfo(): TSessionList;

    procedure LicenseCreateActivationRequest(var outRequest: TBytes);
    procedure LicenseSendActivationRequest(SerialNumber: string; Request: TBytes; var outResponse: TBytes; CustomerInfo: PCustomerInfo = nil);
    procedure LicenseCreateUpdateRequest(LicenseId: Cardinal; var Request: TBytes);
    procedure LicenseSendUpdateRequest(Request: TBytes; var outResponse: TBytes);

    procedure LicenseInstall(Response: TBytes);

    procedure GetHostFingerprint(var Fingerprint: PAnsiChar);

    procedure LicenseExtractForRehost(LicenseId: Cardinal; var outDonorLicense: TBytes);
    procedure LicenseRehost(DonorLicense: TBytes);
    procedure LicenseSendRehostRequest(donorLicense: TBytes; recipientHardwareFingerprint: PAnsiChar; var recipientLicense: TBytes);

    procedure LicenseDetach(featuresList: TProductFeaturePairs;
                            durationInSec: Integer; concurrentResourceToConsume: Word;
                            recipientHardwareFingerprint: PAnsiChar;
                            var recipientLicense: TBytes);
    procedure LicenseReturnToPool(licenseId: Cardinal; var licenseToReturn: TBytes);

    property Error: Integer read FLastError;
    property ErrorDescription: string read GetErrorDescription;
    property ApiVersion: TApiVersion read GetApiVersion;
    property OnGetJsonData: TGetJsonDataEvent read FOnGetJsonData write FOnGetJsonData;
  end;

  TFeatureObject = class(TObject)
  private
    FFeatureId: Cardinal;
    FCodes: TVendorCodes;
    FVisibility: TVisibility;
    FHandle: GrdHandle;
    FLicenseInfo: TLicense;
    FLastError: Integer;
    function GetErrorDescription: string;
    function ParseLicenseInfo(ARawLicenseInfo: PAnsiChar): TLicense;
  public
    constructor Create(FeatureId: Cardinal; Codes: TVendorCodes; Visibility: TVisibility); overload;
    constructor Create(); overload;
    function GetApiVersion(var MajorVersion, MinorVersion: Cardinal): Integer;

    property Visibility: TVisibility read FVisibility write FVisibility;
    property Codes: TVendorCodes read FCodes write FCodes;
    property FeatureID: Cardinal read FFeatureId write FFeatureId;

    function Login(): Integer;
    function Logout(): Integer;
    function FeatureCheck(PublicKeySize: Cardinal; PublicKey: Pointer): Integer;
    function GetInfo(var ALicenseInfo: TLicense): Integer;
    function Encrypt(Buffer: Pointer; BufferSize: Cardinal): Integer;
    function Decrypt(Buffer: Pointer; BufferSize: Cardinal): Integer;
    function UnlimitedEncrypt(Buffer: Pointer; BufferSize: Cardinal): Integer;
    function UnlimitedDecrypt(Buffer: Pointer; BufferSize: Cardinal): Integer;
    function Sign(Buffer: Pointer; BufferSize: Cardinal; SignResult: Pointer; SignResultSize: Cardinal): Integer;
    function VerifyDigest(PubKey: Pointer; PubKeySize: Cardinal; Buffer: Pointer; BufferSize: Cardinal; SignResult: Pointer; SignResultSize: Cardinal): Integer;
    function GetTimeLimit(var RemainingTime: TGrdTime): Integer;
    function GetRealTime(var RealTime: TGrdTime): Integer;
    function GetMaxConcurrentResource(var MaxResource: Cardinal): Integer;
    function GetRunCounter(var RunCounter: Cardinal): Integer;
    function GetMemorySize(MemoryId: Cardinal; var MemorySize: Cardinal): Integer;
    function MemoryRead(MemoryId, Password, DataSize, Offset: Cardinal; Data: Pointer): Integer;
    function MemoryWrite(MemoryId, Password, Mode, DataSize, Offset: Cardinal; Data: Pointer): Integer;
    function LedBlink(): Integer;

    property Handle: GrdHandle read FHandle;
    property ErrorDescription: string read GetErrorDescription;
  end;

  procedure FileToBuffer(PathToFile: string; var Buffer: TBytes);
  procedure BufferToFile(PathToFile: string; Buffer: TBytes);
  function ErrorToString(Error: Integer): string;

implementation

{$REGION 'Public'}

{ Public }

procedure FileToBuffer(PathToFile: string; var Buffer: TBytes);
var
  len: Integer;
  fs: TFileStream;
begin
  if PathToFile = '' then
    Exit;

  fs:= TFileStream.Create(PathToFile, fmOpenRead);
  len:= fs.Size;
  SetLength(Buffer, len);
  try
    fs.ReadData(Buffer, len)
  finally
    fs.Free;
  end;

end;

procedure BufferToFile(PathToFile: string; Buffer: TBytes);
var
  len: Integer;
  fs: TFileStream;
begin
  len:= length(Buffer);
  if len = 0 then
    Exit;

  fs:= TFileStream.Create(PathToFile, fmCreate);
  try
    fs.WriteData(Buffer, len);
  finally
    fs.Free;
  end;
end;

function ErrorToString(Error: Integer): string;
var
  msgBuf: array [0..255] of AnsiChar;
begin
  GrdGetErrorMessage(Error, GRD_LANG_RU, @msgBuf[0], 256);
  Result:= UTF8ToUnicodeString(msgBuf);
end;

{$ENDREGION}

{$REGION 'TLicenseManager'}

{ TLicenseManager }

constructor TLicenseManager.Create();
begin
  FVisibility.LicenseId:= 0;
  FVisibility.RemoteMode:= rmLocal;
  FVisibility.DongleModel:= dmAll;
  FVisibility.ProductNumber:= 0;
  FVisibility.NetworkResourceToConsume:= 1;
  FVisibility.HostName:= 'localhost';
  FVisibility.BroadcastSearch:= False;

  FCodes.PublicCode:= 0;
  FCodes.PrivateReadCode:= 0;
  FCodes.PrivateWriteCode:= 0;

  FLastError:= 0;

  FHost:= '';
  FPort:= 0;
end;

constructor TLicenseManager.Create(Visibility: TVisibility);
begin
  Create();
  FVisibility:= Visibility;
end;

class function TLicenseManager.ErrorMessage(AErrorCode: Integer): string;
var
  msgBuf: array [0..255] of AnsiChar;
begin
  GrdGetErrorMessage(AErrorCode, GRD_LANG_RU, @msgBuf[0], 256);
  Result:= UTF8ToUnicodeString(msgBuf);
end;

function TLicenseManager.LicenseActivate(SerialNumber: string; CustomerInfo: PCustomerInfo): Cardinal;
var
  SeralNumber: PAnsiChar;
  ci: TGrdCustomerInfo;
  RawCustomerInfo: PGrdCustomerInfo;
begin
  SeralNumber:= PAnsiChar(AnsiString(SerialNumber));
  RawCustomerInfo:= nil;
  if CustomerInfo <> nil then
    begin
      ci:= CustomerInfo.GrdCustomerInfo;
      RawCustomerInfo:= @ci;
    end;


  FLastError:= GrdLicenseActivate(SeralNumber, PAnsiChar(FHost), FPort, RawCustomerInfo, @Result);
end;

function TLicenseManager.LicenseRemove(LicenseId: Cardinal): Integer;
begin
  FLastError:= GrdLicenseRemove(LicenseId);
  Result:= FLastError;
end;

function TLicenseManager.LicenseCheckIsNotBanned(LicenseId: Cardinal): Integer;
begin
  FLastError:= GrdLicenseCheckIsNotBanned(LicenseId, PAnsiChar(FHost), FPort);
  Result:= FLastError;
end;

class function TLicenseManager.CheckSerialNumberFormat(Serial: string): Integer;
var
  SeralNumber: PAnsiChar;
begin
  SeralNumber:= PAnsiChar(AnsiString(Serial));
  Result:= GrdLicenseCheckSerialNumberFormat(SeralNumber);
end;

procedure TLicenseManager.SendJsonData(Value: PAnsiChar);
begin
  if Assigned(FOnGetJsonData) then FOnGetJsonData(Self, Value);
end;

function TLicenseManager.LicenseCheckUpdateIsAvailable(LicenseId: Cardinal): Integer;
begin
  FLastError:= GrdLicenseCheckUpdateIsAvailable(LicenseId, PAnsiChar(FHost), FPort);
  Result:= FLastError;
end;

function TLicenseManager.LicenseUpdate(LicenseId: Cardinal): Integer;
begin
  FLastError:= GrdLicenseUpdate(LicenseId, PAnsiChar(FHost), FPort);
  Result:= FLastError;
end;

procedure TLicenseManager.LicenseCreateActivationRequest(var outRequest: TBytes);
var
  OutBufferSize: grd_uint32;
  OutBuffer: PByte;
begin
  FLastError:= GrdLicenseCreateActivationRequest(@OutBuffer, @OutBufferSize);
  if FLastError <> GRD_OK then
    Exit;

  SetLength(outRequest, OutBufferSize);
  Move(OutBuffer^, outRequest[0], OutBufferSize);
  GrdFree(OutBuffer);
end;

procedure TLicenseManager.LicenseCreateUpdateRequest(LicenseId: Cardinal; var Request: TBytes);
var
  OutRequest: Pointer;
  RequestSize: Cardinal;
begin
  FLastError:= GrdLicenseCreateUpdateRequest(LicenseId, @OutRequest, @RequestSize);
  if FLastError <> GRD_OK then
    Exit;

  SetLength(Request, RequestSize);
  Move(OutRequest^, Request[0], RequestSize);
  GrdFree(OutRequest);
end;

procedure TLicenseManager.LicenseSendActivationRequest(SerialNumber: String; Request: TBytes; var outResponse: TBytes; CustomerInfo: PCustomerInfo);
var
  RequestSize: Cardinal;
  OutBuffer: PByte;
  OutBufferSize: Cardinal;
  Seral: PAnsiChar;
  RawCustomerInfo: PGrdCustomerInfo;
begin
  Seral:= PAnsiChar(AnsiString(SerialNumber));
  RequestSize:= Length(Request);
  RawCustomerInfo:= nil;
  if CustomerInfo <> nil then
    RawCustomerInfo^:= CustomerInfo.GrdCustomerInfo;

  FLastError:= GrdLicenseSendActivationRequest(Seral, RawCustomerInfo, Request, RequestSize, PAnsiChar(FHost), FPort, @OutBuffer, @OutBufferSize);
  if FLastError <> GRD_OK then
    Exit;

  SetLength(outResponse, outBufferSize);
  Move(OutBuffer^, outResponse[0], outBufferSize);
  GrdFree(OutBuffer);
end;

procedure TLicenseManager.LicenseSendUpdateRequest(Request: TBytes; var outResponse: TBytes);
var
  RequestSize: Cardinal;
  OutBuffer: PByte;
  outBufferSize: Cardinal;
begin
  RequestSize:= Length(Request);
  FLastError:= GrdLicenseSendUpdateRequest (Request, RequestSize, PAnsiChar(FHost), FPort, @OutBuffer, @outBufferSize);
  if FLastError <> GRD_OK then
    Exit;

  SetLength(outResponse, outBufferSize);
  Move(OutBuffer^, outResponse[0], outBufferSize);
  GrdFree(OutBuffer);
end;

procedure TLicenseManager.SetHost(const Value: string);
begin
  FHost:= AnsiString(Value);
end;

function TLicenseManager.GetHost: string;
begin
  Result:= string(FHost);
end;

procedure TLicenseManager.LicenseInstall(Response: TBytes);
var
  ResponseSize: Cardinal;
begin
  ResponseSize:= Length(Response);
  FLastError:= GrdLicenseInstall(Response, ResponseSize);
end;

procedure TLicenseManager.LicenseDetach(featuresList: TProductFeaturePairs;
                            durationInSec: Integer; concurrentResourceToConsume: Word;
                            recipientHardwareFingerprint: PAnsiChar;
                            var recipientLicense: TBytes);
var
  Visibility: AnsiString;
  OutBuffer: PByte;
  OutBufferSize: Cardinal;
begin
  Visibility:= FVisibility.JsonVisibility;
  FLastError:= GrdLicenseDetach(PAnsiChar(Visibility), @featuresList[0], Length(featuresList),
                                durationInSec, concurrentResourceToConsume, recipientHardwareFingerprint,
                                @OutBuffer, @OutBufferSize);
  if FLastError <> GRD_OK then
    Exit;
  SetLength(recipientLicense, OutBufferSize);
  Move(OutBuffer^, recipientLicense[0], OutBufferSize);
  GrdFree(OutBuffer);
end;

procedure TLicenseManager.LicenseExtractForRehost(LicenseId: Cardinal; var outDonorLicense: TBytes);
var
  OutBuffer: Pointer;
  OutBufferSize: Cardinal;
begin
  FLastError:= GrdLicenseExtractForRehost(LicenseId, @OutBuffer, @OutBufferSize);
  if FLastError <> GRD_OK then
    Exit;

  SetLength(outDonorLicense, OutBufferSize);
  Move(OutBuffer^, outDonorLicense[0], OutBufferSize);
  GrdFree(OutBuffer);
end;

procedure TLicenseManager.LicenseRehost(DonorLicense: TBytes);
var
  DonorLicenseSize: Cardinal;
begin
  DonorLicenseSize:= Length(DonorLicense);
  FLastError:= GrdLicenseRehost(PAnsiChar(FHost), FPort, DonorLicense, DonorLicenseSize);
end;

procedure TLicenseManager.LicenseReturnToPool(licenseId: Cardinal;
  var licenseToReturn: TBytes);
var
  OutBuffer: PByte;
  OutBufferSize: Cardinal;
begin
  FLastError:= GrdLicenseReturnToPool(licenseId, @OutBuffer, @OutBufferSize);

  if FLastError <> GRD_OK then
    Exit;

  SetLength(licenseToReturn, OutBufferSize);
  Move(OutBuffer^, licenseToReturn[0], OutBufferSize);
  GrdFree(OutBuffer);
end;

procedure TLicenseManager.LicenseSendRehostRequest(donorLicense: TBytes; recipientHardwareFingerprint: PAnsiChar; var recipientLicense: TBytes);
var
  donorLicenseSize: Cardinal;
  OutBuffer: PByte;
  OutBufferSize: Cardinal;
begin
  donorLicenseSize:= Length(donorLicense);
  FLastError:= GrdLicenseSendRehostRequest (PAnsiChar(FHost), FPort, donorLicense, donorLicenseSize,
                                            recipientHardwareFingerprint, @OutBuffer, @OutBufferSize);
  if FLastError <> GRD_OK then
    Exit;

  SetLength(recipientLicense, OutBufferSize);
  Move(OutBuffer^, recipientLicense[0], OutBufferSize);
  GrdFree(OutBuffer);
end;

function TLicenseManager.ParseLicenses(RawJsonData: PAnsiChar): TLicenseList;
var
  l, d, p, f, s: Integer;
  jLicenseList:   TJSONObject;
  jLicenes:       TJSONArray;
  jDetached:      TJSONArray;
  jLicense:       TJSONValue;
  jStorageInfo:   TJSONObject;
  jDongleInfo:    TJSONObject;
  jFlagsAsFields: TJSONObject;
  jLicenseInfo:   TJSONObject;
  jProducts:      TJSONArray;
  jProduct:       TJSONValue;
  jFeatures:      TJSONArray;
  jFeature:       TJSONValue;
  jFeatueFlags:   TJSONObject;
  jSessions:      TJSONArray;
  jSession:       TJSONValue;
  jLm:            TJSONValue;
  jLms:           TJSONArray;
  jUser:          TJSONObject;
  jHost:          TJSONObject;
begin
  jLicenseList:= TJSONObject.Create;
  jLicenseList.Parse(TEncoding.UTF8.GetBytes(string(RawJsonData)), 0);

  Result.licenseCount:= jLicenseList.GetValue<Integer>('licenseCount', 0);

  SetLength(Result.licenses, Result.licenseCount);
  jLicenes:= jLicenseList.GetValue<TJSONArray>('licenses');

  for l:= 0 to Result.licenseCount - 1 do
  begin
    jLicense:= jLicenes.Items[l];

    //isBroken
    Result.licenses[l].isBroken:= jLicense.GetValue<Integer>('isBroken', 0);

    // storageInfo
    jStorageInfo:= jLicense.GetValue<TJSONObject>('storageInfo');
    jDetached:= jStorageInfo.GetValue<TJSONArray>('detachedIds');

    SetLength(Result.licenses[l].storageInfo.detachedIds, jDetached.Count);
    for d:= 0 to jDetached.Count - 1 do
    begin
      Result.licenses[l].storageInfo.detachedIds[d]:= jDetached.Items[d].GetValue<Int64>;
    end; // storageInfo


    // dongleInfo
    jDongleInfo:= jLicense.GetValue<TJSONObject>('dongleInfo');
    Result.licenses[l].dongleInfo.dongleId:=                 jDongleInfo.GetValue<Int64>('dongleId', 0);
    Result.licenses[l].dongleInfo.netResource:=              jDongleInfo.GetValue<Integer>('netResource', 0);
    Result.licenses[l].dongleInfo.detachedFromDongleId:=     jDongleInfo.GetValue<Int64>('detachedFromDongleId', 0);
    Result.licenses[l].dongleInfo.detachedFromHostName:=     jDongleInfo.GetValue<String>('detachedFromHostName', '');
    Result.licenses[l].dongleInfo.containerVersion:=         jDongleInfo.GetValue<Integer>('containerVersion', 0);
    Result.licenses[l].dongleInfo.containerActivationTime:=  jDongleInfo.GetValue<Integer>('containerActivationTime', 0);
    Result.licenses[l].dongleInfo.containerDeathTime:=       jDongleInfo.GetValue<Integer>('containerDeathTime', 0);
    Result.licenses[l].dongleInfo.detachDeathTime:=          jDongleInfo.GetValue<Integer>('detachDeathTime', 0);
    Result.licenses[l].dongleInfo.dongleModel:=              jDongleInfo.GetValue<Integer>('dongleModel', 0);
    Result.licenses[l].dongleInfo.typeFlags:=                jDongleInfo.GetValue<Integer>('typeFlags', 0);
    Result.licenses[l].dongleInfo.publicCode:=               jDongleInfo.GetValue<Int64>('publicCode', 0);
    Result.licenses[l].dongleInfo.driverType:=               jDongleInfo.GetValue<Integer>('driverType', 0);
    Result.licenses[l].dongleInfo.dongleMemorySize:=         jDongleInfo.GetValue<Integer>('dongleMemorySize', 0);
    Result.licenses[l].dongleInfo.firmwareState:=            jDongleInfo.GetValue<Integer>('firmwareState', 0);
    Result.licenses[l].dongleInfo.firmwareVersion:=          jDongleInfo.GetValue<Integer>('firmwareVersion', 0);
    Result.licenses[l].dongleInfo.firmwareFeatures:=         jDongleInfo.GetValue<Integer>('firmwareFeatures', 0);
    Result.licenses[l].dongleInfo.hwMcuVersion:=             jDongleInfo.GetValue<Integer>('hwMcuVersion', 0);

    // flagAsFields
    jFlagsAsFields:= jLicense.GetValue<TJSONObject>('flagsAsFields');
    Result.licenses[l].flagsAsFields.bindHardware:=          jFlagsAsFields.GetValue<Boolean>('bindHardware', false);
    Result.licenses[l].flagsAsFields.bindOs:=                jFlagsAsFields.GetValue<Boolean>('bindOs', false);
    Result.licenses[l].flagsAsFields.isDemoLicense:=         jFlagsAsFields.GetValue<Boolean>('isDemoLicense', false);
    Result.licenses[l].flagsAsFields.isRehostAllowed:=       jFlagsAsFields.GetValue<Boolean>('isRehostAllowed', false);
    Result.licenses[l].flagsAsFields.isDetachable:=          jFlagsAsFields.GetValue<Boolean>('isDetachable', false);
    Result.licenses[l].flagsAsFields.isDetachAllowed:=       jFlagsAsFields.GetValue<Boolean>('isDetachAllowed', false);
    Result.licenses[l].flagsAsFields.isDetached:=            jFlagsAsFields.GetValue<Boolean>('isDetached', false);
    Result.licenses[l].flagsAsFields.isTrial:=               jFlagsAsFields.GetValue<Boolean>('isTrial', false);
    Result.licenses[l].flagsAsFields.isTrialLicenseExpired:= jFlagsAsFields.GetValue<Boolean>('isTrialLicenseExpired', false);
    Result.licenses[l].flagsAsFields.timerError:=            jFlagsAsFields.GetValue<Boolean>('timerError', false);
    Result.licenses[l].flagsAsFields.lowBattery:=            jFlagsAsFields.GetValue<Boolean>('lowBattery', false);
    Result.licenses[l].flagsAsFields.externalMicroSd:=       jFlagsAsFields.GetValue<Boolean>('externalMicroSd', false);
    Result.licenses[l].flagsAsFields.driverless:=            jFlagsAsFields.GetValue<Boolean>('driverless', false);
    Result.licenses[l].flagsAsFields.expired:=               jFlagsAsFields.GetValue<Boolean>('expired', false);
    Result.licenses[l].flagsAsFields.inaccessible:=          jFlagsAsFields.GetValue<Boolean>('inaccessible', false);
    Result.licenses[l].flagsAsFields.remote:=                jFlagsAsFields.GetValue<Boolean>('remote', false);
    Result.licenses[l].flagsAsFields.isVtcEnabled:=          jFlagsAsFields.GetValue<Boolean>('isVtcEnabled', false);

    // sessionCount
    Result.licenses[l].sessionsCount:= jLicense.GetValue<Integer>('sessionsCount', 0);

    // LicenseInfo

    if jLicense.TryGetValue<TJSONObject>('licenseInfo', jLicenseInfo) then
    begin
      Result.licenses[l].licenseInfo.currentUpdate:=           jLicenseInfo.GetValue<Integer>('currentUpdate', 0);
      Result.licenses[l].licenseInfo.flags:=                   jLicenseInfo.GetValue<Integer>('flags', 0);
      Result.licenses[l].licenseInfo.freeMemory:=              jLicenseInfo.GetValue<Integer>('freeMemory', 0);
      Result.licenses[l].licenseInfo.licenseId:=               jLicenseInfo.GetValue<Int64>('licenseId', 0);
      Result.licenses[l].licenseInfo.vendorPublicCode:=        jLicenseInfo.GetValue<Int64>('vendorPublicCode', 0);
      Result.licenses[l].licenseInfo.productsCount:=           jLicenseInfo.GetValue<Integer>('productsCount', 0);
      Result.licenses[l].licenseInfo.vendorCompanyName:=       jLicenseInfo.GetValue<String>('vendorCompanyName', '');
      Result.licenses[l].licenseInfo.customerFirstName:=       jLicenseInfo.GetValue<String>('customerFirstName', '');
      Result.licenses[l].licenseInfo.customerLastName:=        jLicenseInfo.GetValue<String>('customerLastName', '');
      Result.licenses[l].licenseInfo.customerEmail:=           jLicenseInfo.GetValue<String>('customerEmail', '');
      Result.licenses[l].licenseInfo.customerCompanyName:=     jLicenseInfo.GetValue<String>('customerCompanyName', '');

      // Products
      jProducts:= jLicenseInfo.GetValue<TJSONArray>('products');
      SetLength(Result.licenses[l].licenseInfo.products, Result.licenses[l].licenseInfo.productsCount);

      for p:= 0 to Result.licenses[l].licenseInfo.productsCount - 1 do
      begin
        jProduct:= jProducts.Items[p];
        Result.licenses[l].licenseInfo.products[p].number:=        jProduct.GetValue<Integer>('number', 0);
        Result.licenses[l].licenseInfo.products[p].modification:=  jProduct.GetValue<Integer>('modification', 0);
        Result.licenses[l].licenseInfo.products[p].flags:=         jProduct.GetValue<Integer>('flags', 0);
        Result.licenses[l].licenseInfo.products[p].featuresCount:= jProduct.GetValue<Integer>('featuresCount', 0);
        Result.licenses[l].licenseInfo.products[p].name:=          jProduct.GetValue<String>('name', '');

        // features
        jFeatures:= jProduct.GetValue<TJSONArray>('features');
        SetLength(Result.licenses[l].licenseInfo.products[p].features, Result.licenses[l].licenseInfo.products[p].featuresCount);

        for f:= 0 to Result.licenses[l].licenseInfo.products[p].featuresCount - 1 do
        begin
          jFeature:= jFeatures.Items[f];
          Result.licenses[l].licenseInfo.products[p].features[f].number:=                 jFeature.GetValue<Integer>('number', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].flags:=                  jFeature.GetValue<Integer>('flags', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].remoteMode:=             jFeature.GetValue<Integer>('remoteMode', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].consumptionMode:=        jFeature.GetValue<Integer>('consumptionMode', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].validFromDate:=          jFeature.GetValue<Integer>('validFromDate', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].validUpToDate:=          jFeature.GetValue<Integer>('validUpToDate', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].restOfLifeTime:=         jFeature.GetValue<Integer>('restOfLifeTime', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].maxRunCounter:=          jFeature.GetValue<Integer>('maxRunCounter', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].maxConcurrentResource:=  jFeature.GetValue<Integer>('maxConcurrentResource', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].floatingResource:=       jFeature.GetValue<Integer>('floatingResource', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].reservedResource:=       jFeature.GetValue<Integer>('reservedResource', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].detachedResource:=       jFeature.GetValue<Integer>('detachedResource', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].sessionsCount:=          jFeature.GetValue<Integer>('sessionsCount', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].currentRunCounterValue:= jFeature.GetValue<Integer>('currentRunCounterValue', 0);
          Result.licenses[l].licenseInfo.products[p].features[f].name:=                   jFeature.GetValue<String>('name', '');
          Result.licenses[l].licenseInfo.products[p].features[f].licenseType:=            jFeature.GetValue<Integer>('licenseType', 0);

          // flagsAsFields
          jFeatueFlags:= jFeature.GetValue<TJSONObject>('flagsAsFields');

          Result.licenses[l].licenseInfo.products[p].features[f].flagsAsFields.expired:=         jFeatueFlags.GetValue<Boolean>('expired', false);
          Result.licenses[l].licenseInfo.products[p].features[f].flagsAsFields.vmForbidden:=     jFeatueFlags.GetValue<Boolean>('vmForbidden', false);
          Result.licenses[l].licenseInfo.products[p].features[f].flagsAsFields.rdpForbidden:=    jFeatueFlags.GetValue<Boolean>('rdpForbidden', false);
          Result.licenses[l].licenseInfo.products[p].features[f].flagsAsFields.isDetachable:=    jFeatueFlags.GetValue<Boolean>('isDetachable', false);
          Result.licenses[l].licenseInfo.products[p].features[f].flagsAsFields.isDetachAllowed:= jFeatueFlags.GetValue<Boolean>('isDetachAllowed', false);
        end; // features

      end; // products

      jLm:= jLicense.GetValue<TJSONObject>('lm', nil);
      if(assigned(jLm)) then
      begin
        Result.licenses[l].lm.hostName:=          jLm.GetValue<String>('hostName', '');
        Result.licenses[l].lm.hostIp:=            jLm.GetValue<String>('hostIp', '');
        Result.licenses[l].lm.hostPort:=          jLm.GetValue<Integer>('hostPort', 0);
        Result.licenses[l].lm.lmsId:=             jLm.GetValue<String>('lmsId', '');
        Result.licenses[l].lm.versionValue:=      jLm.GetValue<Integer>('versionValue', 0);
        Result.licenses[l].lm.version:=           jLm.GetValue<String>('version', '');
        Result.licenses[l].lm.uptimeEng:=         jLm.GetValue<String>('uptimeEng', '');
        Result.licenses[l].lm.uptimeRus:=         jLm.GetValue<String>('uptimeRus', '');
        Result.licenses[l].lm.startTime:=         jLm.GetValue<Integer>('startTime', 0);
        Result.licenses[l].lm.currentTime:=       jLm.GetValue<Integer>('currentTime', 0);
        Result.licenses[l].lm.currentTimeString:= jLm.GetValue<String>('currentTimeString', '');
        Result.licenses[l].lm.osArchitecture:=    jLm.GetValue<String>('osArchitecture', '');
        Result.licenses[l].lm.processorCount:=    jLm.GetValue<Integer>('processorCount', 0);
        Result.licenses[l].lm.osName:=            jLm.GetValue<String>('osName', '');
        Result.licenses[l].lm.protocols:=         jLm.GetValue<String>('protocols', '');
        Result.licenses[l].lm.remote:=            jLm.GetValue<Boolean>('remote', false);
      end;
    end;
  end; // licenses

  if SlkVersion <= 3.20 then
  begin
    jSessions:= jLicenseList.GetValue<TJSONArray>('sessions');
    Result.sessionCount:= jLicenseList.GetValue<Integer>('sessionCount', 0);
    SetLength(Result.sessions, Result.sessionCount);

    for s:= 0 to Result.sessionCount - 1 do
    begin
      jSession:= jSessions.Items[s];

      // product
      jProduct:= jSession.GetValue<TJSONObject>('product');
      Result.sessions[s].product.number:=        jProduct.GetValue<Integer>('number', 0);
      Result.sessions[s].product.modification:=  jProduct.GetValue<Integer>('modification', 0);
      Result.sessions[s].product.flags:=         jProduct.GetValue<Integer>('flags', 0);
      Result.sessions[s].product.featuresCount:= jProduct.GetValue<Integer>('featuresCount', 0);
      Result.sessions[s].product.name:=          jProduct.GetValue<String>('name', '');

      // feature
      jFeature:= jSession.GetValue<TJSONObject>('feature');
      Result.sessions[s].feature.number:=                 jFeature.GetValue<Integer>('number', 0);
      Result.sessions[s].feature.flags:=                  jFeature.GetValue<Integer>('flags', 0);
      Result.sessions[s].feature.remoteMode:=             jFeature.GetValue<Integer>('remoteMode', 0);
      Result.sessions[s].feature.consumptionMode:=        jFeature.GetValue<Integer>('consumptionMode', 0);
      Result.sessions[s].feature.validFromDate:=          jFeature.GetValue<Integer>('validFromDate', 0);
      Result.sessions[s].feature.validUpToDate:=          jFeature.GetValue<Integer>('validUpToDate', 0);
      Result.sessions[s].feature.restOfLifeTime:=         jFeature.GetValue<Integer>('restOfLifeTime', 0);
      Result.sessions[s].feature.maxRunCounter:=          jFeature.GetValue<Integer>('maxRunCounter', 0);
      Result.sessions[s].feature.maxConcurrentResource:=  jFeature.GetValue<Integer>('maxConcurrentResource', 0);
      Result.sessions[s].feature.floatingResource:=       jFeature.GetValue<Integer>('floatingResource', 0);
      Result.sessions[s].feature.reservedResource:=       jFeature.GetValue<Integer>('reservedResource', 0);
      Result.sessions[s].feature.detachedResource:=       jFeature.GetValue<Integer>('detachedResource', 0);
      Result.sessions[s].feature.sessionsCount:=          jFeature.GetValue<Integer>('sessionsCount', 0);
      Result.sessions[s].feature.currentRunCounterValue:= jFeature.GetValue<Integer>('currentRunCounterValue', 0);
      Result.sessions[s].feature.name:=                   jFeature.GetValue<String>('name', '');
      Result.sessions[s].feature.licenseType:=            jFeature.GetValue<Integer>('licenseType', 0);

      // flagsAsFields
      jFeatueFlags:= jFeature.GetValue<TJSONObject>('flagsAsFields');

      Result.sessions[s].feature.flagsAsFields.expired:=         jFeatueFlags.GetValue<Boolean>('expired', false);
      Result.sessions[s].feature.flagsAsFields.vmForbidden:=     jFeatueFlags.GetValue<Boolean>('vmForbidden', false);
      Result.sessions[s].feature.flagsAsFields.rdpForbidden:=    jFeatueFlags.GetValue<Boolean>('rdpForbidden', false);
      Result.sessions[s].feature.flagsAsFields.isDetachable:=    jFeatueFlags.GetValue<Boolean>('isDetachable', false);
      Result.sessions[s].feature.flagsAsFields.isDetachAllowed:= jFeatueFlags.GetValue<Boolean>('isDetachAllowed', false);

      // license
      jLicense:= jSession.GetValue<TJSONObject>('license');
      Result.sessions[s].license.isBroken:= jLicense.GetValue<Integer>('isBroken', 0);

      // dongleInfo
      jDongleInfo:= jLicense.GetValue<TJSONObject>('dongleInfo');

      Result.sessions[s].license.dongleInfo.dongleId:=                 jDongleInfo.GetValue<Int64>('dongleId', 0);
      Result.sessions[s].license.dongleInfo.detachedFromDongleId:=     jDongleInfo.GetValue<Int64>('detachedFromDongleId', 0);
      Result.sessions[s].license.dongleInfo.detachedFromHostName:=     jDongleInfo.GetValue<String>('detachedFromHostName', '');
      Result.sessions[s].license.dongleInfo.dongleModel:=              jDongleInfo.GetValue<Integer>('dongleModel', 0);
      Result.sessions[s].license.dongleInfo.typeFlags:=                jDongleInfo.GetValue<Integer>('typeFlags', 0);
      Result.sessions[s].license.dongleInfo.netResource:=              jDongleInfo.GetValue<Integer>('netResource', 0);
      Result.sessions[s].license.dongleInfo.publicCode:=               jDongleInfo.GetValue<Int64>('publicCode', 0);
      Result.sessions[s].license.dongleInfo.driverType:=               jDongleInfo.GetValue<Integer>('driverType', 0);
      Result.sessions[s].license.dongleInfo.dongleMemorySize:=         jDongleInfo.GetValue<Integer>('dongleMemorySize', 0);
      Result.sessions[s].license.dongleInfo.firmwareState:=            jDongleInfo.GetValue<Integer>('firmwareState', 0);
      Result.sessions[s].license.dongleInfo.firmwareVersion:=          jDongleInfo.GetValue<Integer>('firmwareVersion', 0);
      Result.sessions[s].license.dongleInfo.firmwareFeatures:=         jDongleInfo.GetValue<Integer>('firmwareFeatures', 0);
      Result.sessions[s].license.dongleInfo.hwMcuVersion:=             jDongleInfo.GetValue<Integer>('hwMcuVersion', 0);
      Result.sessions[s].license.dongleInfo.containerVersion:=         jDongleInfo.GetValue<Integer>('containerVersion', 0);
      Result.sessions[s].license.dongleInfo.containerActivationTime:=  jDongleInfo.GetValue<Integer>('containerActivationTime', 0);
      Result.sessions[s].license.dongleInfo.containerDeathTime:=       jDongleInfo.GetValue<Integer>('containerDeathTime', 0);
      Result.sessions[s].license.dongleInfo.detachDeathTime:=          jDongleInfo.GetValue<Integer>('detachDeathTime', 0);

      // flagAsFields
      jFlagsAsFields:= jLicense.GetValue<TJSONObject>('flagsAsFields');
      Result.sessions[s].license.flagsAsFields.bindHardware:=          jFlagsAsFields.GetValue<Boolean>('bindHardware', false);
      Result.sessions[s].license.flagsAsFields.bindOs:=                jFlagsAsFields.GetValue<Boolean>('bindOs', false);
      Result.sessions[s].license.flagsAsFields.isDemoLicense:=         jFlagsAsFields.GetValue<Boolean>('isDemoLicense', false);
      Result.sessions[s].license.flagsAsFields.isRehostAllowed:=       jFlagsAsFields.GetValue<Boolean>('isRehostAllowed', false);
      Result.sessions[s].license.flagsAsFields.isDetachable:=          jFlagsAsFields.GetValue<Boolean>('isDetachable', false);
      Result.sessions[s].license.flagsAsFields.isDetachAllowed:=       jFlagsAsFields.GetValue<Boolean>('isDetachAllowed', false);
      Result.sessions[s].license.flagsAsFields.isDetached:=            jFlagsAsFields.GetValue<Boolean>('isDetached', false);
      Result.sessions[s].license.flagsAsFields.isTrial:=               jFlagsAsFields.GetValue<Boolean>('isTrial', false);
      Result.sessions[s].license.flagsAsFields.isTrialLicenseExpired:= jFlagsAsFields.GetValue<Boolean>('isTrialLicenseExpired', false);
      Result.sessions[s].license.flagsAsFields.timerError:=            jFlagsAsFields.GetValue<Boolean>('timerError', false);
      Result.sessions[s].license.flagsAsFields.lowBattery:=            jFlagsAsFields.GetValue<Boolean>('lowBattery', false);
      Result.sessions[s].license.flagsAsFields.externalMicroSd:=       jFlagsAsFields.GetValue<Boolean>('externalMicroSd', false);
      Result.sessions[s].license.flagsAsFields.driverless:=            jFlagsAsFields.GetValue<Boolean>('driverless', false);
      Result.sessions[s].license.flagsAsFields.expired:=               jFlagsAsFields.GetValue<Boolean>('expired', false);
      Result.sessions[s].license.flagsAsFields.inaccessible:=          jFlagsAsFields.GetValue<Boolean>('inaccessible', false);
      Result.sessions[s].license.flagsAsFields.remote:=                jFlagsAsFields.GetValue<Boolean>('remote', false);

      // sessionCount
      Result.sessions[s].license.sessionsCount:= jLicense.GetValue<Integer>('sessionsCount', 0);
      Result.sessions[s].license.productsCount:= jLicense.GetValue<Integer>('productsCount', 0);

      jLm:= jLicense.GetValue<TJSONObject>('lm');

      Result.sessions[s].license.lm.hostName:=          jLm.GetValue<String>('hostName', '');
      Result.sessions[s].license.lm.hostIp:=            jLm.GetValue<String>('hostIp', '');
      Result.sessions[s].license.lm.hostPort:=          jLm.GetValue<Integer>('hostPort', 0);
      Result.sessions[s].license.lm.lmsId:=             jLm.GetValue<String>('lmsId', '');
      Result.sessions[s].license.lm.versionValue:=      jLm.GetValue<Integer>('versionValue', 0);
      Result.sessions[s].license.lm.version:=           jLm.GetValue<String>('version', '');
      Result.sessions[s].license.lm.uptimeEng:=         jLm.GetValue<String>('uptimeEng', '');
      Result.sessions[s].license.lm.uptimeRus:=         jLm.GetValue<String>('uptimeRus', '');
      Result.sessions[s].license.lm.startTime:=         jLm.GetValue<Integer>('startTime', 0);
      Result.sessions[s].license.lm.currentTime:=       jLm.GetValue<Integer>('currentTime', 0);
      Result.sessions[s].license.lm.currentTimeString:= jLm.GetValue<String>('currentTimeString', '');
      Result.sessions[s].license.lm.osArchitecture:=    jLm.GetValue<String>('osArchitecture', '');
      Result.sessions[s].license.lm.processorCount:=    jLm.GetValue<Integer>('processorCount', 0);
      Result.sessions[s].license.lm.osName:=            jLm.GetValue<String>('osName', '');
      Result.sessions[s].license.lm.protocols:=         jLm.GetValue<String>('protocols', '');
      Result.sessions[s].license.lm.remote:=            jLm.GetValue<Boolean>('remote', False);

      Result.sessions[s].sessionId:=     jSession.GetValue<Integer>('sessionId', 0);
      Result.sessions[s].sessionIdText:= jSession.GetValue<String>('sessionIdText', '');
      Result.sessions[s].issueTime:=     jSession.GetValue<Integer>('issueTime', 0);
      Result.sessions[s].issueTimeText:= jSession.GetValue<String>('issueTimeText', '');
      Result.sessions[s].processId:=     jSession.GetValue<Integer>('processId', 0);
      Result.sessions[s].processName:=   jSession.GetValue<String>('processName', '');
      Result.sessions[s].processPath:=   jSession.GetValue<String>('processPath', '');

      // user
      jUser:= jSession.GetValue<TJSONObject>('user');

      Result.sessions[s].user.name:= jUser.GetValue<String>('name', '');
      Result.sessions[s].user.id:=   jUser.GetValue<Integer>('id', 0);

      // host
      jHost:= jSession.GetValue<TJSONObject>('host');

      Result.sessions[s].host.fqdn:=   jHost.GetValue<String>('fqdn', '');
      Result.sessions[s].host.hostId:= jHost.GetValue<Uint64>('hostId', 0);
      Result.sessions[s].host.ip:=     jHost.GetValue<String>('ip', '');
      Result.sessions[s].host.name:=   jHost.GetValue<String>('name', '');
    end; // sessions

    // Lms
    jLms:= jLicenseList.GetValue<TJSONArray>('lms');
    SetLength(Result.lms, jLms.Count);

    for l:= 0 to jLms.Count - 1 do
    begin
      jLm:= jLms.Items[l];

      Result.lms[l].hostName:=          jLm.GetValue<String>('hostName', '');
      Result.lms[l].hostIp:=            jLm.GetValue<String>('hostIp', '');
      Result.lms[l].hostPort:=          jLm.GetValue<Integer>('hostPort', 0);
      Result.lms[l].lmsId:=             jLm.GetValue<String>('lmsId', '');
      Result.lms[l].versionValue:=      jLm.GetValue<Integer>('versionValue', 0);
      Result.lms[l].version:=           jLm.GetValue<String>('version', '');
      Result.lms[l].uptimeEng:=         jLm.GetValue<String>('uptimeEng', '');
      Result.lms[l].uptimeRus:=         jLm.GetValue<String>('uptimeRus', '');
      Result.lms[l].startTime:=         jLm.GetValue<Integer>('startTime', 0);
      Result.lms[l].currentTime:=       jLm.GetValue<Integer>('currentTime', 0);
      Result.lms[l].currentTimeString:= jLm.GetValue<String>('currentTimeString', '');
      Result.lms[l].osArchitecture:=    jLm.GetValue<String>('osArchitecture', '');
      Result.lms[l].processorCount:=    jLm.GetValue<Integer>('processorCount', 0);
      Result.lms[l].osName:=            jLm.GetValue<String>('osName', '');
      Result.lms[l].protocols:=         jLm.GetValue<String>('protocols', '');
      Result.lms[l].remote:=            jLm.GetValue<Boolean>('remote', false);
    end; // Lms
  end;
end;


function TLicenseManager.ParseSessions(RawJsonData: PAnsiChar): TSessionList;
var
  s: Integer;
  jSessionList:   TJSONObject;
  jSessions:      TJSONArray;
  jSession:       TJSONValue;
  jUser:          TJSONObject;
  jHost:          TJSONObject;
begin
  jSessionList:= TJSONObject.Create;
  jSessionList.Parse(TEncoding.UTF8.GetBytes(string(RawJsonData)), 0);

  Result.sessionCount:= jSessionList.GetValue<Integer>('sessionCount', 0);

  SetLength(Result.sessions, Result.sessionCount);
  jSessions:= jSessionList.GetValue<TJSONArray>('sessions');

  for s:= 0 to Result.sessionCount - 1 do
  begin
    jSession:= jSessions.Items[s];

    Result.sessions[s].featureNumber:= jSession.GetValue<Integer>('featureNumber', 0);

    // host
    jHost:= jSession.GetValue<TJSONObject>('host');

    Result.sessions[s].host.fqdn:=   jHost.GetValue<String>('fqdn', '');
    Result.sessions[s].host.hostId:= jHost.GetValue<Int64>('hostId', 0);
    Result.sessions[s].host.ip:=     jHost.GetValue<String>('ip', '');
    Result.sessions[s].host.name:=   jHost.GetValue<String>('name', '');

    Result.sessions[s].issueTime:=     jSession.GetValue<Integer>('issueTime', 0);
    Result.sessions[s].issueTimeText:= jSession.GetValue<String>('issueTimeText', '');

    Result.sessions[s].licenseId:=     jSession.GetValue<Int64>('licenseId', 0);

    Result.sessions[s].processId:=     jSession.GetValue<Integer>('processId', 0);
    Result.sessions[s].processName:=   jSession.GetValue<String>('processName', '');
    Result.sessions[s].processPath:=   jSession.GetValue<String>('processPath', '');

    Result.sessions[s].productNumber:= jSession.GetValue<Integer>('productNumber', 0);

    Result.sessions[s].sessionId:=     jSession.GetValue<Integer>('sessionId', 0);
    Result.sessions[s].sessionIdText:= jSession.GetValue<String>('sessionIdText', '');

    // user
    jUser:= jSession.GetValue<TJSONObject>('user');

    Result.sessions[s].user.name:= jUser.GetValue<String>('name', '');
    Result.sessions[s].user.id:=   jUser.GetValue<Int64>('id', 0);
  end; // sessions

end;

function TLicenseManager.GetApiVersion: TApiVersion;
begin
  GrdGetApiVersion(@Result.Major, @Result.Minor);
end;

function TLicenseManager.GetErrorDescription: string;
var
  msgBuf: array [0..255] of AnsiChar;
begin

  GrdGetErrorMessage(FLastError, GRD_LANG_RU, @msgBuf[0], 256);
  Result:= UTF8ToUnicodeString(msgBuf);
end;

function TLicenseManager.GetLicenseInfo: TLicenseList;
var
  Visibility: AnsiString;
  RawLicenseInfoList: PAnsiChar;
  //Codes: TGrdVendorCodes;
begin
  Visibility:= FVisibility.JsonVisibility;
  FLastError:= GrdGetLicenseInfo(@Visibility[1], @FCodes, @RawLicenseInfoList);
  if FLastError <> GRD_OK then
  begin
    Exit;
  end;

  SendJsonData(RawLicenseInfoList);
  Result:= ParseLicenses(RawLicenseInfoList);
  GrdFree(RawLicenseInfoList);
end;

function TLicenseManager.GetSessionInfo: TSessionList;
var
  Visibility: AnsiString;
  RawSessionInfoList: PAnsiChar;
  //Codes: TGrdVendorCodes;
begin
  Visibility:= FVisibility.JsonVisibility;
  FLastError:= GrdGetSessionInfo(@Visibility[1], @FCodes, @RawSessionInfoList);
  if FLastError <> GRD_OK then
  begin
    Exit;
  end;

  SendJsonData(RawSessionInfoList);
  Result:= ParseSessions(RawSessionInfoList);
  GrdFree(RawSessionInfoList);

end;

procedure TLicenseManager.GetHostFingerprint(var Fingerprint: PAnsiChar);
var
  Visibility: AnsiString;
  //hostFingerprintJson: PAnsiChar;
begin
  Visibility:= FVisibility.JsonVisibility;
  FLastError:= GrdGetHostFingerprint(@Fingerprint, PAnsiChar(Visibility));
end;

{$ENDREGION}

{$REGION 'TVisibility'}
  
{ TVisibility }
  
function TVisibility.JsonVisibility: AnsiString;
var
  JsonObject: TJSONObject;
  JSONNestedObject: TJSONObject;
  js: string;
begin
  JsonObject:= TJSONObject.Create;
  if Self.LicenseId <> $FFFFFFFF then
    JsonObject.AddPair('licenseId', TJSONNumber.Create(Self.LicenseId));

  JsonObject.AddPair('remoteMode', TJSONNumber.Create(Cardinal(Self.RemoteMode)));

  JsonObject.AddPair('dongleModel', TJSONNumber.Create(Cardinal(Self.DongleModel)));

  if Self.ProductNumber <> $FFFFFFFF then
    JsonObject.AddPair('productNumber', TJSONNumber.Create(Self.ProductNumber));

  if Self.NetworkResourceToConsume <> $FFFFFFFF then
    JsonObject.AddPair('networkResourceToConsume', TJSONNumber.Create(Self.NetworkResourceToConsume));

  JSONNestedObject:= TJSONObject.Create;

  if Self.HostName <> '' then
    JSONNestedObject.AddPair('hostName', TJSONString.Create(string(Self.HostName)));

  JSONNestedObject.AddPair('broadcastSearch', TJSONBool.Create(Self.BroadcastSearch));

  JsonObject.AddPair('controlCenter', JSONNestedObject);

  js:= JsonObject.ToString;
  Result:= AnsiString(js);

end;
{$ENDREGION}

{$REGION 'TVendorCodes'}

{ TVendorCodes }

function TVendorCodes.GrdVendorCodes: TGrdVendorCodes;
begin
  Result.publicCode:= Self.PublicCode;
  Result.privateReadCode:= Self.PrivateReadCode;
  Result.privateWriteCode:= Self.PrivateWriteCode;
end;

{$ENDREGION}

{$REGION 'TCustomerInfo'}

function TCustomerInfo.GrdCustomerInfo: TGrdCustomerInfo;
var
  Astr: AnsiString;
begin
  Astr:= AnsiString(Self.FirstName);
  Move(Astr[1], Result.firstName[0], Length(Astr));

  Astr:= AnsiString(Self.LastName);
  Move(Astr[1], Result.lastName[0], Length(Astr));

  Astr:= AnsiString(Self.Email);
  Move(Astr[1], Result.email[0], Length(Astr));

  Astr:= AnsiString(Self.CompanyName);
  Move(Astr[1], Result.companyName[0], Length(Astr));

  Astr:= AnsiString(Self.Phone);
  Move(Astr[1], Result.phone[0], Length(Astr));
end;

{$ENDREGION}

{$REGION 'TFeature'}

{ TFeature }

constructor TFeatureObject.Create;
begin
  FFeatureId:= 0;
  FCodes.PublicCode:= 0;
  FCodes.PrivateReadCode:= 0;
  FCodes.PrivateWriteCode:= 0;
  FVisibility.LicenseId:= 0;
  FVisibility.RemoteMode:= rmAll;
  FVisibility.DongleModel:= dmAll;
  FVisibility.ProductNumber:= 0;
  FVisibility.NetworkResourceToConsume:= 1;
  FVisibility.HostName:= '';
  FVisibility.BroadcastSearch:= True;
  FHandle:= 0;
  FLastError:= 0;
end;

constructor TFeatureObject.Create(FeatureId: Cardinal; Codes: TVendorCodes; Visibility: TVisibility);
begin
  FFeatureId:= FeatureId;
  FCodes:= Codes;
  FVisibility:= Visibility;
  FHandle:= 0;
  FLastError:= 0;
end;

function TFeatureObject.GetApiVersion(var MajorVersion, MinorVersion: Cardinal): Integer;
var
  Ver: Integer;
begin
  Ver:= GrdGetApiVersion(@MajorVersion, @MinorVersion);
  if Ver > 40 then
    FLastError:= 0
  else
    FLastError:= Ver;

  Result:= FLastError;
end;

function TFeatureObject.GetErrorDescription: string;
begin
  Result:= ErrorToString(FLastError);
end;

function TFeatureObject.LedBlink(): Integer;
begin
  FLastError:= GrdLedBlink(FLicenseInfo.licenseInfo.licenseId);
  Result:= FLastError;
end;

function TFeatureObject.Login: Integer;
var
  Visibility: AnsiString;
  GrdVendorCodes: TGrdVendorCodes;
begin
  Visibility:= FVisibility.JsonVisibility;
  GrdVendorCodes:= FCodes.GrdVendorCodes();
  FLastError:= GrdFeatureLogin(FFeatureId, @GrdVendorCodes, PAnsiChar(Visibility), @FHandle);
  Result:= FLastError;
end;

function TFeatureObject.Logout: Integer;
begin
  FLastError:= GrdFeatureLogout(FHandle);
  Result:= FLastError;
  FHandle:= 0;
end;

function TFeatureObject.ParseLicenseInfo(ARawLicenseInfo: PAnsiChar): TLicense;
var
  p, f: Integer;
  jLicense:       TJSONObject;
  jDongleInfo:    TJSONObject;
  jFlagsAsFields: TJSONObject;
  jLicenseInfo:   TJSONObject;
  jProducts:      TJSONArray;
  jProduct:       TJSONValue;
  jFeatures:      TJSONArray;
  jFeature:       TJSONValue;
  jFeatueFlags:   TJSONObject;
  jLm:            TJSONValue;
begin

  jLicense:= TJSONObject.Create;
  jLicense.Parse(TEncoding.UTF8.GetBytes(string(ARawLicenseInfo)), 0);

  begin
    //isBroken
    Result.isBroken:= jLicense.GetValue<Integer>('isBroken', 0);

    // storageInfo

    // dongleInfo
    jDongleInfo:= jLicense.GetValue<TJSONObject>('dongleInfo');
    Result.dongleInfo.dongleId:=                 jDongleInfo.GetValue<Int64>('dongleId', 0);
    Result.dongleInfo.detachedFromDongleId:=     jDongleInfo.GetValue<Int64>('detachedFromDongleId', 0);
    Result.dongleInfo.detachedFromHostName:=     jDongleInfo.GetValue<String>('detachedFromHostName', '');
    Result.dongleInfo.dongleModel:=              jDongleInfo.GetValue<Integer>('dongleModel', 0);
    Result.dongleInfo.typeFlags:=                jDongleInfo.GetValue<Integer>('typeFlags', 0);
    Result.dongleInfo.netResource:=              jDongleInfo.GetValue<Integer>('netResource', 0);
    Result.dongleInfo.publicCode:=               jDongleInfo.GetValue<Int64>('publicCode', 0);
    Result.dongleInfo.driverType:=               jDongleInfo.GetValue<Integer>('driverType', 0);
    Result.dongleInfo.dongleMemorySize:=         jDongleInfo.GetValue<Integer>('dongleMemorySize', 0);
    Result.dongleInfo.firmwareState:=            jDongleInfo.GetValue<Integer>('firmwareState', 0);
    Result.dongleInfo.firmwareVersion:=          jDongleInfo.GetValue<Integer>('firmwareVersion', 0);
    Result.dongleInfo.firmwareFeatures:=         jDongleInfo.GetValue<Integer>('firmwareFeatures', 0);
    Result.dongleInfo.hwMcuVersion:=             jDongleInfo.GetValue<Integer>('hwMcuVersion', 0);
    Result.dongleInfo.containerVersion:=         jDongleInfo.GetValue<Integer>('containerVersion', 0);
    Result.dongleInfo.containerActivationTime:=  jDongleInfo.GetValue<Integer>('containerActivationTime', 0);
    Result.dongleInfo.containerDeathTime:=       jDongleInfo.GetValue<Integer>('containerDeathTime', 0);
    Result.dongleInfo.detachDeathTime:=          jDongleInfo.GetValue<Integer>('detachDeathTime', 0);

    // flagAsFields
    jFlagsAsFields:= jLicense.GetValue<TJSONObject>('flagsAsFields');
    Result.flagsAsFields.bindHardware:=          jFlagsAsFields.GetValue<Boolean>('bindHardware', false);
    Result.flagsAsFields.bindOs:=                jFlagsAsFields.GetValue<Boolean>('bindOs', false);
    Result.flagsAsFields.isDemoLicense:=         jFlagsAsFields.GetValue<Boolean>('isDemoLicense', false);
    Result.flagsAsFields.isRehostAllowed:=       jFlagsAsFields.GetValue<Boolean>('isRehostAllowed', false);
    Result.flagsAsFields.isDetachable:=          jFlagsAsFields.GetValue<Boolean>('isDetachable', false);
    Result.flagsAsFields.isDetachAllowed:=       jFlagsAsFields.GetValue<Boolean>('isDetachAllowed', false);
    Result.flagsAsFields.isDetached:=            jFlagsAsFields.GetValue<Boolean>('isDetached', false);
    Result.flagsAsFields.isTrial:=               jFlagsAsFields.GetValue<Boolean>('isTrial', false);
    Result.flagsAsFields.isTrialLicenseExpired:= jFlagsAsFields.GetValue<Boolean>('isTrialLicenseExpired', false);
    Result.flagsAsFields.timerError:=            jFlagsAsFields.GetValue<Boolean>('timerError', false);
    Result.flagsAsFields.lowBattery:=            jFlagsAsFields.GetValue<Boolean>('lowBattery', false);
    Result.flagsAsFields.externalMicroSd:=       jFlagsAsFields.GetValue<Boolean>('externalMicroSd', false);
    Result.flagsAsFields.driverless:=            jFlagsAsFields.GetValue<Boolean>('driverless', false);
    Result.flagsAsFields.expired:=               jFlagsAsFields.GetValue<Boolean>('expired', false);
    Result.flagsAsFields.inaccessible:=          jFlagsAsFields.GetValue<Boolean>('inaccessible', false);
    Result.flagsAsFields.remote:=                jFlagsAsFields.GetValue<Boolean>('remote', false);

    // sessionCount
    Result.sessionsCount:= jLicense.GetValue<Integer>('sessionsCount', 0);

    // LicenseInfo
    jLicenseInfo:= jLicense.GetValue<TJSONObject>('licenseInfo');
    Result.licenseInfo.currentUpdate:=           jLicenseInfo.GetValue<Integer>('currentUpdate', 0);
    Result.licenseInfo.flags:=                   jLicenseInfo.GetValue<Integer>('flags', 0);
    Result.licenseInfo.freeMemory:=              jLicenseInfo.GetValue<Integer>('freeMemory', 0);
    Result.licenseInfo.licenseId:=               jLicenseInfo.GetValue<Int64>('licenseId', 0);
    Result.licenseInfo.vendorPublicCode:=        jLicenseInfo.GetValue<Int64>('vendorPublicCode', 0);
    Result.licenseInfo.productsCount:=           jLicenseInfo.GetValue<Integer>('productsCount', 0);
    Result.licenseInfo.vendorCompanyName:=       jLicenseInfo.GetValue<String>('vendorCompanyName', '');
    Result.licenseInfo.customerFirstName:=       jLicenseInfo.GetValue<String>('customerFirstName', '');
    Result.licenseInfo.customerLastName:=        jLicenseInfo.GetValue<String>('customerLastName', '');
    Result.licenseInfo.customerEmail:=           jLicenseInfo.GetValue<String>('customerEmail', '');

    // Products
    jProducts:= jLicenseInfo.GetValue<TJSONArray>('products');
    SetLength(Result.licenseInfo.products, Result.licenseInfo.productsCount);

    for p:= 0 to Result.licenseInfo.productsCount - 1 do
    begin
      jProduct:= jProducts.Items[p];
      Result.licenseInfo.products[p].number:=        jProduct.GetValue<Integer>('number', 0);
      Result.licenseInfo.products[p].modification:=  jProduct.GetValue<Integer>('modification', 0);
      Result.licenseInfo.products[p].flags:=         jProduct.GetValue<Integer>('flags', 0);
      Result.licenseInfo.products[p].featuresCount:= jProduct.GetValue<Integer>('featuresCount', 0);
      Result.licenseInfo.products[p].name:=          jProduct.GetValue<String>('name', '');

      // features
      jFeatures:= jProduct.GetValue<TJSONArray>('features');
      SetLength(Result.licenseInfo.products[p].features, Result.licenseInfo.products[p].featuresCount);

      for f:= 0 to Result.licenseInfo.products[p].featuresCount - 1 do
      begin
        jFeature:= jFeatures.Items[f];
        Result.licenseInfo.products[p].features[f].number:=                 jFeature.GetValue<Integer>('number', 0);
        Result.licenseInfo.products[p].features[f].flags:=                  jFeature.GetValue<Integer>('flags', 0);
        Result.licenseInfo.products[p].features[f].remoteMode:=             jFeature.GetValue<Integer>('remoteMode', 0);
        Result.licenseInfo.products[p].features[f].consumptionMode:=        jFeature.GetValue<Integer>('consumptionMode', 0);
        Result.licenseInfo.products[p].features[f].validFromDate:=          jFeature.GetValue<Integer>('validFromDate', 0);
        Result.licenseInfo.products[p].features[f].validUpToDate:=          jFeature.GetValue<Integer>('validUpToDate', 0);
        Result.licenseInfo.products[p].features[f].restOfLifeTime:=         jFeature.GetValue<Integer>('restOfLifeTime', 0);
        Result.licenseInfo.products[p].features[f].maxRunCounter:=          jFeature.GetValue<Integer>('maxRunCounter', 0);
        Result.licenseInfo.products[p].features[f].maxConcurrentResource:=  jFeature.GetValue<Integer>('maxConcurrentResource', 0);
        Result.licenseInfo.products[p].features[f].floatingResource:=       jFeature.GetValue<Integer>('floatingResource', 0);
        Result.licenseInfo.products[p].features[f].reservedResource:=       jFeature.GetValue<Integer>('reservedResource', 0);
        Result.licenseInfo.products[p].features[f].detachedResource:=       jFeature.GetValue<Integer>('detachedResource', 0);
        Result.licenseInfo.products[p].features[f].sessionsCount:=          jFeature.GetValue<Integer>('sessionsCount', 0);
        Result.licenseInfo.products[p].features[f].currentRunCounterValue:= jFeature.GetValue<Integer>('currentRunCounterValue', 0);
        Result.licenseInfo.products[p].features[f].name:=                   jFeature.GetValue<String>('name', '');
        Result.licenseInfo.products[p].features[f].licenseType:=            jFeature.GetValue<Integer>('licenseType', 0);

        // flagsAsFields
        jFeatueFlags:= jFeature.GetValue<TJSONObject>('flagsAsFields');

        Result.licenseInfo.products[p].features[f].flagsAsFields.expired:=         jFeatueFlags.GetValue<Boolean>('expired', false);
        Result.licenseInfo.products[p].features[f].flagsAsFields.vmForbidden:=     jFeatueFlags.GetValue<Boolean>('vmForbidden', false);
        Result.licenseInfo.products[p].features[f].flagsAsFields.rdpForbidden:=    jFeatueFlags.GetValue<Boolean>('rdpForbidden', false);
        Result.licenseInfo.products[p].features[f].flagsAsFields.isDetachable:=    jFeatueFlags.GetValue<Boolean>('isDetachable', false);
        Result.licenseInfo.products[p].features[f].flagsAsFields.isDetachAllowed:= jFeatueFlags.GetValue<Boolean>('isDetachAllowed', false);
      end; // features

    end; // products

    jLm:= jLicense.GetValue<TJSONObject>('lm', nil);
    if(assigned(jLm)) then
    begin
      Result.lm.hostName:=          jLm.GetValue<String>('hostName', '');
      Result.lm.hostIp:=            jLm.GetValue<String>('hostIp', '');
      Result.lm.hostPort:=          jLm.GetValue<Integer>('hostPort', 0);
      Result.lm.lmsId:=             jLm.GetValue<String>('lmsId', '');
      Result.lm.versionValue:=      jLm.GetValue<Integer>('versionValue', 0);
      Result.lm.version:=           jLm.GetValue<String>('version', '');
      Result.lm.uptimeEng:=         jLm.GetValue<String>('uptimeEng', '');
      Result.lm.uptimeRus:=         jLm.GetValue<String>('uptimeRus', '');
      Result.lm.startTime:=         jLm.GetValue<Integer>('startTime', 0);
      Result.lm.currentTime:=       jLm.GetValue<Integer>('currentTime', 0);
      Result.lm.currentTimeString:= jLm.GetValue<String>('currentTimeString', '');
      Result.lm.osArchitecture:=    jLm.GetValue<String>('osArchitecture', '');
      Result.lm.processorCount:=    jLm.GetValue<Integer>('processorCount', 0);
      Result.lm.osName:=            jLm.GetValue<String>('osName', '');
      Result.lm.protocols:=         jLm.GetValue<String>('protocols', '');
      Result.lm.remote:=            jLm.GetValue<Boolean>('remote', false);
    end;
  end; // licenses

end;

function TFeatureObject.FeatureCheck(PublicKeySize: Cardinal; PublicKey: Pointer): Integer;
begin
  FLastError:= GrdFeatureCheck(FHandle, PublicKeySize, PublicKey);
  Result:= FLastError;
end;

function TFeatureObject.GetInfo(var ALicenseInfo: TLicense): Integer;
var
  License: PAnsiChar;
begin
  FLastError:= GrdFeatureGetInfo(FHandle, @License);
  Result:= FLastError;
  if Result <> GRD_OK then
    Exit;

  ALicenseInfo:= ParseLicenseInfo(License);
  FLicenseInfo:= ParseLicenseInfo(License);
  GrdFree(License);
end;

function TFeatureObject.GetMemorySize(MemoryId: Cardinal; var MemorySize: Cardinal): Integer;
begin
  FLastError:= GrdMemoryGetSize(FHandle, MemoryId, @MemorySize);
  Result:= FLastError;
end;

function TFeatureObject.MemoryRead(MemoryId, Password, DataSize, Offset: Cardinal; Data: Pointer): Integer;
begin
  FLastError:= GrdMemoryRead(FHandle, MemoryId, Password, DataSize, Offset, Data);
  Result:= FLastError;
end;

function TFeatureObject.MemoryWrite(MemoryId, Password, Mode, DataSize, Offset: Cardinal; Data: Pointer): Integer;
begin
  FLastError:= GrdMemoryWrite(FHandle, MemoryId, Password, Mode, DataSize, Offset, Data);
  Result:= FLastError;
end;

function TFeatureObject.Encrypt(Buffer: Pointer; BufferSize: Cardinal): Integer;
begin
  FLastError:= GrdFeatureEncrypt(FHandle, BufferSize, Buffer, GRD_EM_ECB, nil, 0);
  Result:= FLastError;
end;

function TFeatureObject.Decrypt(Buffer: Pointer; BufferSize: Cardinal): Integer;
begin
  FLastError:= GrdFeatureDecrypt(FHandle, BufferSize, Buffer, GRD_EM_ECB, nil, 0);
  Result:= FLastError;
end;

function TFeatureObject.UnlimitedEncrypt(Buffer: Pointer; BufferSize: Cardinal): Integer;
begin
  FLastError:= GrdFeatureEncrypt(FHandle, BufferSize, Buffer, GRD_EM_ECB + GRD_NO_COUNTER_DECREMENT, nil, 0);
  Result:= FLastError;
end;

function TFeatureObject.UnlimitedDecrypt(Buffer: Pointer; BufferSize: Cardinal): Integer;
begin
  FLastError:= GrdFeatureDecrypt(FHandle, BufferSize, Buffer, GRD_EM_ECB + GRD_NO_COUNTER_DECREMENT, nil, 0);
  Result:= FLastError;
end;

function TFeatureObject.Sign(Buffer: Pointer; BufferSize: Cardinal; SignResult: Pointer; SignResultSize: Cardinal): Integer;
begin
  FLastError:= GrdFeatureSign(FHandle, BufferSize, Buffer, SignResultSize, SignResult);
  Result:= FLastError;
end;

function TFeatureObject.VerifyDigest(PubKey: Pointer; PubKeySize: Cardinal; Buffer: Pointer; BufferSize: Cardinal; SignResult: Pointer; SignResultSize: Cardinal): Integer;
begin
  FLastError:= GrdVerifyDigest(PubKeySize, PubKey, BufferSize, Buffer, SignResultSize, SignResult);
  Result:= FLastError;
end;

function TFeatureObject.GetTimeLimit(var RemainingTime: TGrdTime): Integer;
var
  RawTime: Int64;
  DelTime: TDateTime;
begin
  FLastError:= GrdFeatureGetTimeLimit(FHandle, @RawTime);
  DelTime:= UnixToDateTime(RawTime, True);
  DecodeDate(DelTime, RemainingTime.wYear, RemainingTime.wMonth, RemainingTime.wDay);
  DecodeTime(DelTime, RemainingTime.wHour, RemainingTime.wMinute, RemainingTime.wSecond, RemainingTime.wMilliseconds);
  RemainingTime.wYear:= RemainingTime.wYear - 1970;
  Result:= FLastError;
end;

function TFeatureObject.GetRealTime(var RealTime: TGrdTime): Integer;
var
  RawTime: Int64;
  DelTime: TDateTime;
begin
  FLastError:= GrdGetRealTime(FHandle, @RawTime);
  DelTime:= UnixToDateTime(RawTime, True);
  DecodeDate(DelTime, RealTime.wYear, RealTime.wMonth, RealTime.wDay);
  DecodeTime(DelTime, RealTime.wHour, RealTime.wMinute, RealTime.wSecond, RealTime.wMilliseconds);
  Result:= FLastError;
end;

function TFeatureObject.GetRunCounter(var RunCounter: Cardinal): Integer;
begin
  FLastError:= GrdFeatureGetRunCounter(FHandle, @RunCounter);
  Result:= FLastError;
end;

function TFeatureObject.GetMaxConcurrentResource(var MaxResource: Cardinal): Integer;
begin
  FLastError:= GrdFeatureGetMaxConcurrentResource(FHandle, @MaxResource);
  Result:= FLastError;
end;


{$ENDREGION}

end.
