using System;
using System.Runtime.InteropServices;
using System.Collections.Generic;
using System.Text;
using System.Resources;
using System.Reflection;
using System.IO;

namespace grdlic
{
    public sealed class GrdlicApi
    {
        private static string GrdDllName = "";

        private static PlatformInvoker GrdGetApiVersionInvoker;
        private static PlatformInvoker GrdGetErrorMessageInvoker;
        private static PlatformInvoker GrdGetLicenseInfoInvoker;
        private static PlatformInvoker GrdLedBlinkInvoker;
        private static PlatformInvoker GrdSetDriverTypeInvoker;
        private static PlatformInvoker GrdFeatureLoginInvoker;
        private static PlatformInvoker GrdFeatureLogoutInvoker;
        private static PlatformInvoker GrdFeatureCheckInvoker;
        private static PlatformInvoker GrdFeatureGetInfoInvoker;
        private static PlatformInvoker GrdFeatureEncryptInvoker;
        private static PlatformInvoker GrdFeatureDecryptInvoker;
        private static PlatformInvoker GrdFeatureCryptAndCountInvoker;
        private static PlatformInvoker GrdFeatureSignInvoker;
        private static PlatformInvoker GrdVerifyDigestInvoker;
        private static PlatformInvoker GrdFeatureGetTimeLimitInvoker;
        private static PlatformInvoker GrdFeatureGetRealTimeInvoker;
        private static PlatformInvoker GrdFeatureGetMaxConcurrentResourceInvoker;
        private static PlatformInvoker GrdFeatureGetRunCounterInvoker;
        private static PlatformInvoker GrdMemoryGetSizeInvoker;
        private static PlatformInvoker GrdMemoryReadInvoker;
        private static PlatformInvoker GrdMemoryWriteInvoker;
        private static PlatformInvoker GrdFreeInvoker;

        private static PlatformInvoker GrdLicenseCheckSerialNumberFormatInvoker;
        private static PlatformInvoker GrdLicenseActivateInvoker;
        private static PlatformInvoker GrdLicenseUpdateInvoker;
        private static PlatformInvoker GrdLicenseCheckUpdateIsAvailableInvoker;
        private static PlatformInvoker GrdLicenseCheckIsNotBannedInvoker;
        private static PlatformInvoker GrdLicenseRemoveInvoker;
        private static PlatformInvoker GrdLicenseCreateActivationRequestInvoker;
        private static PlatformInvoker GrdLicenseCreateUpdateRequestInvoker;
        private static PlatformInvoker GrdLicenseSendActivationRequestInvoker;
        private static PlatformInvoker GrdLicenseSendUpdateRequestInvoker;
        private static PlatformInvoker GrdLicenseExtractForRehostInvoker;
        private static PlatformInvoker GrdLicenseRehostInvoker;
        private static PlatformInvoker GrdLicenseSendRehostRequestInvoker;
        private static PlatformInvoker GrdLicenseDetachInvoker;
        private static PlatformInvoker GrdLicenseReturnToPoolInvoker;
        private static PlatformInvoker GrdLicenseInstallInvoker;
        private static PlatformInvoker GrdGetHostFingerprintInvoker;
        private static PlatformInvoker GrdDongleSetLicenseInvoker;
        private static PlatformInvoker GrdGetSessionInfoInvoker;
        private static PlatformInvoker GrdGetHostInfoInvoker;

        private GrdlicApi()
        {
        }

        private static unsafe int GetCharArraySize(byte* arr)
        {
            int sz = 0;
            while (*arr != 0)
            {
                sz++;
                arr++;
            }

            return sz;
        }

        private static unsafe string GetStringFromByteArray(byte* arr)
        {
            int sz = GetCharArraySize(arr);
            if (sz == 0)
                return "";

            byte[] buf = new byte[sz];
            Marshal.Copy(new IntPtr(arr), buf, 0, buf.Length);
            return Encoding.UTF8.GetString(buf);
        }

        private static unsafe GrdVendorCodes getRawVendorCodes(VendorCodes vc)
        {
            return new GrdVendorCodes(vc.m_publicCode, vc.m_privateReadCode, vc.m_privateWriteCode);
        }

        static GrdlicApi()
        {
            SetApiPlatform();
        }

        private static string GetDefaultNativeLibName()
        {
#if !NETFRAMEWORK
            if (RuntimeInformation.IsOSPlatform(OSPlatform.Linux))
                return "libgrdlic.so";
#endif
            return "grdlic.dll";
        }

        public static void SetPathToNativeLib(string path)
        {
            if (path.Length != 0)
            {
                char p = path[path.Length - 1];
                if (p != '\\' && p != '/')
#if !NETFRAMEWORK
                    path += "/";
#else
                    path += "\\";
#endif
            }

            GrdlicApi.GrdDllName = GetDefaultNativeLibName();
        }

        public static void SetPathToNativeLib(string pathToNativeLib, string libname_x86, string libname_x64)
        {
            if (pathToNativeLib.Length != 0)
            {
                char p = pathToNativeLib[pathToNativeLib.Length - 1];
                if (p != '\\' && p != '/')
#if !NETFRAMEWORK
                    pathToNativeLib += "/";
#else
                    pathToNativeLib += "\\";
#endif
            }

            if (IntPtr.Size == 8)
                GrdlicApi.GrdDllName = pathToNativeLib + libname_x64;
            else
                GrdlicApi.GrdDllName = pathToNativeLib + libname_x86;
        }

        private static void SetApiPlatform()
        {
            if (GrdlicApi.GrdDllName == "")
                GrdlicApi.GrdDllName = GetDefaultNativeLibName();
        }

        public static unsafe Status GrdGetApiVersion(out UInt32 retval, out UInt32 major, out UInt32 minor)
        {
            if (GrdlicApi.GrdGetApiVersionInvoker == null)
            {
                Type[] parameters = { typeof(IntPtr), typeof(IntPtr) };

                GrdlicApi.GrdGetApiVersionInvoker = new PlatformInvoker("GrdGetApiVersion", GrdDllName,
                    typeof(UInt32), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            retval = 0;
            major = 0;
            minor = 0;

            object result = null;

            fixed (UInt32* pMajor = &major)
            fixed (UInt32* pMinor = &minor)
                result = GrdGetApiVersionInvoker.Invoke(new IntPtr(pMajor), new IntPtr(pMinor));
            if (result is UInt32)
            {
                retval = (UInt32)result;
                return Status.OK;
            }
            else if (result is Status)
                return (Status)result;
            else
                return Status.MANAGE_ERROR;
        }

        public static unsafe Status GrdGetErrorMessage(Status error,  GrdLanguageId lang, out string message)
        {
            if (error == Status.FUNCTION_NOT_FOUND)
            {
                message = "Function not found";
                return Status.OK;
            }
            if (error == Status.DLL_NOT_FOUND)
            {
                message = "Library not found";
                return Status.OK;
            }
            if (error == Status.MANAGE_ERROR)
            {
                message = "Internal error";
                return Status.OK;
            }

            if (GrdlicApi.GrdGetErrorMessageInvoker == null)
            {
                Type[] parameters = { typeof(Int32), typeof(Int32),  typeof(IntPtr), typeof(Int32)};

                GrdlicApi.GrdGetErrorMessageInvoker = new PlatformInvoker("GrdGetErrorMessage", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            message = "Unknown error";

            byte[] m = new byte[1024];
            Status st;
            fixed (byte* pM = m)
            {
                st = (Status)GrdGetErrorMessageInvoker.Invoke((Int32)error, (Int32)lang, new IntPtr(pM), 1024);
                if (st == Status.OK)
                    message = GetStringFromByteArray(pM);
            }

            return st;
        }

        public static unsafe Status GrdGetLicenseInfo(string visibility, VendorCodes codes, out string licenses)
        {
            if (GrdlicApi.GrdGetLicenseInfoInvoker == null)
            {
                Type[] parameters = {typeof(IntPtr), typeof(IntPtr), typeof(byte**) };

                GrdlicApi.GrdGetLicenseInfoInvoker = new PlatformInvoker("GrdGetLicenseInfo", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            licenses = null;

            IntPtr cds = IntPtr.Zero;
            GrdVendorCodes rawVendorCodes;
            if (codes != null)
            {
                rawVendorCodes = getRawVendorCodes(codes);
                cds = new IntPtr(&rawVendorCodes);
            }

            byte[] visBuf = new byte[0];
            if (visibility != null)
                visBuf = Encoding.UTF8.GetBytes(visibility);
            byte* pLicenses;
            Status result;
            fixed(byte* vis = visBuf)
                result = (Status)GrdGetLicenseInfoInvoker.Invoke(new IntPtr(vis), cds, new IntPtr(&pLicenses));
            if (result != Status.OK)
                return result;

            licenses = GetStringFromByteArray(pLicenses);

            // free raw data
            GrdFree(new IntPtr(pLicenses));
            return Status.OK;
        }

        public static unsafe Status GrdVerifyDigest(byte[] publicKey, byte[] data, byte[] signResult)
        {
            if (GrdlicApi.GrdVerifyDigestInvoker == null)
            {
                Type[] parameters = { typeof(Int32), typeof(IntPtr), typeof(Int32), typeof(IntPtr), typeof(Int32), typeof(IntPtr) };

                GrdlicApi.GrdVerifyDigestInvoker = new PlatformInvoker("GrdVerifyDigest", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            if (publicKey == null)
                publicKey = new byte[0];
            if (data == null)
                data = new byte[0];
            if (signResult == null)
                signResult = new byte[0];

            fixed (byte* pPublicKey = publicKey)
            fixed (byte* pData = data)
            fixed (byte* pSignResult = signResult)
                return (Status)GrdVerifyDigestInvoker.Invoke(publicKey.Length, new IntPtr(pPublicKey),
                                                                  data.Length, new IntPtr(pData),
                                                                  signResult.Length, new IntPtr(pSignResult));
        }

        public static unsafe Status GrdLedBlink(UInt32 dongleId)
        {
            if (GrdlicApi.GrdLedBlinkInvoker == null)
            {
                Type[] parameters = { typeof(UInt32) };

                GrdlicApi.GrdLedBlinkInvoker = new PlatformInvoker("GrdLedBlink", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            return (Status)GrdLedBlinkInvoker.Invoke(dongleId);
        }

        public static unsafe Status GrdSetDriverType(VendorCodes codes, UInt32 dongleId, UInt32 driverType)
        {
            if (GrdlicApi.GrdSetDriverTypeInvoker == null)
            {
                Type[] parameters = { typeof(IntPtr), typeof(UInt32), typeof(UInt32) };

                GrdlicApi.GrdSetDriverTypeInvoker = new PlatformInvoker("GrdSetDriverType", GrdDllName,
                typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            if (codes == null)
                return Status.INVALID_PARAMETER;

            GrdVendorCodes rawVendorCodes = getRawVendorCodes(codes);
            return (Status)GrdSetDriverTypeInvoker.Invoke(new IntPtr(&rawVendorCodes), dongleId, driverType);
        }

        public static unsafe Status InstallLicense(byte[] response)
        {
            if (GrdlicApi.GrdLicenseInstallInvoker == null)
            {
                Type[] parameters = { typeof(IntPtr), typeof(Int32) };

                GrdlicApi.GrdLicenseInstallInvoker = new PlatformInvoker("GrdLicenseInstall", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            if (response == null)
                response = new byte[0];

            fixed (byte* pResponse = response)
                return (Status)GrdLicenseInstallInvoker.Invoke(new IntPtr(pResponse), response.Length);
        }

        private static unsafe void GrdFree(IntPtr anyObjRef)
        {
            if (anyObjRef == null)
                return;

            if (GrdlicApi.GrdFreeInvoker == null)
            {
                Type[] parameters = { typeof(IntPtr) };

                GrdlicApi.GrdFreeInvoker = new PlatformInvoker("GrdFree", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            GrdFreeInvoker.Invoke(anyObjRef);
        }

        public static unsafe Status GrdLicenseDetach(string visibility, GrdProductFeaturePair[] featuresList,
                                                     UInt32 durationInSec, UInt16 concurrentResourceToConsume,
                                                     string recipientHardwareFingerpint,
                                                     out byte[] recipientLicense)
        {
            if (GrdlicApi.GrdLicenseDetachInvoker == null)
            {
                Type[] parameters = { typeof(IntPtr), typeof(IntPtr), typeof(IntPtr), typeof(UInt32),
                                     typeof(UInt32), typeof(UInt16), typeof(IntPtr), typeof(IntPtr), typeof(IntPtr)};

                GrdlicApi.GrdLicenseDetachInvoker = new PlatformInvoker("GrdLicenseDetach", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            recipientLicense = null;

            if (featuresList == null || featuresList.Length == 0)
                return Status.INVALID_PARAMETER;

            byte[] visBuf = new byte[0];
            if (visibility != null)
                visBuf = Encoding.UTF8.GetBytes(visibility);
            byte* pRecipientLicense;
            UInt32 recipientLicenseSize = 0;
            byte[] fingerPrintBuf = Encoding.UTF8.GetBytes(recipientHardwareFingerpint);

            GrdProductFeaturePairInternal[] featuresListInternal = new GrdProductFeaturePairInternal[featuresList.Length];
            for (int i = 0; i < featuresList.Length; ++i)
                featuresListInternal[i] = new GrdProductFeaturePairInternal(featuresList[i].productNumber, featuresList[i].featureNumber);

            Status r;
            fixed(byte* vis = visBuf)
            fixed (byte* pFingerPrint = fingerPrintBuf)
            fixed (GrdProductFeaturePairInternal* ptr = &featuresListInternal[0])
                r = (Status)GrdLicenseDetachInvoker.Invoke(new IntPtr(vis),
                                            new IntPtr(ptr), (UInt32)featuresList.Length,
                                            durationInSec, concurrentResourceToConsume,
                                            new IntPtr(pFingerPrint),
                                            new IntPtr(&pRecipientLicense), new IntPtr(&recipientLicenseSize));

            if (r != Status.OK)
                return r;

            // copy data to output
            recipientLicense = new byte[recipientLicenseSize];
            Marshal.Copy(new IntPtr(pRecipientLicense), recipientLicense, 0, recipientLicense.Length);
            // free raw data
            GrdFree(new IntPtr(pRecipientLicense));
            return Status.OK;
        }

        public static unsafe Status GrdLicenseReturnToPool(UInt32 licenseId, out byte[] licenseToReturn)
        {
            if (GrdlicApi.GrdLicenseReturnToPoolInvoker == null)
            {
                Type[] parameters = {typeof(UInt32), typeof(IntPtr), typeof(IntPtr) };

                GrdlicApi.GrdLicenseReturnToPoolInvoker = new PlatformInvoker("GrdLicenseReturnToPool", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            licenseToReturn = null;

            Status r;
            byte* pLicenseToReturn;
            UInt32 licenseToReturnSize = 0;
            r = (Status)GrdLicenseReturnToPoolInvoker.Invoke(licenseId, new IntPtr(&pLicenseToReturn), new IntPtr(&licenseToReturnSize));
            if (r != Status.OK)
                return r;

            // copy data to output
            licenseToReturn = new byte[licenseToReturnSize];
            Marshal.Copy(new IntPtr(pLicenseToReturn), licenseToReturn, 0, licenseToReturn.Length);
            // free raw data
            GrdFree(new IntPtr(pLicenseToReturn));
            return Status.OK;
        }

        public static unsafe Status GrdGetHostFingerprint(out string hostFingerprintJson, string visibility)
        {
            if (GrdlicApi.GrdGetHostFingerprintInvoker == null)
            {
                Type[] parameters = { typeof(IntPtr), typeof(IntPtr) };

                GrdlicApi.GrdGetHostFingerprintInvoker = new PlatformInvoker("GrdGetHostFingerprint", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            hostFingerprintJson = null;

            byte[] visibilityBuf = null;
            if (visibility != null)
                visibilityBuf = Encoding.UTF8.GetBytes(visibility);

            Status r;
            byte* pHostFingerprintJson;
            fixed (byte* pVisibility = visibilityBuf)
                r = (Status)GrdGetHostFingerprintInvoker.Invoke(new IntPtr(&pHostFingerprintJson), new IntPtr(pVisibility));
            if (r != Status.OK)
                return r;


            hostFingerprintJson = GetStringFromByteArray(pHostFingerprintJson);

            // free raw data
            GrdFree(new IntPtr(pHostFingerprintJson));
            return Status.OK;
        }

        public static unsafe Status GrdGetSessionInfo(string visibilityJson, VendorCodes codes, out string sessionInfoJsonArray)
        {
            if (GrdlicApi.GrdGetLicenseInfoInvoker == null)
            {
                Type[] parameters = {typeof(IntPtr), typeof(IntPtr), typeof(byte**) };

                GrdlicApi.GrdGetSessionInfoInvoker = new PlatformInvoker("GrdGetSessionInfo", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            sessionInfoJsonArray = null;

            IntPtr cds = IntPtr.Zero;
            GrdVendorCodes rawVendorCodes;
            if (codes != null)
            {
                rawVendorCodes = getRawVendorCodes(codes);
                cds = new IntPtr(&rawVendorCodes);
            }

            byte[] visBuf = new byte[0];
            if (visibilityJson != null)
                visBuf = Encoding.UTF8.GetBytes(visibilityJson);
            byte* pSessionInfo;
            Status result;
            fixed(byte* vis = visBuf)
                result = (Status)GrdGetSessionInfoInvoker.Invoke(new IntPtr(vis), cds, new IntPtr(&pSessionInfo));
            if (result != Status.OK)
                return result;

            sessionInfoJsonArray = GetStringFromByteArray(pSessionInfo);

            // free raw data
            GrdFree(new IntPtr(pSessionInfo));
            return Status.OK;
        }

        public static unsafe Status GrdGetHostInfo(string visibilityJson, out string hostInfoJsonArray)
        {
            if (GrdlicApi.GrdGetLicenseInfoInvoker == null)
            {
                Type[] parameters = {typeof(IntPtr), typeof(byte**) };

                GrdlicApi.GrdGetHostInfoInvoker = new PlatformInvoker("GrdGetHostInfo", GrdDllName,
                    typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
            }

            hostInfoJsonArray = null;

            byte[] visBuf = new byte[0];
            if (visibilityJson != null)
                visBuf = Encoding.UTF8.GetBytes(visibilityJson);
            byte* pHostsInfo;
            Status result;
            fixed(byte* vis = visBuf)
                result = (Status)GrdGetHostInfoInvoker.Invoke(new IntPtr(vis), new IntPtr(&pHostsInfo));
            if (result != Status.OK)
                return result;

            hostInfoJsonArray = GetStringFromByteArray(pHostsInfo);

            // free raw data
            GrdFree(new IntPtr(pHostsInfo));
            return Status.OK;
        }

        public class Feature
        {
            private UInt32 m_number;
            private UInt32 m_handle = (UInt32)HandleValue.INVALID;

            public Feature(UInt32 number)
            {
                SetApiPlatform();
                m_number = number;
            }

            ~Feature()
            {
                if (m_handle != (UInt32)HandleValue.INVALID)
                    Logout();
            }

            public unsafe Status Login(VendorCodes codes, string visibility)
            {
                if (GrdlicApi.GrdFeatureLoginInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr), typeof(IntPtr), typeof(UInt32*) };

                    GrdlicApi.GrdFeatureLoginInvoker = new PlatformInvoker("GrdFeatureLogin", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (codes == null)
                    return Status.INVALID_PARAMETER;

                GrdVendorCodes rawVendorCodes = getRawVendorCodes(codes);
                IntPtr cds = new IntPtr(&rawVendorCodes);

                byte[] visBuf = new byte[0];
                if (visibility != null)
                    visBuf = Encoding.UTF8.GetBytes(visibility);

                fixed(byte* vis = visBuf)
                fixed (UInt32* pHandle = &m_handle)
                {
                    Status result = (Status)GrdFeatureLoginInvoker.Invoke(m_number, cds, new IntPtr(vis), new IntPtr(pHandle));
                    if (result != Status.OK)
                        return result;
                }

                return Status.OK;
            }

            public unsafe Status Logout()
            {
                if (GrdlicApi.GrdFeatureLogoutInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32) };

                    GrdlicApi.GrdFeatureLogoutInvoker = new PlatformInvoker("GrdFeatureLogout", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                Status r = (Status)GrdFeatureLogoutInvoker.Invoke(m_handle);
                m_handle = (UInt32)HandleValue.INVALID;
                return r;
            }

            public unsafe Status Check(byte[] publicKey)
            {
                if (GrdlicApi.GrdFeatureCheckInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(Int32), typeof(IntPtr)};

                    GrdlicApi.GrdFeatureCheckInvoker = new PlatformInvoker("GrdFeatureCheck", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (publicKey == null)
                    publicKey = new byte[0];

                fixed (byte* pData = publicKey)
                return (Status)GrdFeatureCheckInvoker.Invoke(m_handle, publicKey.Length, new IntPtr(pData));
            }

            public unsafe Status GetInfo(out string licenseInfo)
            {
                if (GrdlicApi.GrdFeatureGetInfoInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(byte**) };

                    GrdlicApi.GrdFeatureGetInfoInvoker = new PlatformInvoker("GrdFeatureGetInfo", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                licenseInfo = null;

                byte* pLicenseInfo;
                Status result = (Status)GrdFeatureGetInfoInvoker.Invoke(m_handle, new IntPtr(&pLicenseInfo));
                if (result != Status.OK)
                    return result;

                // copy data to output
                licenseInfo = GetStringFromByteArray(pLicenseInfo);

                // free raw data
                GrdFree(new IntPtr(pLicenseInfo));
                return Status.OK;
            }

            public unsafe Status Encrypt(byte[] data, FeatureEncryptMode mode, GrdAesContext context)
            {
                if (GrdlicApi.GrdFeatureEncryptInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(Int32), typeof(IntPtr), typeof(Int32), typeof(IntPtr), typeof(Int32) };

                    GrdlicApi.GrdFeatureEncryptInvoker = new PlatformInvoker("GrdFeatureEncrypt", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (data == null)
                    data = new byte[0];

                IntPtr pContext = IntPtr.Zero;
                GrdAesContextInternal ctx;
                if (context != null)
                {
                    ctx = new GrdAesContextInternal();
                    Marshal.Copy(context.iv, 0, new IntPtr(ctx.iv), context.iv.Length);
                    Marshal.Copy(context.inf, 0, new IntPtr(ctx.inf), context.inf.Length);
                    pContext = new IntPtr(&ctx);
                }

                Status st;
                fixed (byte* pData = data)
                    st = (Status)GrdFeatureEncryptInvoker.Invoke(m_handle, data.Length, new IntPtr(pData), mode, pContext, Marshal.SizeOf(ctx));

                if (st == Status.OK && context != null)
                {
                    Marshal.Copy(new IntPtr(ctx.iv), context.iv, 0, context.iv.Length);
                    Marshal.Copy(new IntPtr(ctx.inf), context.inf, 0, context.inf.Length);
                }

                return st;
            }

            public unsafe Status Decrypt(byte[] data, FeatureEncryptMode mode, GrdAesContext context)
            {
                if (GrdlicApi.GrdFeatureDecryptInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(Int32), typeof(IntPtr), typeof(Int32), typeof(IntPtr), typeof(Int32) };

                    GrdlicApi.GrdFeatureDecryptInvoker = new PlatformInvoker("GrdFeatureDecrypt", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (data == null)
                    data = new byte[0];

                IntPtr pContext = IntPtr.Zero;
                GrdAesContextInternal ctx;
                if (context != null)
                {
                    ctx = new GrdAesContextInternal();
                    Marshal.Copy(context.iv, 0, new IntPtr(ctx.iv), context.iv.Length);
                    Marshal.Copy(context.inf, 0, new IntPtr(ctx.inf), context.inf.Length);
                    pContext = new IntPtr(&ctx);
                }

                Status st;
                fixed (byte* pData = data)
                    st = (Status)GrdFeatureDecryptInvoker.Invoke(m_handle, data.Length, new IntPtr(pData), mode, pContext, Marshal.SizeOf(ctx));

                if (st == Status.OK && context != null)
                {
                    Marshal.Copy(new IntPtr(ctx.iv), context.iv, 0, context.iv.Length);
                    Marshal.Copy(new IntPtr(ctx.inf), context.inf, 0, context.inf.Length);
                }

                return st;
            }

            public unsafe Status CryptAndCount(byte[] data, FeatureEncryptMode mode, GrdAesContext context, bool doDecryption, UInt32 n, UInt32 flags)
            {
                if (GrdlicApi.GrdFeatureCryptAndCountInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(Int32), typeof(IntPtr), typeof(Int32), typeof(IntPtr), typeof(Int32), typeof(UInt32), typeof(UInt32), typeof(UInt32) };

                    GrdlicApi.GrdFeatureCryptAndCountInvoker = new PlatformInvoker("GrdFeatureCryptAndCount", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (data == null)
                    data = new byte[0];

                IntPtr pContext = IntPtr.Zero;
                GrdAesContextInternal ctx;
                if (context != null)
                {
                    ctx = new GrdAesContextInternal();
                    Marshal.Copy(context.iv, 0, new IntPtr(ctx.iv), context.iv.Length);
                    Marshal.Copy(context.inf, 0, new IntPtr(ctx.inf), context.inf.Length);
                    pContext = new IntPtr(&ctx);
                }

                Status st;
                UInt32 decr = 0;
                if (doDecryption)
                    decr = 1;
                fixed (byte* pData = data)
                    st = (Status)GrdFeatureCryptAndCountInvoker.Invoke(m_handle, data.Length, new IntPtr(pData), mode, pContext, Marshal.SizeOf(ctx), decr, n, flags);

                if (st == Status.OK && context != null)
                {
                    Marshal.Copy(new IntPtr(ctx.iv), context.iv, 0, context.iv.Length);
                    Marshal.Copy(new IntPtr(ctx.inf), context.inf, 0, context.inf.Length);
                }

                return st;
            }

            public unsafe Status Sign(byte[] data, out byte[] signResult)
            {
                if (GrdlicApi.GrdFeatureSignInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(Int32), typeof(IntPtr), typeof(Int32), typeof(IntPtr)};

                    GrdlicApi.GrdFeatureSignInvoker = new PlatformInvoker("GrdFeatureSign", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (data == null)
                    data = new byte[0];

                signResult = new byte[(int)GrdECC160.DIGEST_SIZE];

                fixed (byte* pData = data)
                fixed (byte* pSignResult = signResult)
                    return (Status)GrdFeatureSignInvoker.Invoke(m_handle, data.Length, new IntPtr(pData), signResult.Length, new IntPtr(pSignResult));
            }

            public unsafe Status GetTimeLimit(out long remainingTime)
            {
                if (GrdlicApi.GrdFeatureGetTimeLimitInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr) };

                    GrdlicApi.GrdFeatureGetTimeLimitInvoker = new PlatformInvoker("GrdFeatureGetTimeLimit", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                remainingTime = 0;
                fixed (long* pRemainingTime = &remainingTime)
                    return (Status)GrdFeatureGetTimeLimitInvoker.Invoke(m_handle, new IntPtr(pRemainingTime));
            }

            public unsafe Status GetRealTime(out long realTime)
            {
                if (GrdlicApi.GrdFeatureGetRealTimeInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr) };

                    GrdlicApi.GrdFeatureGetRealTimeInvoker = new PlatformInvoker("GrdGetRealTime", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                realTime = 0;
                fixed (long* pRealTime = &realTime)
                    return (Status)GrdFeatureGetRealTimeInvoker.Invoke(m_handle, new IntPtr(pRealTime));
            }

            public unsafe Status GetMaxConcurrentResource(out UInt32 value)
            {
                if (GrdlicApi.GrdFeatureGetMaxConcurrentResourceInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr) };

                    GrdlicApi.GrdFeatureGetMaxConcurrentResourceInvoker = new PlatformInvoker("GrdFeatureGetMaxConcurrentResource", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                value = 0;
                fixed (UInt32* pValue = &value)
                    return (Status)GrdFeatureGetMaxConcurrentResourceInvoker.Invoke(m_handle, new IntPtr(pValue));
            }

            public unsafe Status GetRunCounter(out UInt32 value)
            {
                if (GrdlicApi.GrdFeatureGetRunCounterInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr) };

                    GrdlicApi.GrdFeatureGetRunCounterInvoker = new PlatformInvoker("GrdFeatureGetRunCounter", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                value = 0;
                fixed (UInt32* pValue = &value)
                    return (Status)GrdFeatureGetRunCounterInvoker.Invoke(m_handle, new IntPtr(pValue));
            }

            public unsafe Status MemoryGetSize(UInt32 memoryId, out long size)
            {
                if (GrdlicApi.GrdMemoryGetSizeInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(UInt32), typeof(IntPtr) };

                    GrdlicApi.GrdMemoryGetSizeInvoker = new PlatformInvoker("GrdMemoryGetSize", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                size = 0;
                fixed (long* pSize = &size)
                    return (Status)GrdMemoryGetSizeInvoker.Invoke(m_handle, memoryId, new IntPtr(pSize));
            }

            public unsafe Status MemoryRead(UInt32 memoryId, UInt32 password, UInt32 offset, byte[] data)
            {
                if (GrdlicApi.GrdMemoryReadInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(UInt32), typeof(UInt32), typeof(Int32), typeof(UInt32), typeof(IntPtr) };

                    GrdlicApi.GrdMemoryReadInvoker = new PlatformInvoker("GrdMemoryRead", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (data == null)
                    data = new byte[0];

                fixed (byte* pData = data)
                    return (Status)GrdMemoryReadInvoker.Invoke(m_handle, memoryId, password, data.Length, offset, new IntPtr(pData));
            }

            public unsafe Status MemoryWrite(UInt32 memoryId, UInt32 password, GrdMemoryWriteMode mode, UInt32 offset, byte[] data)
            {
                if (GrdlicApi.GrdMemoryWriteInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(UInt32), typeof(UInt32), typeof(Int32), typeof(Int32), typeof(UInt32), typeof(IntPtr) };

                    GrdlicApi.GrdMemoryWriteInvoker = new PlatformInvoker("GrdMemoryWrite", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

            if (data == null)
                data = new byte[0];

                fixed (byte* pData = data)
                    return (Status)GrdMemoryWriteInvoker.Invoke(m_handle, memoryId, password, mode, data.Length, offset, new IntPtr(pData));
            }
        }

        public class LicenseActivator
        {
            string m_host;
            byte[] m_hostBuf;
            UInt32 m_port;

            public LicenseActivator(string host, int port)
            {
                m_host = host;
                m_hostBuf = Encoding.UTF8.GetBytes(m_host);
                m_port = (UInt32)port;
            }

            private unsafe CustomerInfoRaw GetRawCustomerInfo(CustomerInfo customerInfo)
            {
                CustomerInfoRaw ci = new CustomerInfoRaw();
                if (customerInfo != null)
                {
                    byte[] tmp = Encoding.UTF8.GetBytes(customerInfo.m_firstName);
                    int len = Math.Min(63, tmp.Length);
                    Marshal.Copy(tmp, 0, new IntPtr(ci.firstName), len);

                    tmp = Encoding.UTF8.GetBytes(customerInfo.m_lastName);
                    len = Math.Min(63, tmp.Length);
                    Marshal.Copy(tmp, 0, new IntPtr(ci.lastName), len);

                    tmp = Encoding.UTF8.GetBytes(customerInfo.m_email);
                    len = Math.Min(127, tmp.Length);
                    Marshal.Copy(tmp, 0, new IntPtr(ci.email), len);

                    tmp = Encoding.UTF8.GetBytes(customerInfo.m_companyName);
                    len = Math.Min(127, tmp.Length);
                    Marshal.Copy(tmp, 0, new IntPtr(ci.companyName), len);

                    tmp = Encoding.UTF8.GetBytes(customerInfo.m_phone);
                    len = Math.Min(31, tmp.Length);
                    Marshal.Copy(tmp, 0, new IntPtr(ci.phone), len);
                }

                return ci;
            }

            public unsafe Status CheckSerialNumberFormat(string serialNumber)
            {
                if (GrdlicApi.GrdLicenseCheckSerialNumberFormatInvoker == null)
                {
                    Type[] parameters = { typeof(IntPtr) };

                    GrdlicApi.GrdLicenseCheckSerialNumberFormatInvoker = new PlatformInvoker("GrdLicenseCheckSerialNumberFormat", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (serialNumber == null)
                    return Status.INVALID_PARAMETER;

                byte[] serialBuf = Encoding.UTF8.GetBytes(serialNumber);
                fixed (byte* pSerial = serialBuf)
                    return (Status)GrdLicenseCheckSerialNumberFormatInvoker.Invoke(new IntPtr(pSerial));
            }

            public unsafe Status Activate(string serialNumber, CustomerInfo customerInfo, out UInt32 outLicenseId)
            {
                if (GrdlicApi.GrdLicenseActivateInvoker == null)
                {
                    Type[] parameters = { typeof(IntPtr), typeof(IntPtr), typeof(UInt32), typeof(IntPtr), typeof(IntPtr) };

                    GrdlicApi.GrdLicenseActivateInvoker = new PlatformInvoker("GrdLicenseActivate", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                outLicenseId = 0;

                if (serialNumber == null)
                    return Status.INVALID_PARAMETER;

                CustomerInfoRaw customerInfoRaw = GetRawCustomerInfo(customerInfo);

                byte[] serialBuf = Encoding.UTF8.GetBytes(serialNumber);

                fixed (byte* pSerial = serialBuf)
                fixed (byte* pHost = m_hostBuf)
                fixed (UInt32* pOutLicenseId = &outLicenseId)
                    return (Status)GrdLicenseActivateInvoker.Invoke(new IntPtr(pSerial), new IntPtr(pHost), m_port, new IntPtr(&customerInfoRaw), new IntPtr(pOutLicenseId));
            }

            public unsafe Status Update(UInt32 licenseId)
            {
                if (GrdlicApi.GrdLicenseUpdateInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr), typeof(UInt32) };

                    GrdlicApi.GrdLicenseUpdateInvoker = new PlatformInvoker("GrdLicenseUpdate", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                fixed (byte* pHost = m_hostBuf)
                    return (Status)GrdLicenseUpdateInvoker.Invoke(licenseId, new IntPtr(pHost), m_port);
            }

            public unsafe Status CheckUpdateIsAvailable(UInt32 licenseId)
            {
                if (GrdlicApi.GrdLicenseCheckUpdateIsAvailableInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr), typeof(UInt32) };

                    GrdlicApi.GrdLicenseCheckUpdateIsAvailableInvoker = new PlatformInvoker("GrdLicenseCheckUpdateIsAvailable", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                fixed (byte* pHost = m_hostBuf)
                    return (Status)GrdLicenseCheckUpdateIsAvailableInvoker.Invoke(licenseId, new IntPtr(pHost), m_port);
            }

            public unsafe Status CheckIsNotBanned(UInt32 licenseId)
            {
                if (GrdlicApi.GrdLicenseCheckIsNotBannedInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr), typeof(UInt32) };

                    GrdlicApi.GrdLicenseCheckIsNotBannedInvoker = new PlatformInvoker("GrdLicenseCheckIsNotBanned", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                fixed (byte* pHost = m_hostBuf)
                    return (Status)GrdLicenseCheckIsNotBannedInvoker.Invoke(licenseId, new IntPtr(pHost), m_port);
            }

            public unsafe Status Remove(UInt32 licenseId)
            {
                if (GrdlicApi.GrdLicenseRemoveInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32)};

                    GrdlicApi.GrdLicenseRemoveInvoker = new PlatformInvoker("GrdLicenseRemove", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                return (Status)GrdLicenseRemoveInvoker.Invoke(licenseId);
            }

            public unsafe Status CreateActivationRequest(out byte[] outRequest)
            {
                if (GrdlicApi.GrdLicenseCreateActivationRequestInvoker == null)
                {
                    Type[] parameters = { typeof(IntPtr), typeof(IntPtr) };

                    GrdlicApi.GrdLicenseCreateActivationRequestInvoker = new PlatformInvoker("GrdLicenseCreateActivationRequest", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                outRequest = null;

                Status r = Status.OK;
                byte* pOutRequest;
                UInt32 outRequestSize = 0;
                r = (Status)GrdLicenseCreateActivationRequestInvoker.Invoke(new IntPtr(&pOutRequest), new IntPtr(&outRequestSize));
                if (r != Status.OK)
                    return r;

                // copy data to output
                outRequest = new byte[outRequestSize];
                Marshal.Copy(new IntPtr(pOutRequest), outRequest, 0, outRequest.Length);

                // free raw data
                GrdFree(new IntPtr(pOutRequest));
                return Status.OK;
            }

            public unsafe Status CreateUpdateRequest(UInt32 licenseId, out byte[] outRequest)
            {
                if (GrdlicApi.GrdLicenseCreateUpdateRequestInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr), typeof(IntPtr) };

                    GrdlicApi.GrdLicenseCreateUpdateRequestInvoker = new PlatformInvoker("GrdLicenseCreateUpdateRequest", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                outRequest = null;

                byte* pOutRequest;
                UInt32 outRequestSize = 0;
                Status r = (Status)GrdLicenseCreateUpdateRequestInvoker.Invoke(licenseId, new IntPtr(&pOutRequest), new IntPtr(&outRequestSize));
                if (r != Status.OK)
                    return r;

                // copy data to output
                outRequest = new byte[outRequestSize];
                Marshal.Copy(new IntPtr(pOutRequest), outRequest, 0, outRequest.Length);

                // free raw data
                GrdFree(new IntPtr(pOutRequest));
                return Status.OK;
            }

            public unsafe Status SendActivationRequest(string serialNumber, CustomerInfo customerInfo, byte[] activationRequest, out byte[] outResponse)
            {
                if (GrdlicApi.GrdLicenseSendActivationRequestInvoker == null)
                {
                    Type[] parameters = { typeof(IntPtr), typeof(IntPtr), typeof(IntPtr), typeof(Int32), typeof(IntPtr), typeof(UInt32), typeof(IntPtr), typeof(IntPtr) };

                    GrdlicApi.GrdLicenseSendActivationRequestInvoker = new PlatformInvoker("GrdLicenseSendActivationRequest", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                outResponse = null;

                if (activationRequest == null)
                    activationRequest = new byte[0];

                if (serialNumber == null)
                    return Status.INVALID_PARAMETER;

                CustomerInfoRaw customerInfoRaw = GetRawCustomerInfo(customerInfo);

                byte[] serialBuf = Encoding.UTF8.GetBytes(serialNumber);

                Status r = Status.OK;
                byte* pOutResponse;
                UInt32 outResponseSize = 0;
                fixed (byte* pSerial = serialBuf)
                fixed (byte* pActivationRequest = activationRequest)
                fixed (byte* pHost = m_hostBuf)
                    r = (Status)GrdLicenseSendActivationRequestInvoker.Invoke(new IntPtr(pSerial), new IntPtr(&customerInfoRaw),
                                                                              new IntPtr(pActivationRequest), activationRequest.Length,
                                                                              new IntPtr(pHost), m_port,
                                                                              new IntPtr(&pOutResponse), new IntPtr(&outResponseSize));
                if (r != Status.OK)
                    return r;

                // copy data to output
                outResponse = new byte[outResponseSize];
                Marshal.Copy(new IntPtr(pOutResponse), outResponse, 0, outResponse.Length);

                // free raw data
                GrdFree(new IntPtr(pOutResponse));
                return Status.OK;
            }

            public unsafe Status SendUpdateRequest(byte[] updateRequest, out byte[] outResponse)
            {
                if (GrdlicApi.GrdLicenseSendUpdateRequestInvoker == null)
                {
                    Type[] parameters = { typeof(IntPtr), typeof(Int32), typeof(IntPtr), typeof(UInt32), typeof(IntPtr), typeof(IntPtr) };

                    GrdlicApi.GrdLicenseSendUpdateRequestInvoker = new PlatformInvoker("GrdLicenseSendUpdateRequest", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                outResponse = null;

                if (updateRequest == null)
                    updateRequest = new byte[0];

                Status r = Status.OK;
                byte* pOutResponse;
                UInt32 outResponseSize = 0;
                fixed (byte* pUpdateRequest = updateRequest)
                fixed (byte* pHost = m_hostBuf)
                    r = (Status)GrdLicenseSendUpdateRequestInvoker.Invoke(new IntPtr(pUpdateRequest), updateRequest.Length,
                                                                              new IntPtr(pHost), m_port,
                                                                              new IntPtr(&pOutResponse), new IntPtr(&outResponseSize));
                if (r != Status.OK)
                    return r;

                // copy data to output
                outResponse = new byte[outResponseSize];
                Marshal.Copy(new IntPtr(pOutResponse), outResponse, 0, outResponse.Length);

                // free raw data
                GrdFree(new IntPtr(pOutResponse));
                return Status.OK;
            }

            public unsafe Status LicenseExtractForRehost(UInt32 licenseId, out byte[] outDonorLicense)
            {
                if (GrdlicApi.GrdLicenseExtractForRehostInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr), typeof(IntPtr) };

                    GrdlicApi.GrdLicenseExtractForRehostInvoker = new PlatformInvoker("GrdLicenseExtractForRehost", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                outDonorLicense = null;

                Status r = Status.OK;
                byte* pOutResponse;
                UInt32 outResponseSize = 0;
                r = (Status)GrdLicenseExtractForRehostInvoker.Invoke(licenseId, new IntPtr(&pOutResponse), new IntPtr(&outResponseSize));
                if (r != Status.OK)
                    return r;

                // copy data to output
                outDonorLicense = new byte[outResponseSize];
                Marshal.Copy(new IntPtr(pOutResponse), outDonorLicense, 0, outDonorLicense.Length);

                // free raw data
                GrdFree(new IntPtr(pOutResponse));
                return Status.OK;
            }

            public unsafe Status LicenseRehost(byte[] donorLicense)
            {
                if (GrdlicApi.GrdLicenseRehostInvoker == null)
                {
                    Type[] parameters = { typeof(IntPtr), typeof(UInt32), typeof(IntPtr), typeof(Int32) };

                    GrdlicApi.GrdLicenseRehostInvoker = new PlatformInvoker("GrdLicenseRehost", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (donorLicense == null)
                    donorLicense = new byte[0];

                Status r = Status.OK;
                fixed (byte* pDonorLicense = donorLicense)
                fixed (byte* pHost = m_hostBuf)
                    r = (Status)GrdLicenseRehostInvoker.Invoke(new IntPtr(pHost), m_port, new IntPtr(pDonorLicense), donorLicense.Length);
                return r;
            }

            public unsafe Status SendRehostRequest(byte[] donorLicense, string recipientHardwareFingerpint, out byte[] recipientLicense)
            {
                if (GrdlicApi.GrdLicenseSendRehostRequestInvoker == null)
                {
                    Type[] parameters = { typeof(IntPtr), typeof(UInt32),
                                          typeof(IntPtr), typeof(Int32),
                                          typeof(IntPtr), typeof(Int32),
                                          typeof(IntPtr), typeof(IntPtr)};

                    GrdlicApi.GrdLicenseSendRehostRequestInvoker = new PlatformInvoker("GrdLicenseSendRehostRequest", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                recipientLicense = null;
                if (donorLicense == null)
                    donorLicense = new byte[0];

                byte[] fingerPrintBuf = Encoding.UTF8.GetBytes(recipientHardwareFingerpint);

                Status r = Status.OK;
                byte* pRecipientLicense;
                UInt32 recipientLicenseSize = 0;
                fixed (byte* pDonorLicense = donorLicense)
                fixed (byte* pRecipientHardwareFingerpint = fingerPrintBuf)
                fixed (byte* pHost = m_hostBuf)
                    r = (Status)GrdLicenseSendRehostRequestInvoker.Invoke(new IntPtr(pHost), m_port,
                                                                          new IntPtr(pDonorLicense), donorLicense.Length,
                                                                          new IntPtr(pRecipientHardwareFingerpint), recipientHardwareFingerpint.Length,
                                                                          new IntPtr(&pRecipientLicense), new IntPtr(&recipientLicenseSize));
                if (r != Status.OK)
                    return r;

                // copy data to output
                recipientLicense = new byte[recipientLicenseSize];
                Marshal.Copy(new IntPtr(pRecipientLicense), recipientLicense, 0, recipientLicense.Length);

                // free raw data
                GrdFree(new IntPtr(pRecipientLicense));
                return Status.OK;
            }

            public unsafe Status DongleSetLicense(UInt32 hardwareDongleId, string serialNumber, UInt32 driverType)
            {
                if (GrdlicApi.GrdDongleSetLicenseInvoker == null)
                {
                    Type[] parameters = { typeof(UInt32), typeof(IntPtr), typeof(UInt32), typeof(IntPtr), typeof(UInt32) };

                    GrdlicApi.GrdDongleSetLicenseInvoker = new PlatformInvoker("GrdDongleSetLicense", GrdDllName,
                        typeof(Status), parameters, CallingConvention.Winapi, CharSet.Auto);
                }

                if (serialNumber == null)
                    return Status.INVALID_PARAMETER;

                byte[] serialBuf = Encoding.UTF8.GetBytes(serialNumber);

                fixed (byte* pSerial = serialBuf)
                fixed (byte* pHost = m_hostBuf)
                    return (Status)GrdDongleSetLicenseInvoker.Invoke(hardwareDongleId, new IntPtr(pSerial), driverType, new IntPtr(pHost), m_port);
            }
        }
    }
}
