package grdlic;

public class Feature
{
    private int mFeatureNumber;
    private GrdInt mHandle = new GrdInt();

    public Feature(int featureNumber)
    {
        mFeatureNumber = featureNumber;
    }

    public GrdStatus Login(GrdVendorCodes codes, String visibility)
    {
        try
        {
            return new GrdStatus(api.FeatureLogin(mFeatureNumber, codes, visibility, mHandle));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Logout()
    {
        try
        {
            return new GrdStatus(api.FeatureLogout(mHandle));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Check(byte[] publicKey)
    {
        try
        {
            return new GrdStatus(api.FeatureCheck(mHandle, (publicKey != null)?publicKey.length:0, publicKey));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus GetInfo(GrdString outLicenseInfoJson)
    {
        try
        {
            return new GrdStatus(api.FeatureGetInfo(mHandle, outLicenseInfoJson));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Encrypt(byte[] data, int mode, GrdAesContext context)
    {
        try
        {
            return new GrdStatus(api.FeatureEncrypt(mHandle, data.length, data, mode, context));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Decrypt(byte[] data, int mode, GrdAesContext context)
    {
        try
        {
            return new GrdStatus(api.FeatureDecrypt(mHandle, data.length, data, mode, context));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Sign(byte[] message, byte[] digest)
    {
        try
        {
            return new GrdStatus(api.FeatureSign(mHandle, message.length, message, digest.length, digest));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus GetTimeLimit(GrdInt remainingTime)
    {
        try
        {
            return new GrdStatus(api.FeatureGetTimeLimit(mHandle, remainingTime));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus GetRealTime(GrdInt realTime)
    {
        try
        {
            return new GrdStatus(api.RealTime(mHandle, realTime));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus GetMaxConcurrentResource(GrdInt maxResource)
    {
        try
        {
            return new GrdStatus(api.FeatureGetMaxConcurrentResource(mHandle, maxResource));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus GetRunCounter(GrdInt runCounter)
    {
        try
        {
            return new GrdStatus(api.FeatureGetRunCounter(mHandle, runCounter));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus MemoryGetSize(int memoryId, GrdInt size)
    {
        try
        {
            return new GrdStatus(api.GrdMemoryGetSize(mHandle, memoryId, size));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus MemoryRead(int memoryId, int password, int offset, byte[] data)
    {
        try
        {
            return new GrdStatus(api.GrdMemoryRead(mHandle, memoryId,password, data.length, offset, data));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus MemoryWrite(int memoryId, int password, int mode, int offset, byte[] data)
    {
        try
        {
            return new GrdStatus(api.GrdMemoryWrite(mHandle, memoryId, password, mode, data.length, offset, data));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }
}
