package grdlic;

public class LicenseActivator
{
    private String mHost;
    private int mPort;

    public LicenseActivator(String host, int port)
    {
        mHost = host;
        mPort = port;
    }

    public GrdStatus CheckSerialNumberFormat(String serialNumber)
    {
        try
        {
            return new GrdStatus(api.LicenseCheckSerialNumberFormat(serialNumber));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Activate(String serialNumber, GrdCustomerInfo customerInfo, GrdInt outLicenseId)
    {
        try
        {
            return new GrdStatus(api.LicenseActivate(serialNumber, mHost, mPort, customerInfo, outLicenseId));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Update(int licenseId)
    {
        try
        {
            return new GrdStatus(api.LicenseUpdate(licenseId, mHost, mPort));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus CheckUpdateIsAvailable(int licenseId)
    {
        try
        {
            return new GrdStatus(api.LicenseCheckUpdateIsAvailable(licenseId, mHost, mPort));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Remove(int licenseId)
    {
        try
        {
            return new GrdStatus(api.LicenseRemove(licenseId));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus CheckIsNotBanned(int licenseId)
    {
        try
        {
            return new GrdStatus(api.LicenseCheckIsNotBanned(licenseId, mHost, mPort));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus CreateActivationRequest(GrdByteArray outRequest)
    {
        try
        {
            return new GrdStatus(api.LicenseCreateActivationRequest(outRequest));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus SendActivationRequest(String serialNumber, GrdCustomerInfo customerInfo, byte[] request, GrdByteArray outResponse)
    {
        try
        {
            return new GrdStatus(api.LicenseSendActivationRequest(serialNumber, customerInfo, request, request.length, mHost, mPort, outResponse));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Install(byte[] response)
    {
        try
        {
            return new GrdStatus(api.LicenseInstall(response, response.length));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus CreateUpdateRequest(int licenseId, GrdByteArray outRequest)
    {
        try
        {
            return new GrdStatus(api.LicenseCreateUpdateRequest(licenseId, outRequest));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus SendUpdateRequest(byte[] request, GrdByteArray outResponse)
    {
        try
        {
            return new GrdStatus(api.LicenseSendUpdateRequest(request, request.length, mHost, mPort, outResponse));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus ExtractForRehost(int licenseId, GrdByteArray outDonorLicense)
    {
        try
        {
            return new GrdStatus(api.LicenseExtractForRehost(licenseId, outDonorLicense));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus Rehost(byte[] donorLicense)
    {
        try
        {
            return new GrdStatus(api.LicenseRehost(mHost, mPort, donorLicense, donorLicense.length));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public GrdStatus SendRehostRequest(byte[] donorLicense, String recipientHardwareFingerpint, GrdByteArray recipientLicense)
    {
        try
        {
            return new GrdStatus(api.LicenseSendRehostRequest(mHost, mPort, donorLicense, donorLicense.length,
                                                                            recipientHardwareFingerpint,
                                                                            recipientLicense));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }
}

