package grdlic;


/**
*  Grdlic Java API – is function set for licensing and protection Java applications
*
*  Before using Grdlic Java API in application code you must load grdlic_java library:
*  x86/grdlic_java.dll or x86_64/grdlic_java.dll for windows;
*  x86/libgrdlic_java.so or x86_64/libgrdlic_java.so for linux;
*  x86/libgrdlic_java.dylib or x86_64/libgrdlic_java.dylib for macos.
*/

/// Feature search option (visibility) example (in JSON format). All fields are optional
/*
   {
    // Perform feature search only inside of specified license. By default, the feature search is performed in all licenses
    "licenseId": 0x12345678,
    // The mask that specifying what kind of features can be found - local (0x01), remote (0x02) or both (0x03). See GrdRemoteMode enumeration.
    // By default, the search is performed for all features
    "remoteMode": 3,
    // The mask for specifying the dongle models to which feature can be bound to (0 means all models). See GrdDongleModel enumeration.
    // By default, the feature search is performed in all dongle models
    "dongleModel": 0x400,
    // The number of the product to which feature bound to (0 means all products). By default, the feature search is performed in all products
    "productNumber": 777,
    // Number of network resource to consume. By default, 1
    "networkResourceToConsume" : 1,
    // Configuration for Guardant Control Center
    "controlCenter":
    {
        // HostName or IP address of the computer on which Guardant Control Center is running
       "hostName": [ "192.168.77.77" ],
       // Broadcast search for remote licenses (enabled by default)
       "broadcastSearch" : true,
       // Ipv4 broadcast address (255.255.255.255 by default)
       "broadcastAddress": "255.255.255.255"
    }
   }
*/

public class api
{
    private api() {}

    public static native int GetApiVersion(GrdInt majorVersion, GrdInt minorVersion);
    public static int GrdGetApiVersion(GrdInt majorVersion, GrdInt minorVersion)
    {
        try
        {
            return GetApiVersion(majorVersion, minorVersion);
        }
        catch (UnsatisfiedLinkError e)
        {
            return -1;
        }
    }

    public static native int GetErrorMessage(int error, int lang, GrdString message);
    public static GrdStatus GrdGetErrorMessage(GrdStatus error, int lang, GrdString message)
    {
        if (error.equals(GrdStatus.LIBRARY_NOT_FOUND))
        {
            message = new GrdString("Library not found");
            return GrdStatus.OK;
        }

        try
        {
            return new GrdStatus(GetErrorMessage(error.getValue(), lang, message));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public static native int GetLicenseInfo(String visibility, GrdVendorCodes codes, GrdString licenseInfoJsonArray);
    public static GrdStatus GrdGetLicenseInfo(String visibility, GrdVendorCodes codes, GrdString licenseInfoJsonArray)
    {
        try
        {
            return new GrdStatus(GetLicenseInfo(visibility, codes, licenseInfoJsonArray));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public static native int VerifyDigest(int publicKeySize, byte[] publicKey,
                                        int messageSize, byte[] message,
                                        int digestSize, byte[] digest);

    public static GrdStatus GrdVerifyDigest(byte[] publicKey, byte[] message, byte[] digest)
    {
        try
        {
            return new GrdStatus(VerifyDigest(publicKey.length, publicKey, message.length, message, digest.length, digest));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public static native int GetSessionInfo(String visibility, GrdVendorCodes codes, GrdString sessionInfoJsonArray);
    public static GrdStatus GrdGetSessionInfo(String visibility, GrdVendorCodes codes, GrdString sessionInfoJsonArray)
    {
        try
        {
            return new GrdStatus(GetSessionInfo(visibility, codes, sessionInfoJsonArray));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public static native int GetHostInfo(String visibility, GrdString hostInfoJsonArray);
    public static GrdStatus GrdGetHostInfo(String visibility, GrdVendorCodes codes, GrdString hostInfoJsonArray)
    {
        try
        {
            return new GrdStatus(GetHostInfo(visibility, hostInfoJsonArray));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    // feature functions
    public static native int FeatureLogin(int featureNumber, GrdVendorCodes codes, String visibility, GrdInt handle);
    public static native int FeatureLogout(GrdInt handle);
    public static native int FeatureCheck(GrdInt handle, int publicKeySize, byte[] publicKey);
    public static native int FeatureGetInfo(GrdInt handle, GrdString outLicenseInfoJson);
    public static native int FeatureEncrypt(GrdInt handle, int dataSize, byte[] data, int mode, GrdAesContext context);
    public static native int FeatureDecrypt(GrdInt handle, int dataSize, byte[] data, int mode, GrdAesContext context);
    public static native int FeatureCryptAndCount(GrdInt handle, int dataSize, byte[] data, int mode, GrdAesContext context, int doDecryption, int n, int flags);
    public static native int FeatureSign(GrdInt handle,
                                        int messageSize, byte[] message,
                                        int digestSize, byte[] digest);
    public static native int FeatureGetTimeLimit(GrdInt handle, GrdInt remainingTime);
    public static native int RealTime(GrdInt handle, GrdInt realTime);
    public static native int FeatureGetMaxConcurrentResource(GrdInt handle, GrdInt maxResource);
    public static native int FeatureGetRunCounter(GrdInt handle, GrdInt runCounter);
    public static native int GrdMemoryGetSize(GrdInt handle, int memoryId, GrdInt size);
    public static native int GrdMemoryRead(GrdInt handle, int memoryId, int password, int dataSize, int offset, byte[] data);
    public static native int GrdMemoryWrite(GrdInt handle, int memoryId, int password, int mode, int dataSize, int offset, byte[] data);

    // for hardware dongles only
    public static native int LedBlink(int dongleId);
    public static GrdStatus GrdLedBlink(int dongleId)
    {
        try
        {
            return new GrdStatus(LedBlink(dongleId));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    public static native int SetDriverType(GrdVendorCodes codes, int dongleId, int driverType);
    public static GrdStatus GrdSetDriverType(GrdVendorCodes codes, int dongleId, int driverType)
    {
        try
        {
            return new GrdStatus(SetDriverType(codes, dongleId, driverType));
        }
        catch (UnsatisfiedLinkError e)
        {
            return GrdStatus.LIBRARY_NOT_FOUND;
        }
    }

    // license activation and update functions
    public static native int LicenseCheckSerialNumberFormat(String serialNumber);
    public static native int LicenseActivate(String serialNumber, String host, int port, GrdCustomerInfo customerInfo, GrdInt outLicenseId);
    public static native int LicenseUpdate(int licenseId, String host, int port);
    public static native int LicenseCheckUpdateIsAvailable(int licenseId, String host, int port);
    public static native int LicenseRemove(int licenseId);
    public static native int LicenseCheckIsNotBanned(int licenseId, String host, int port);
    public static native int LicenseCreateActivationRequest(GrdByteArray outRequest);
    public static native int LicenseSendActivationRequest(String serialNumber, GrdCustomerInfo customerInfo,
                                                byte[] request, int requestSize,
                                                String host, int port,
                                                GrdByteArray outResponse);
    public static native int LicenseInstall(byte[] response, int responseSize);
    public static native int LicenseCreateUpdateRequest(int licenseId, GrdByteArray outRequest);
    public static native int LicenseSendUpdateRequest(byte[] request, int requestSize,
                                            String host, int port,
                                            GrdByteArray outResponse);
    public static native int LicenseExtractForRehost(int licenseId, GrdByteArray outDonorLicense);
    public static native int LicenseRehost(String host, int port, byte[] donorLicense, int donorLicenseSize);
    public static native int LicenseSendRehostRequest(String host, int port,
                                                      byte[] donorLicense, int donorLicenseSize,
                                                      String recipientHardwareFingerpint,
                                                      GrdByteArray recipientLicense);
    public static native int LicenseDetach(String visibility, GrdVendorCodes codes,
                               GrdProductFeaturePair[] featuresList,
                               int durationInSec, int concurrentResourceToConsume,
                               String recipientHardwareFingerpint,
                               GrdByteArray recipientLicense);
    public static native int LicenseReturnToPool(int licenseId, GrdByteArray licenseToReturn);
    public static native int GetHostFingerprint(GrdString fingerprint, String host, int port);
    public static native int DongleSetLicense(int hardwareDongleId, String serialNumber, int driverType, String host, int port);
}
