#include "platformdef.h"
#include <stdlib.h>
#include <string.h>
#include <grdlic/grdlic.h>
#include <map>
#include <assert.h>


#ifdef WINDOWS_APP
#include "windows.h"
#elif defined(LINUX_APP)
#include <limits.h>
#include <unistd.h>
#include <dlfcn.h>
#endif

#include "grdlic_api.h"
#include "jniFunction.h"

#ifdef LINUX_APP
// full path for library
Dl_info dl_info;
__attribute__((constructor)) void on_load(void)
{
    dladdr((void*)on_load, &dl_info);
}
#endif // LINUX_APP

#define SIZE_OF_MEMBER(cls, member) sizeof(((cls*)0)->member)

static inline void SetBufferForArray(JNIEnv* env, const void* pData, jbyteArray dataArray)
{
    if (pData == NULL || dataArray == NULL)
        return;

    jsize length = env->GetArrayLength(dataArray);
    if (length <= 0)
        return;

    env->SetByteArrayRegion(dataArray, 0, length, (jbyte*)pData);
}

static inline void SetBufferForArray(JNIEnv* env, const void* pData, jshortArray dataArray)
{
    if (pData == NULL || dataArray == NULL)
        return;

    jsize length = env->GetArrayLength(dataArray);
    if (length <= 0)
        return;

    env->SetShortArrayRegion(dataArray, 0, length, (jshort*)pData);
}

static inline void SetBufferForArray(JNIEnv* env, const void* pData, jintArray dataArray)
{
    if (pData == NULL || dataArray == NULL)
        return;

    jsize length = env->GetArrayLength(dataArray);
    if (length <= 0)
        return;

    env->SetIntArrayRegion(dataArray, 0, length, (jint*)pData);
}

static inline void SetBufferForArray(JNIEnv* env, const void* pData, jlongArray dataArray)
{
    if (pData == NULL || dataArray == NULL)
        return;

    jsize length = env->GetArrayLength(dataArray);
    if (length <= 0)
        return;

    env->SetLongArrayRegion(dataArray, 0, length, (jlong*)pData);
}

static inline void SetBufferForArray(JNIEnv* env, const void* pData, jdoubleArray dataArray)
{
    if (pData == NULL || dataArray == NULL)
        return;

    jsize length = env->GetArrayLength(dataArray);
    if (length <= 0)
        return;

    env->SetDoubleArrayRegion(dataArray, 0, length, (jdouble*)pData);
}

static inline void* GetBufferForArray(JNIEnv* env, const jbyteArray dataArray, jsize& dataSize)
{
    dataSize = 0;
    if (dataArray == NULL)
        return NULL;

    jsize length = env->GetArrayLength(dataArray);
    if (length <= 0)
        return NULL;

    dataSize = length * sizeof(jbyte);
    void* pData = malloc(dataSize);
    env->GetByteArrayRegion(dataArray, 0, length, (jbyte*)pData);
    return pData;
}

static inline void* GetBufferForArray(JNIEnv* env, const jshortArray dataArray, jsize& dataSize)
{
    dataSize = 0;
    if (dataArray == NULL)
        return NULL;

    jsize length = env->GetArrayLength(dataArray);
    if (length <= 0)
        return NULL;

    dataSize = length * sizeof(jshort);
    void* pData = malloc(dataSize);
    env->GetShortArrayRegion(dataArray, 0, length, (jshort*)pData);
    return pData;
}

static inline void* GetBufferForArray(JNIEnv* env, const jintArray dataArray, jsize& dataSize)
{
    dataSize = 0;
    if (dataArray == NULL)
        return NULL;

    jsize length = env->GetArrayLength(dataArray);
    if (length <= 0)
        return NULL;

    dataSize = length * sizeof(jint);
    void* pData = malloc(dataSize);
    env->GetIntArrayRegion(dataArray, 0, length, (jint*)pData);
    return pData;
}

static inline void* GetBufferForArray(JNIEnv* env, const jlongArray data, jsize& dataSize)
{
    dataSize = 0;
    if (data == NULL)
        return NULL;

    jsize length = env->GetArrayLength(data);
    if (length <= 0)
        return NULL;

    dataSize = length * sizeof(jlong);
    void* pData = malloc(dataSize);
    env->GetLongArrayRegion(data, 0, length, (jlong*)pData);
    return pData;
}

static inline void* GetBufferForArray(JNIEnv* env, const jdoubleArray data, jsize& dataSize)
{
    dataSize = 0;
    if (data == NULL)
        return NULL;

    jsize length = env->GetArrayLength(data);
    if (length <= 0)
        return NULL;

    dataSize = length * sizeof(jdouble);
    void* pData = malloc(dataSize);
    env->GetDoubleArrayRegion(data, 0, length, (jdouble*)pData);
    return pData;
}

static inline void FreeBufferForArray(void* p)
{
    if (p != NULL)
        free(p);
}

/*
 * Class:     grdlic_api
 * Method:    GetApiVersion
 * Signature: (Lgrdlic/GrdInt;Lgrdlic/GrdInt;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_GetApiVersion
    (JNIEnv* env, jclass, jobject majorVersion, jobject minorVersion)
{
    grd_uint32 majVersion = 0;
    grd_uint32 minVersion = 0;
    jint ret = (jint)GrdGetApiVersion(&majVersion, &minVersion);

    SetIntField(env, majorVersion, "mValue", (jint)majVersion);
    SetIntField(env, minorVersion, "mValue", (jint)minVersion);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    GetErrorMessage
 * Signature: (IILgrdlic/GrdString;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_GetErrorMessage
    (JNIEnv* env, jclass, jint error, jint lang, jobject msg)
{
    char m[1024];
    jint ret = GrdGetErrorMessage(error, lang, m, 1024);
    if (ret == GRD_OK)
        SetObjectField(env, msg, "mValue", "Ljava/lang/String;", env->NewStringUTF(m));
    return ret;
}

void GetVendorCodes(JNIEnv* env, jobject joCodes, GrdVendorCodes& codes)
{
    memset(&codes, 0, sizeof(GrdVendorCodes));
    GetIntField(env, joCodes, "publicCode", (jint*)&codes.publicCode);
    GetIntField(env, joCodes, "privateReadCode", (jint*)&codes.privateReadCode);
    GetIntField(env, joCodes, "privateWriteCode", (jint*)&codes.privateWriteCode);
}

/*
 * Class:     grdlic_api
 * Method:    GetLicenseInfo
 * Signature: (Ljava/lang/String;Lgrdlic/GrdVendorCodes;Lgrdlic/GrdString;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_GetLicenseInfo
    (JNIEnv* env, jclass, jstring visibility, jobject codes, jobject outJson)
{
    GrdVendorCodes vCodes;
    GrdVendorCodes* vCodesPtr = NULL;
    if (codes != NULL)
    {
        vCodesPtr = &vCodes;
        GetVendorCodes(env, codes, vCodes);
    }

    jboolean isCopy;
    char* vis = (char*)env->GetStringUTFChars(visibility, &isCopy);
    char* licenseInfoJson = NULL;
    jint ret = GrdGetLicenseInfo(vis, vCodesPtr, &licenseInfoJson);
    if (!isCopy)
        env->ReleaseStringUTFChars(visibility, vis);
    if (ret == GRD_OK)
        SetObjectField(env, outJson, "mValue", "Ljava/lang/String;", env->NewStringUTF(licenseInfoJson));

    if (licenseInfoJson)
        GrdFree(licenseInfoJson);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    VerifyDigest
 * Signature: (I[BI[BI[B)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_VerifyDigest
    (JNIEnv* env, jclass, jint publicKeySize, jbyteArray publicKey, jint messageSize, jbyteArray message, jint digestSize, jbyteArray digest)
{
    if (!publicKeySize || !publicKey || !messageSize || !message || !digestSize || !digest)
        return GRD_INVALID_PARAMETER;

    jsize sz = env->GetArrayLength(publicKey);
    if (sz != (jsize)publicKeySize)
        return GRD_INVALID_PARAMETER;

    void* pub = GetBufferForArray(env, publicKey, sz);

    sz = env->GetArrayLength(message);
    if (sz != (jsize)messageSize)
        return GRD_INVALID_PARAMETER;

    void* mes = GetBufferForArray(env, message, sz);

    sz = env->GetArrayLength(digest);
    if (sz != (jsize)digestSize)
        return GRD_INVALID_PARAMETER;

    void* dig = GetBufferForArray(env, digest, sz);

    jint ret = GrdVerifyDigest((grd_uint32)publicKeySize, pub, (grd_uint32)messageSize, mes, (grd_uint32)digestSize, dig);
    FreeBufferForArray(mes);
    FreeBufferForArray(dig);
    FreeBufferForArray(pub);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureLogin
 * Signature: (ILgrdlic/GrdVendorCodes;Ljava/lang/String;Lgrdlic/GrdInt;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureLogin
    (JNIEnv* env, jclass, jint featureNumber, jobject codes, jstring visibility, jobject handle)
{
    GrdVendorCodes vCodes;
    GrdVendorCodes* vCodesPtr = NULL;
    if (codes != NULL)
    {
        vCodesPtr = &vCodes;
        GetVendorCodes(env, codes, vCodes);
    }

    GrdHandle h = GRD_INVALID_HANDLE_VALUE;
    jboolean isCopy;
    char* vis = (char*)env->GetStringUTFChars(visibility, &isCopy);
    jint ret = GrdFeatureLogin((grd_uint32)featureNumber, vCodesPtr, vis, &h);
    if (!isCopy)
        env->ReleaseStringUTFChars(visibility, vis);
    if (ret == GRD_OK)
        SetIntField(env, handle, "mValue", (jint)h);

    return ret;
}

GrdHandle GetHandleValue(JNIEnv* env, jobject handle)
{
    jint h = (jint)GRD_INVALID_HANDLE_VALUE;
    GetIntField(env, handle, "mValue", &h);
    return (GrdHandle)h;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureLogout
 * Signature: (Lgrdlic/GrdInt;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureLogout
    (JNIEnv* env, jclass, jobject handle)
{
    return (jint)GrdFeatureLogout(GetHandleValue(env, handle));
}

/*
 * Class:     grdlic_api
 * Method:    FeatureCheck
 * Signature: (Lgrdlic/GrdInt;I[B)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureCheck
    (JNIEnv* env, jclass, jobject handle, jint publicKeySize, jbyteArray publicKey)
{
    jsize sz = 0;
    if (publicKey != NULL)
        sz = env->GetArrayLength(publicKey);
    if (publicKeySize != (jint)sz)
        return GRD_INVALID_PARAMETER;

    void* pub = GetBufferForArray(env, publicKey, sz);
    jint ret = GrdFeatureCheck(GetHandleValue(env, handle), (grd_uint32)publicKeySize, pub);
    FreeBufferForArray(pub);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureGetInfo
 * Signature: (Lgrdlic/GrdInt;Lgrdlic/GrdString;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureGetInfo
    (JNIEnv* env, jclass, jobject handle, jobject outJson)
{
    char* json = NULL;
    jint ret = GrdFeatureGetInfo(GetHandleValue(env, handle), &json);

    if (ret == GRD_OK)
        SetObjectField(env, outJson, "mValue", "Ljava/lang/String;", env->NewStringUTF(json));

    if (json)
        GrdFree(json);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureEncrypt
 * Signature: (Lgrdlic/GrdInt;I[BILgrdlic/GrdAesContext;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureEncrypt
    (JNIEnv* env, jclass, jobject handle, jint dataSize, jbyteArray data, jint mode, jobject context)
{
    if (!data || !dataSize)
        return GRD_INVALID_PARAMETER;

    jsize sz = env->GetArrayLength(data);
    if (sz != (jsize)dataSize)
        return GRD_INVALID_PARAMETER;

    void* buf = GetBufferForArray(env, data, sz);


    void* ctx = NULL;
    grd_uint32 ctxSize = 0;
    GrdAesContext ctxStruct = {0};
    if (context != NULL)
    {

        jobject iv, inf;
        if (GetObjectField(env, context, "iv", "[B", &iv)
            && GetObjectField(env, context, "inf", "[B", &inf))
        {
            jboolean isCopy;
            jbyte* ivTmp = env->GetByteArrayElements((jbyteArray)iv, &isCopy);
            if (!ivTmp)
                return GRD_INVALID_PARAMETER;

            memcpy(ctxStruct.iv, ivTmp, SIZE_OF_MEMBER(GrdAesContext, iv));
            if (!isCopy)
                env->ReleaseByteArrayElements((jbyteArray)iv, ivTmp, JNI_ABORT);

            jbyte* infTmp = env->GetByteArrayElements((jbyteArray)inf, &isCopy);
            if (!infTmp)
                return GRD_INVALID_PARAMETER;

            memcpy(ctxStruct.inf, infTmp, SIZE_OF_MEMBER(GrdAesContext, inf));
            if (!isCopy)
                env->ReleaseByteArrayElements((jbyteArray)inf, infTmp, JNI_ABORT);

            ctx = &ctxStruct;
        }
        else
            return GRD_INVALID_PARAMETER;
    }

    jint ret = GrdFeatureEncrypt(GetHandleValue(env, handle), (grd_uint32)dataSize, buf, (grd_uint32)mode, ctx, (grd_uint32)sizeof(GrdAesContext));
    if (ret == GRD_OK)
    {
        SetBufferForArray(env, buf, data);

        // set context
        if (context)
        {
            jboolean isCopy;
            jbyte* buf;

            jbyteArray ivArr = env->NewByteArray((jsize)SIZE_OF_MEMBER(GrdAesContext, iv));
            buf = env->GetByteArrayElements(ivArr, &isCopy);
            memcpy(buf, ctxStruct.iv, (size_t)SIZE_OF_MEMBER(GrdAesContext, iv));
            SetObjectField(env, context, "iv", "[B", ivArr);
            env->ReleaseByteArrayElements(ivArr, buf, 0);

            jbyteArray infArr = env->NewByteArray((jsize)SIZE_OF_MEMBER(GrdAesContext, inf));
            buf = env->GetByteArrayElements(infArr, &isCopy);
            memcpy(buf, ctxStruct.inf, (size_t)SIZE_OF_MEMBER(GrdAesContext, inf));
            SetObjectField(env, context, "inf", "[B", infArr);
            env->ReleaseByteArrayElements(infArr, buf, 0);
        }
    }

    FreeBufferForArray(buf);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureDecrypt
 * Signature: (Lgrdlic/GrdInt;I[BILgrdlic/GrdAesContext;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureDecrypt
    (JNIEnv* env, jclass, jobject handle, jint dataSize, jbyteArray data, jint mode, jobject context)
{
    if (!data || !dataSize)
        return GRD_INVALID_PARAMETER;

    jsize sz = env->GetArrayLength(data);
    if (sz != (jsize)dataSize)
        return GRD_INVALID_PARAMETER;

    void* buf = GetBufferForArray(env, data, sz);


    void* ctx = NULL;
    grd_uint32 ctxSize = 0;
    GrdAesContext ctxStruct = {0};
    if (context != NULL)
    {

        jobject iv, inf;
        if (GetObjectField(env, context, "iv", "[B", &iv)
            && GetObjectField(env, context, "inf", "[B", &inf))
        {
            jboolean isCopy;
            jbyte* ivTmp = env->GetByteArrayElements((jbyteArray)iv, &isCopy);
            if (!ivTmp)
                return GRD_INVALID_PARAMETER;

            memcpy(ctxStruct.iv, ivTmp, SIZE_OF_MEMBER(GrdAesContext, iv));
            if (!isCopy)
                env->ReleaseByteArrayElements((jbyteArray)iv, ivTmp, JNI_ABORT);

            jbyte* infTmp = env->GetByteArrayElements((jbyteArray)inf, &isCopy);
            if (!infTmp)
                return GRD_INVALID_PARAMETER;

            memcpy(ctxStruct.inf, infTmp, SIZE_OF_MEMBER(GrdAesContext, inf));
            if (!isCopy)
                env->ReleaseByteArrayElements((jbyteArray)inf, infTmp, JNI_ABORT);

            ctx = &ctxStruct;
        }
        else
            return GRD_INVALID_PARAMETER;
    }

    jint ret = GrdFeatureDecrypt(GetHandleValue(env, handle), (grd_uint32)dataSize, buf, (grd_uint32)mode, ctx, (grd_uint32)sizeof(GrdAesContext));
    if (ret == GRD_OK)
    {
        SetBufferForArray(env, buf, data);

        // set context
        if (context)
        {
            jboolean isCopy;
            jbyte* buf;

            jbyteArray ivArr = env->NewByteArray((jsize)SIZE_OF_MEMBER(GrdAesContext, iv));
            buf = env->GetByteArrayElements(ivArr, &isCopy);
            memcpy(buf, ctxStruct.iv, (size_t)SIZE_OF_MEMBER(GrdAesContext, iv));
            SetObjectField(env, context, "iv", "[B", ivArr);
            env->ReleaseByteArrayElements(ivArr, buf, 0);

            jbyteArray infArr = env->NewByteArray((jsize)SIZE_OF_MEMBER(GrdAesContext, inf));
            buf = env->GetByteArrayElements(infArr, &isCopy);
            memcpy(buf, ctxStruct.inf, (size_t)SIZE_OF_MEMBER(GrdAesContext, inf));
            SetObjectField(env, context, "inf", "[B", infArr);
            env->ReleaseByteArrayElements(infArr, buf, 0);
        }
    }

    FreeBufferForArray(buf);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureDecrypt
 * Signature: (Lgrdlic/GrdInt;I[BILgrdlic/GrdAesContext;III)
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureCryptAndCount
    (JNIEnv* env, jclass, jobject handle, jint dataSize, jbyteArray data, jint mode, jobject context, jint doDecryption, jint n, jint flags)
{
    if (!data || !dataSize)
        return GRD_INVALID_PARAMETER;

    jsize sz = env->GetArrayLength(data);
    if (sz != (jsize)dataSize)
        return GRD_INVALID_PARAMETER;

    void* buf = GetBufferForArray(env, data, sz);


    void* ctx = NULL;
    grd_uint32 ctxSize = 0;
    GrdAesContext ctxStruct = {0};
    if (context != NULL)
    {

        jobject iv, inf;
        if (GetObjectField(env, context, "iv", "[B", &iv)
            && GetObjectField(env, context, "inf", "[B", &inf))
        {
            jboolean isCopy;
            jbyte* ivTmp = env->GetByteArrayElements((jbyteArray)iv, &isCopy);
            if (!ivTmp)
                return GRD_INVALID_PARAMETER;

            memcpy(ctxStruct.iv, ivTmp, SIZE_OF_MEMBER(GrdAesContext, iv));
            if (!isCopy)
                env->ReleaseByteArrayElements((jbyteArray)iv, ivTmp, JNI_ABORT);

            jbyte* infTmp = env->GetByteArrayElements((jbyteArray)inf, &isCopy);
            if (!infTmp)
                return GRD_INVALID_PARAMETER;

            memcpy(ctxStruct.inf, infTmp, SIZE_OF_MEMBER(GrdAesContext, inf));
            if (!isCopy)
                env->ReleaseByteArrayElements((jbyteArray)inf, infTmp, JNI_ABORT);

            ctx = &ctxStruct;
        }
        else
            return GRD_INVALID_PARAMETER;
    }

    jint ret = GrdFeatureCryptAndCount(GetHandleValue(env, handle), (grd_uint32)dataSize, buf, (grd_uint32)mode, ctx, (grd_uint32)sizeof(GrdAesContext),
                                       (grd_uint32)doDecryption, (grd_uint32)n, (grd_uint32)flags);
    if (ret == GRD_OK)
    {
        SetBufferForArray(env, buf, data);

        // set context
        if (context)
        {
            jboolean isCopy;
            jbyte* buf;

            jbyteArray ivArr = env->NewByteArray((jsize)SIZE_OF_MEMBER(GrdAesContext, iv));
            buf = env->GetByteArrayElements(ivArr, &isCopy);
            memcpy(buf, ctxStruct.iv, (size_t)SIZE_OF_MEMBER(GrdAesContext, iv));
            SetObjectField(env, context, "iv", "[B", ivArr);
            env->ReleaseByteArrayElements(ivArr, buf, 0);

            jbyteArray infArr = env->NewByteArray((jsize)SIZE_OF_MEMBER(GrdAesContext, inf));
            buf = env->GetByteArrayElements(infArr, &isCopy);
            memcpy(buf, ctxStruct.inf, (size_t)SIZE_OF_MEMBER(GrdAesContext, inf));
            SetObjectField(env, context, "inf", "[B", infArr);
            env->ReleaseByteArrayElements(infArr, buf, 0);
        }
    }

    FreeBufferForArray(buf);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureSign
 * Signature: (Lgrdlic/GrdInt;I[BI[B)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureSign
    (JNIEnv* env, jclass, jobject handle, jint messageSize, jbyteArray message, jint digestSize, jbyteArray digest)
{
    if (!messageSize || !message || !digestSize || !digest)
        return GRD_INVALID_PARAMETER;

    jsize sz = env->GetArrayLength(message);
    if (sz != (jsize)messageSize)
        return GRD_INVALID_PARAMETER;

    void* mes = GetBufferForArray(env, message, sz);

    sz = env->GetArrayLength(digest);
    if (sz != (jsize)digestSize)
        return GRD_INVALID_PARAMETER;

    void* dig = GetBufferForArray(env, digest, sz);

    jint ret = GrdFeatureSign(GetHandleValue(env, handle), (grd_uint32)messageSize, mes, (grd_uint32)digestSize, dig);
    if (ret == GRD_OK)
        SetBufferForArray(env, dig, digest);

    FreeBufferForArray(mes);
    FreeBufferForArray(dig);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureGetTimeLimit
 * Signature: (Lgrdlic/GrdInt;Lgrdlic/GrdInt;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureGetTimeLimit
    (JNIEnv* env, jclass, jobject handle, jobject remainingTime)
{
    grd_time_t t = 0;
    jint ret = GrdFeatureGetTimeLimit(GetHandleValue(env, handle), &t);
    if (ret == GRD_OK)
        SetIntField(env, remainingTime, "mValue", (jint)t);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    RealTime
 * Signature: (Lgrdlic/GrdInt;Lgrdlic/GrdInt;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_RealTime
    (JNIEnv* env, jclass, jobject handle, jobject realTime)
{
    grd_time_t t = 0;
    jint ret = GrdGetRealTime(GetHandleValue(env, handle), &t);
    if (ret == GRD_OK)
        SetIntField(env, realTime, "mValue", (jint)t);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureGetMaxConcurrentResource
 * Signature: (Lgrdlic/GrdInt;Lgrdlic/GrdInt;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureGetMaxConcurrentResource
    (JNIEnv* env, jclass, jobject handle, jobject maxResource)
{
    grd_uint32 v = 0;
    jint ret = GrdFeatureGetMaxConcurrentResource(GetHandleValue(env, handle), &v);
    if (ret == GRD_OK)
        SetIntField(env, maxResource, "mValue", (jint)v);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    FeatureGetRunCounter
 * Signature: (Lgrdlic/GrdInt;Lgrdlic/GrdInt;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_FeatureGetRunCounter
    (JNIEnv* env, jclass, jobject handle, jobject runCounter)
{
    grd_uint32 v = 0;
    jint ret = GrdFeatureGetRunCounter(GetHandleValue(env, handle), &v);
    if (ret == GRD_OK)
        SetIntField(env, runCounter, "mValue", (jint)v);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    GrdMemoryGetSize
 * Signature: (Lgrdlic/GrdInt;ILgrdlic/GrdInt;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_GrdMemoryGetSize
    (JNIEnv* env, jclass, jobject handle, jint memoryId, jobject size)
{
    if (!size)
        return GRD_INVALID_PARAMETER;

    grd_uint32 sz = 0;
    jint ret = GrdMemoryGetSize(GetHandleValue(env, handle), (grd_uint32)memoryId, &sz);
    if (ret == GRD_OK)
        SetIntField(env, size, "mValue", (jint)sz);

    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    GrdMemoryRead
 * Signature: (Lgrdlic/GrdInt;IIII[B)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_GrdMemoryRead
    (JNIEnv* env, jclass, jobject handle, jint memoryId, jint password, jint dataSize, jint offset, jbyteArray data)
{
    if (!dataSize || !data)
        return GRD_INVALID_PARAMETER;

    jsize sz = env->GetArrayLength(data);
    if (sz != (jsize)dataSize)
        return GRD_INVALID_PARAMETER;

    void* d = GetBufferForArray(env, data, sz);
    jint ret = GrdMemoryRead(GetHandleValue(env, handle), (grd_uint32)memoryId, (grd_uint32)password, (grd_uint32)dataSize, (grd_uint32)offset, d);
    if (ret == GRD_OK)
        SetBufferForArray(env, d, data);

    FreeBufferForArray(d);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    GrdMemoryWrite
 * Signature: (Lgrdlic/GrdInt;IIIII[B)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_GrdMemoryWrite
    (JNIEnv* env, jclass, jobject handle, jint memoryId, jint password, jint mode, jint dataSize, jint offset, jbyteArray data)
{
    if (!dataSize || !data)
        return GRD_INVALID_PARAMETER;

    jsize sz = env->GetArrayLength(data);
    if (sz != (jsize)dataSize)
        return GRD_INVALID_PARAMETER;

    void* d = GetBufferForArray(env, data, sz);
    jint ret = GrdMemoryWrite(GetHandleValue(env, handle), (grd_uint32)memoryId, (grd_uint32)password, (grd_uint32)mode, (grd_uint32)dataSize, (grd_uint32)offset, d);
    FreeBufferForArray(d);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LedBlink
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LedBlink
    (JNIEnv*, jclass, jint dongleId)
{
    return GrdLedBlink((grd_int32)dongleId);
}

/*
 * Class:     grdlic_api
 * Method:    SetDriverType
 * Signature: (Lgrdlic/GrdVendorCodes;II)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_SetDriverType
    (JNIEnv* env, jclass, jobject codes, jint dongleId, jint driverType)
{
    GrdVendorCodes vCodes;
    GrdVendorCodes* vCodesPtr = NULL;
    if (codes != NULL)
    {
        vCodesPtr = &vCodes;
        GetVendorCodes(env, codes, vCodes);
    }

    return GrdSetDriverType(vCodesPtr, (grd_int32)dongleId, (grd_int32)driverType);
}

/*
 * Class:     grdlic_api
 * Method:    LicenseCheckSerialNumberFormat
 * Signature: (Ljava/lang/String;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseCheckSerialNumberFormat
    (JNIEnv* env, jclass, jstring serialNumber)
{
    jboolean isCopy;
    char* serial = (char*)env->GetStringUTFChars(serialNumber, &isCopy);
    jint ret = GrdLicenseCheckSerialNumberFormat(serial);
    if (!isCopy)
        env->ReleaseStringUTFChars(serialNumber, serial);
    return ret;
}

bool GetCustomerInfo(JNIEnv* env, GrdCustomerInfo& raw, jobject customerInfo)
{
    if (!customerInfo)
        return false;

    memset(&raw, 0, sizeof(GrdCustomerInfo));
    jobject joFirstName;
    if (GetObjectField(env, customerInfo, "mFirstName", "Ljava/lang/String;", &joFirstName))
    {
        jsize length = env->GetStringLength((jstring)joFirstName);
        if ((size_t)length > SIZE_OF_MEMBER(GrdCustomerInfo, firstName))
            return false;

        jboolean isCopy;
        char* firstNameArr = (char*)env->GetStringUTFChars((jstring)joFirstName, &isCopy);
        memcpy(raw.firstName, firstNameArr, (size_t)length);
        if (!isCopy)
            env->ReleaseStringUTFChars((jstring)joFirstName, firstNameArr);
    }

    jobject joLastName;
    if (GetObjectField(env, customerInfo, "mLastName", "Ljava/lang/String;", &joLastName))
    {
        jsize length = env->GetStringLength((jstring)joLastName);
        if ((size_t)length > SIZE_OF_MEMBER(GrdCustomerInfo, lastName))
            return false;

        jboolean isCopy;
        char* lastNameArr = (char*)env->GetStringUTFChars((jstring)joLastName, &isCopy);
        memcpy(raw.lastName, lastNameArr, (size_t)length);
        if (!isCopy)
            env->ReleaseStringUTFChars((jstring)joLastName, lastNameArr);
    }

    jobject joEmail;
    if (GetObjectField(env, customerInfo, "mEmail", "Ljava/lang/String;", &joEmail))
    {
        jsize length = env->GetStringLength((jstring)joEmail);
        if ((size_t)length > SIZE_OF_MEMBER(GrdCustomerInfo, email))
            return false;

        jboolean isCopy;
        char* emailArr = (char*)env->GetStringUTFChars((jstring)joEmail, &isCopy);
        memcpy(raw.email, emailArr, (size_t)length);
        if (!isCopy)
            env->ReleaseStringUTFChars((jstring)joEmail, emailArr);
    }

    jobject joCompanyName;
    if (GetObjectField(env, customerInfo, "mCompanyName", "Ljava/lang/String;", &joCompanyName))
    {
        jsize length = env->GetStringLength((jstring)joCompanyName);
        if ((size_t)length > SIZE_OF_MEMBER(GrdCustomerInfo, companyName))
            return false;

        jboolean isCopy;
        char* companyNameArr = (char*)env->GetStringUTFChars((jstring)joCompanyName, &isCopy);
        memcpy(raw.companyName, companyNameArr, (size_t)length);
        if (!isCopy)
            env->ReleaseStringUTFChars((jstring)joCompanyName, companyNameArr);
    }

    jobject joPhone;
    if (GetObjectField(env, customerInfo, "mPhone", "Ljava/lang/String;", &joPhone))
    {
        jsize length = env->GetStringLength((jstring)joPhone);
        if ((size_t)length > SIZE_OF_MEMBER(GrdCustomerInfo, phone))
            return false;

        jboolean isCopy;
        char* phoneArr = (char*)env->GetStringUTFChars((jstring)joPhone, &isCopy);
        memcpy(raw.phone, phoneArr, (size_t)length);
        if (!isCopy)
            env->ReleaseStringUTFChars((jstring)joPhone, phoneArr);
    }

    return true;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseActivate
 * Signature: (Ljava/lang/String;Ljava/lang/String;ILgrdlic/GrdCustomerInfo;Lgrdlic/GrdInt;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseActivate
    (JNIEnv* env, jclass, jstring serialNumber, jstring host, jint port, jobject customerInfo, jobject joLicenseId)
{
    GrdCustomerInfo cus;
    GrdCustomerInfo* cusPtr = NULL;
    if (customerInfo)
    {
        cusPtr = &cus;
        if (!GetCustomerInfo(env, cus, customerInfo))
            return GRD_INVALID_PARAMETER;
    }

    jboolean isCopy1;
    char* serialPtr = (char*)env->GetStringUTFChars(serialNumber, &isCopy1);
    jboolean isCopy2;
    char* hostPtr = (char*)env->GetStringUTFChars(host, &isCopy2);

    grd_uint32 licId = 0;
    jint ret = GrdLicenseActivate(serialPtr, hostPtr, (grd_uint32)port, cusPtr, &licId);
    if (ret == GRD_OK)
        SetIntField(env, joLicenseId, "mValue", (jint)licId);

    if (!isCopy1)
        env->ReleaseStringUTFChars(serialNumber, serialPtr);
    if (!isCopy2)
        env->ReleaseStringUTFChars(host, hostPtr);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseUpdate
 * Signature: (ILjava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseUpdate
    (JNIEnv* env, jclass, jint licenseId, jstring host, jint port)
{
    jboolean isCopy;
    char* hostPtr = (char*)env->GetStringUTFChars(host, &isCopy);
    jint ret = GrdLicenseUpdate((grd_uint32)licenseId, hostPtr, (grd_uint32)port);
    if (!isCopy)
        env->ReleaseStringUTFChars(host, hostPtr);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseCheckUpdateIsAvailable
 * Signature: (ILjava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseCheckUpdateIsAvailable
    (JNIEnv* env, jclass, jint licenseId, jstring host, jint port)
{
    jboolean isCopy;
    char* hostPtr = (char*)env->GetStringUTFChars(host, &isCopy);
    jint ret = GrdLicenseCheckUpdateIsAvailable((grd_uint32)licenseId, hostPtr, (grd_uint32)port);
    if (!isCopy)
        env->ReleaseStringUTFChars(host, hostPtr);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseRemove
 * Signature: (I)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseRemove
    (JNIEnv*, jclass, jint licenseId)
{
    return GrdLicenseRemove(licenseId);
}

/*
 * Class:     grdlic_api
 * Method:    LicenseCheckIsNotBanned
 * Signature: (ILjava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseCheckIsNotBanned
    (JNIEnv* env, jclass, jint licenseId, jstring host, jint port)
{
    jboolean isCopy;
    char* hostPtr = (char*)env->GetStringUTFChars(host, &isCopy);
    jint ret = GrdLicenseCheckIsNotBanned((grd_uint32)licenseId, hostPtr, (grd_uint32)port);
    if (!isCopy)
        env->ReleaseStringUTFChars(host, hostPtr);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseCreateActivationRequest
 * Signature: (Lgrdlic/GrdByteArray;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseCreateActivationRequest
    (JNIEnv* env, jclass, jobject request)
{
    void* r = NULL;
    grd_uint32 rSize = 0;
    jint ret = GrdLicenseCreateActivationRequest(&r, &rSize);
    if (ret == GRD_OK)
    {
        jbyteArray requestArray = env->NewByteArray((jsize)rSize);
        jboolean isCopy;
        jbyte* buf = env->GetByteArrayElements(requestArray, &isCopy);
        memcpy(buf, r, (size_t)rSize);
        SetObjectField(env, request, "mValue", "[B", requestArray);
        env->ReleaseByteArrayElements(requestArray, buf, 0);
        GrdFree(r);
    }

    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseSendActivationRequest
 * Signature: (Ljava/lang/String;Lgrdlic/GrdCustomerInfo;[BILjava/lang/String;ILgrdlic/GrdByteArray;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseSendActivationRequest
    (JNIEnv* env, jclass, jstring serialNumber, jobject customerInfo, jbyteArray request, jint requestSize, jstring host, jint port, jobject outResponse)
{
    jsize sz = env->GetArrayLength(request);
    if (sz != (jsize)requestSize)
        return GRD_INVALID_PARAMETER;

    GrdCustomerInfo cus;
    GrdCustomerInfo* cusPtr = NULL;
    if (customerInfo)
    {
        cusPtr = &cus;
        if (!GetCustomerInfo(env, cus, customerInfo))
            return GRD_INVALID_PARAMETER;
    }

    jboolean isCopy1;
    char* serialPtr = (char*)env->GetStringUTFChars(serialNumber, &isCopy1);
    jboolean isCopy2;
    char* hostPtr = (char*)env->GetStringUTFChars(host, &isCopy2);

    void* req = GetBufferForArray(env, request, sz);

    grd_uint32 rSize = 0;
    void* response = NULL;
    jint ret = GrdLicenseSendActivationRequest(serialPtr, cusPtr, req, (grd_uint32)requestSize, hostPtr, (grd_uint32)port, &response, &rSize);
    if (ret == GRD_OK)
    {
        jbyteArray responseArray = env->NewByteArray((jsize)rSize);
        jboolean isCopy;
        jbyte* buf = env->GetByteArrayElements(responseArray, &isCopy);
        memcpy(buf, response, (size_t)rSize);
        SetObjectField(env, outResponse, "mValue", "[B", responseArray);
        env->ReleaseByteArrayElements(responseArray, buf, 0);
        GrdFree(response);
    }

    FreeBufferForArray(req);
    if (!isCopy1)
        env->ReleaseStringUTFChars(serialNumber, serialPtr);
    if (!isCopy2)
        env->ReleaseStringUTFChars(host, hostPtr);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseInstall
 * Signature: ([BI)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseInstall
    (JNIEnv* env, jclass, jbyteArray response, jint responseSize)
{
    jsize sz = env->GetArrayLength(response);
    if (sz != (jsize)responseSize)
        return GRD_INVALID_PARAMETER;

    void* r = GetBufferForArray(env, response, sz);
    jint ret = GrdLicenseInstall(r, (grd_uint32)responseSize);
    FreeBufferForArray(r);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseCreateUpdateRequest
 * Signature: (ILgrdlic/GrdByteArray;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseCreateUpdateRequest
    (JNIEnv* env, jclass, jint licenseId, jobject outRequest)
{
    void* r = NULL;
    grd_uint32 rSize = 0;
    jint ret = GrdLicenseCreateUpdateRequest((grd_uint32)licenseId, &r, &rSize);
    if (ret == GRD_OK)
    {
        jbyteArray requestArray = env->NewByteArray((jsize)rSize);
        jboolean isCopy;
        jbyte* buf = env->GetByteArrayElements(requestArray, &isCopy);
        memcpy(buf, r, (size_t)rSize);
        SetObjectField(env, outRequest, "mValue", "[B", requestArray);
        env->ReleaseByteArrayElements(requestArray, buf, 0);
        GrdFree(r);
    }
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseSendUpdateRequest
 * Signature: ([BILjava/lang/String;ILgrdlic/GrdByteArray;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseSendUpdateRequest
    (JNIEnv* env, jclass, jbyteArray request, jint requestSize, jstring host, jint port, jobject outResponse)
{
    jsize sz = env->GetArrayLength(request);
    if (sz != (jsize)requestSize)
        return GRD_INVALID_PARAMETER;

    jboolean isCopy;
    char* hostPtr = (char*)env->GetStringUTFChars(host, &isCopy);

    void* req = GetBufferForArray(env, request, sz);

    grd_uint32 rSize = 0;
    void* response = NULL;
    jint ret = GrdLicenseSendUpdateRequest(req, (grd_uint32)requestSize, hostPtr, (grd_uint32)port, &response, &rSize);
    if (ret == GRD_OK)
    {
        jbyteArray responseArray = env->NewByteArray((jsize)rSize);
        jboolean isCopy1;
        jbyte* buf = env->GetByteArrayElements(responseArray, &isCopy1);
        memcpy(buf, response, (size_t)rSize);
        SetObjectField(env, outResponse, "mValue", "[B", responseArray);
        env->ReleaseByteArrayElements(responseArray, buf, 0);
        GrdFree(response);
    }

    FreeBufferForArray(req);
    if (!isCopy)
        env->ReleaseStringUTFChars(host, hostPtr);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseExtractForRehost
 * Signature: (ILgrdlic/GrdByteArray;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseExtractForRehost
    (JNIEnv* env, jclass, jint licenseId, jobject outDonorLicense)
{
    void* outDonorLicensePtr = NULL;
    grd_uint32 rSize = 0;
    jint ret = GrdLicenseExtractForRehost((grd_uint32)licenseId, &outDonorLicensePtr, &rSize);
    if (ret == GRD_OK)
    {
        jbyteArray responseArray = env->NewByteArray((jsize)rSize);
        jboolean isCopy1;
        jbyte* buf = env->GetByteArrayElements(responseArray, &isCopy1);
        memcpy(buf, outDonorLicensePtr, (size_t)rSize);
        SetObjectField(env, outDonorLicense, "mValue", "[B", responseArray);
        env->ReleaseByteArrayElements(responseArray, buf, 0);
        GrdFree(outDonorLicensePtr);
    }
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseRehost
 * Signature: (Ljava/lang/String;I[BI)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseRehost
    (JNIEnv* env, jclass, jstring host, jint port, jbyteArray donorLicense, jint donorLicenseSize)
{
    jsize sz = env->GetArrayLength(donorLicense);
    if (sz != (jsize)donorLicenseSize)
        return GRD_INVALID_PARAMETER;

    jboolean isCopy;
    char* hostPtr = (char*)env->GetStringUTFChars(host, &isCopy);

    void* dnr = GetBufferForArray(env, donorLicense, sz);

    jint ret = GrdLicenseRehost(hostPtr, (grd_uint32)port, dnr, (grd_uint32)donorLicenseSize);
    FreeBufferForArray(dnr);
    if (!isCopy)
        env->ReleaseStringUTFChars(host, hostPtr);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseSendRehostRequest
 * Signature: (Ljava/lang/String;I[BILjava/lang/String;Lgrdlic/GrdByteArray;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseSendRehostRequest
    (JNIEnv* env, jclass,
    jstring host, jint port,
    jbyteArray donorLicense, jint donorLicenseSize,
    jstring recipientFingerprint,
    jobject recipientLicense)
{
    jsize sz = env->GetArrayLength(donorLicense);
    if (sz != (jsize)donorLicenseSize)
        return GRD_INVALID_PARAMETER;

    jboolean isCopy0;
    char* recipientFingerprintPtr = (char*)env->GetStringUTFChars(recipientFingerprint, &isCopy0);
    if (!recipientFingerprintPtr)
        return GRD_INVALID_PARAMETER;

    jboolean isCopy;
    char* hostPtr = (char*)env->GetStringUTFChars(host, &isCopy);

    void* dnr = GetBufferForArray(env, donorLicense, donorLicenseSize);

    void* outRecipientLicensePtr = NULL;
    grd_uint32 rSize = 0;
    jint ret = GrdLicenseSendRehostRequest(hostPtr, (grd_uint32)port,
                                           dnr, (grd_uint32)donorLicenseSize,
                                           recipientFingerprintPtr,
                                           &outRecipientLicensePtr, &rSize);
    if (ret == GRD_OK)
    {
        jbyteArray responseArray = env->NewByteArray((jsize)rSize);
        jboolean isCopy1;
        jbyte* buf = env->GetByteArrayElements(responseArray, &isCopy1);
        memcpy(buf, outRecipientLicensePtr, (size_t)rSize);
        SetObjectField(env, recipientLicense, "mValue", "[B", responseArray);
        env->ReleaseByteArrayElements(responseArray, buf, 0);
        GrdFree(outRecipientLicensePtr);
    }

    FreeBufferForArray(dnr);
    if (!isCopy)
        env->ReleaseStringUTFChars(host, hostPtr);
    if (!isCopy0)
        env->ReleaseStringUTFChars(recipientFingerprint, recipientFingerprintPtr);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseDetach
 * Signature: (Ljava/lang/String;Lgrdlic/GrdVendorCodes;[Lgrdlic/GrdProductFeaturePair;IILjava/lang/String;Lgrdlic/GrdByteArray;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseDetach
    (JNIEnv* env, jclass, jstring visibility, jobjectArray featuresListArr, jint duration, jint resource2Consume,
    jstring recipientFingerprint, jobject recipientLicenseObj)
{
    jboolean isCopy0;
    char* recipientFingerprintPtr = (char*)env->GetStringUTFChars(recipientFingerprint, &isCopy0);
    if (!recipientFingerprintPtr)
        return GRD_INVALID_PARAMETER;

    jboolean isCopy;
    char* vis = (char*)env->GetStringUTFChars(visibility, &isCopy);

    jsize featureListSize = env->GetArrayLength(featuresListArr);
    if (!featureListSize)
    {
        if (!isCopy)
            env->ReleaseStringUTFChars(visibility, vis);
        if (!isCopy0)
            env->ReleaseStringUTFChars(recipientFingerprint, recipientFingerprintPtr);
        return GRD_INVALID_PARAMETER;
    }

    GrdProductFeaturePair* fl = (GrdProductFeaturePair*)malloc((size_t)featureListSize * sizeof(GrdProductFeaturePair));
    for (jsize i = 0; i < featureListSize; ++i)
    {
        jobject f = env->GetObjectArrayElement(featuresListArr, i);
        jint productNumber;
        jint featureNumber;
        if (GetIntField(env, f, "mProductNumber", &productNumber)
            && GetIntField(env, f, "mFeatureNumber", &featureNumber))
        {
            fl[i].productNumber = (grd_uint32)productNumber;
            fl[i].featureNumber = (grd_uint32)featureNumber;
        }
        else
        {
            if (!isCopy)
                env->ReleaseStringUTFChars(visibility, vis);
            if (!isCopy0)
                env->ReleaseStringUTFChars(recipientFingerprint, recipientFingerprintPtr);
            return GRD_INVALID_PARAMETER;
        }
    }

    void* outRecipientLicensePtr = NULL;
    grd_uint32 rSize = 0;
    jint ret = GrdLicenseDetach(vis, fl, (grd_uint32)featureListSize,
                                (grd_time_t)duration, (grd_uint16)resource2Consume,
                                recipientFingerprintPtr,
                                &outRecipientLicensePtr, &rSize);
    free(fl);
    if (!isCopy)
        env->ReleaseStringUTFChars(visibility, vis);
    if (!isCopy0)
        env->ReleaseStringUTFChars(recipientFingerprint, recipientFingerprintPtr);

    if (ret == GRD_OK)
    {
        jbyteArray responseArray = env->NewByteArray((jsize)rSize);
        jboolean isCopy1;
        jbyte* buf = env->GetByteArrayElements(responseArray, &isCopy1);
        memcpy(buf, outRecipientLicensePtr, (size_t)rSize);
        SetObjectField(env, recipientLicenseObj, "mValue", "[B", responseArray);
        env->ReleaseByteArrayElements(responseArray, buf, 0);
        GrdFree(outRecipientLicensePtr);
    }

    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    LicenseReturnToPool
 * Signature: (ILgrdlic/GrdByteArray;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_LicenseReturnToPool
    (JNIEnv* env, jclass, jint licenseId, jobject licenseToReturnObj)
{
    void* ptr = NULL;
    grd_uint32 sz = 0;
    jint ret = GrdLicenseReturnToPool((grd_uint32)licenseId, &ptr, &sz);
    if (ret == GRD_OK)
    {
        jbyteArray responseArray = env->NewByteArray((jsize)sz);
        jboolean isCopy1;
        jbyte* buf = env->GetByteArrayElements(responseArray, &isCopy1);
        memcpy(buf, ptr, (size_t)sz);
        SetObjectField(env, licenseToReturnObj, "mValue", "[B", responseArray);
        env->ReleaseByteArrayElements(responseArray, buf, 0);
        GrdFree(ptr);
    }
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    GetHostFingerprint
 * Signature: (Lgrdlic/GrdString;Ljava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_GetHostFingerprint
    (JNIEnv* env, jclass, jobject outFingerprintObj, jstring visibility)
{
    char* pVisibility = NULL;
    jboolean isCopy;
    if (visibility)
        pVisibility = (char*)env->GetStringUTFChars(visibility, &isCopy);

    char* p = NULL;
    jint r = GrdGetHostFingerprint(&p, pVisibility);
    if (r == GRD_OK)
    {
        SetObjectField(env, outFingerprintObj, "mValue", "Ljava/lang/String;", env->NewStringUTF(p));
        GrdFree(p);
    }

    if (pVisibility && !isCopy)
        env->ReleaseStringUTFChars(visibility, pVisibility);

    return r;
}

/*
 * Class:     grdlic_api
 * Method:    DongleSetLicense
 * Signature: (ILjava/lang/String;ILjava/lang/String;I)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_DongleSetLicense
    (JNIEnv* env, jclass, jint hardwareDongleId, jstring serialNumber, jint driverType, jstring host, jint port)
{
    jboolean isCopy1;
    char* serialPtr = (char*)env->GetStringUTFChars(serialNumber, &isCopy1);
    jboolean isCopy2;
    char* hostPtr = (char*)env->GetStringUTFChars(host, &isCopy2);

    jint ret = GrdDongleSetLicense((grd_uint32)hardwareDongleId, serialPtr, (grd_uint32)driverType, hostPtr, (grd_uint32)port);

    if (!isCopy1)
        env->ReleaseStringUTFChars(serialNumber, serialPtr);
    if (!isCopy2)
        env->ReleaseStringUTFChars(host, hostPtr);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    GetSessionInfo
 * Signature: (Ljava/lang/String;Lgrdlic/GrdVendorCodes;Lgrdlic/GrdString;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_GetSessionInfo
  (JNIEnv *env, jclass, jstring visibility, jobject codes, jobject outJson)
{
    GrdVendorCodes vCodes;
    GrdVendorCodes* vCodesPtr = NULL;
    if (codes != NULL)
    {
        vCodesPtr = &vCodes;
        GetVendorCodes(env, codes, vCodes);
    }

    jboolean isCopy;
    char* vis = (char*)env->GetStringUTFChars(visibility, &isCopy);
    char* sessionInfoJson = NULL;
    jint ret = GrdGetSessionInfo(vis, vCodesPtr, &sessionInfoJson);
    if (!isCopy)
        env->ReleaseStringUTFChars(visibility, vis);
    if (ret == GRD_OK)
        SetObjectField(env, outJson, "mValue", "Ljava/lang/String;", env->NewStringUTF(sessionInfoJson));

    if (sessionInfoJson)
        GrdFree(sessionInfoJson);
    return ret;
}

/*
 * Class:     grdlic_api
 * Method:    GetHostInfo
 * Signature: (Ljava/lang/String;Lgrdlic/GrdString;)I
 */
JNIEXPORT jint JNICALL Java_grdlic_api_GetHostInfo
(JNIEnv *env, jclass, jstring visibility, jobject outJson)
{
  jboolean isCopy;
  char* vis = (char*)env->GetStringUTFChars(visibility, &isCopy);
  char* hostInfoJson = NULL;
  jint ret = GrdGetHostInfo(vis, &hostInfoJson);
  if (!isCopy)
      env->ReleaseStringUTFChars(visibility, vis);
  if (ret == GRD_OK)
      SetObjectField(env, outJson, "mValue", "Ljava/lang/String;", env->NewStringUTF(hostInfoJson));

  if (hostInfoJson)
      GrdFree(hostInfoJson);
  return ret;
}
