#ifndef GRD_JNI_FUNCTION_H
#define GRD_JNI_FUNCTION_H

#include <jni.h>

#ifdef __cplusplus
extern "C" {
#endif // __cplusplus

jfieldID GetFieldID(JNIEnv* env, jobject obj, const char* name, const char* sig);

bool SetByteField(JNIEnv* env, jobject obj, const char* name, jbyte value);
bool SetShortField(JNIEnv* env, jobject obj, const char* name, jshort value);
bool SetIntField(JNIEnv* env, jobject obj, const char* name, jint value);
bool SetLongField(JNIEnv* env, jobject obj, const char* name, jlong value);
bool SetObjectField(JNIEnv* env, jobject obj, const char* name, const char* sig, jobject value);

bool GetByteField(JNIEnv* env, jobject obj, const char* name, jbyte* value);
bool GetShortField(JNIEnv* env, jobject obj, const char* name, jshort* value);
bool GetIntField(JNIEnv* env, jobject obj, const char* name, jint* value);
bool GetLongField(JNIEnv* env, jobject obj, const char* name, jlong* value);
bool GetObjectField(JNIEnv* env, jobject obj, const char* name, const char* sig, jobject* ret_value);

bool GetStaticLongField(JNIEnv* env, const char* cls_name, const char* name, jlong* ret_value);

jmethodID GetMethodID(JNIEnv* env, jobject obj, const char* method_name, const char* method_sig);
jmethodID GetStaticMethodID(JNIEnv* env, const char* cls_name, const char* method_name, const char* method_sig, jclass* cls);

bool CallStaticIntMethod(JNIEnv* env, const char* cls_name, const char* method_name, const char* method_sig, jint* ret_value, ...);
bool CallStaticObjectMethod(JNIEnv* env, const char* cls_name, const char* method_name, const char* method_sig, jobject* ret_value, ...);

bool CallVoidMethodV(JNIEnv* env, jobject obj, const char* method_name, const char* method_sig, ...);
bool CallIntMethodV(JNIEnv* env, jobject obj, const char* method_name, const char* method_sig, jint* ret_value, ...);
bool CallObjectMethodV(JNIEnv* env, jobject obj, const char* method_name, const char* method_sig, jobject* ret_value, ...);
bool CallBooleanMethodV(JNIEnv* env, jobject obj, const char* method_name, const char* method_sig, jboolean* ret_value, ...);

bool CreateObjectV(JNIEnv* env, const char* cls_name, const char* ctor_name, const char* ctor_sign, jobject* newObj, ...);

#ifdef __cplusplus
}
#endif // __cplusplus

#endif // GRD_JNI_FUNCTION_H
