//------------------------------------------------------------------------------
// (C) 1997 - 2019 Aktiv Co. All rights reserved.
//  Guardant Licensing API example C
//------------------------------------------------------------------------------
// This example demonstrates Guardant Licensing API usage.
// This code sample is for API demonstration only!
// Please do not use this code in real projects!
//------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#include <windows.h>
#endif
#include <grdlic.h>

// demo access codes
#define DEMO_ACCESSCODE_PUBLIC       0x519175b7Lu  // Demo public code
#define DEMO_ACCESSCODE_PRIVATEREAD  0x51917645Lu  // Demo private read code
#define DEMO_ACCESSCODE_PRIVATEWRITE 0x51917603Lu  // Demo private write code

void PrintError(const char* Text);
void PrintText(const char* Text);
int PrintCode(int nErrorCode);
int ErrorHandling(GrdHandle handle, int nRet, int exitProgram);

//----------------------------------------------------------------------------------
//  Main function
//----------------------------------------------------------------------------------
int main(int argc, char** argv)
{
    GrdHandle handle = GRD_INVALID_HANDLE_VALUE;
    grd_uint32 featureId = 0;
    int status = GRD_OK;
    GrdVendorCodes vendorCodes = {DEMO_ACCESSCODE_PUBLIC, DEMO_ACCESSCODE_PRIVATEREAD, 0};
    //------------------------------------------------------------------------------
    // Setup license and feature search params
    //------------------------------------------------------------------------------
    const char* visibility =
        "{\"dongleModel\": 0x480, \"remoteMode\": 3}";
    //------------------------------------------------------------------------------
    // Retrieve information about all available licenses
    //------------------------------------------------------------------------------
    {
        char* licenseInfoList = NULL;
        printf("Retrieve information about all available licenses: ");
        status = GrdGetLicenseInfo(visibility, NULL, &licenseInfoList);
        ErrorHandling(handle, status, 1);

        printf("Information about all available licenses:\n%s\n", licenseInfoList);

        // Set up feature id from command line (if specified) or set arbitrary
        if (argc > 1)
            featureId = atoi(argv[1]);
        else
            featureId = 1;

        printf("For further work chosen feature with ID: %lu\n", featureId);

        GrdFree(licenseInfoList);
    }

    //------------------------------------------------------------------------------
    // Login to feature
    //------------------------------------------------------------------------------
    printf("Login to the feature with id %lu: ", featureId);
    status = GrdFeatureLogin(featureId, &vendorCodes, visibility, &handle);
    ErrorHandling(handle, status, 1);

    //------------------------------------------------------------------------------
    // Get information about the feature we are logged in
    //------------------------------------------------------------------------------
    {
        char* outLicenseInfo = NULL;
        printf("Get information about the feature we are logged in: ");
        status = GrdFeatureGetInfo(handle, &outLicenseInfo);
        ErrorHandling(handle, status, 1);

        printf("Information about the feature we are logged in:\n%s\n", outLicenseInfo);

        GrdFree(outLicenseInfo);
    }

    //------------------------------------------------------------------------------
    // Encryption and decryption by AES128
    //------------------------------------------------------------------------------
    {
        size_t bufSize = 128;
        grd_uint8 buf[128];
        grd_uint8 outBuf[128];
        memset(buf, 0xFF, bufSize);

        printf("Encrypt some data using the feature with id %lu and taking into account"
               " it's license limitations (run counter or life time period): ", featureId);
        memcpy(outBuf, buf, bufSize);
        status = GrdFeatureEncrypt(handle, bufSize, outBuf, GRD_EM_ECB, NULL, 0);
        ErrorHandling(handle, status, 1);
        printf("Decrypt previously encrypted data using the feature with id %lu and taking into account"
               " it's license limitations (run counter or life time period): ", featureId);
        status = GrdFeatureDecrypt(handle, bufSize, outBuf, GRD_EM_ECB, NULL, 0);
        ErrorHandling(handle, status, 1);

        if (memcmp(buf, outBuf, bufSize) != 0)
        {
            PrintError("Result of decryption mismatched with origin data\n!");
            exit(-1);
        }

        printf("Encrypt some data using the feature with id %lu without licensing limitations: ", featureId);
        memcpy(outBuf, buf, bufSize);
        status = GrdFeatureEncrypt(handle, bufSize, outBuf, GRD_NO_COUNTER_DECREMENT | GRD_EM_ECB, NULL, 0);
        ErrorHandling(handle, status, 1);
        printf("Decrypte previously encrypted data using the feature with id %lu without licensing limitations: ", featureId);
        status = GrdFeatureDecrypt(handle, bufSize, outBuf, GRD_NO_COUNTER_DECREMENT | GRD_EM_ECB, NULL, 0);
        ErrorHandling(handle, status, 1);

        if (memcmp(buf, outBuf, bufSize) != 0)
        {
            PrintError("Result of decryption mismatched with origin data\n!");
            exit(-1);
        }
    }

    //------------------------------------------------------------------------------
    // Digitally sign a block of bytes
    //------------------------------------------------------------------------------
    {
        char digest[GRD_ECC160_DIGEST_SIZE];
        char message[GRD_ECC160_MESSAGE_SIZE] = "__Message for sign_";
        grd_uint8 publicKey[] = "\x1D\xCE\x42\x91\xEF\xF6\x8C\x6A\xEC\xB6\xC6\x76\x7E\xF0\xCC\x0B\x61\xD5\xA1\x73\x69\x14\x53\x05\xC3\xFD\x98\x93\xD6\x0A\xF8\xFA\x03\xE1\x8C\x37\x99\x4B\x95\x6B";

        printf("Digitally sign a message: ");
        status = GrdFeatureSign(handle, GRD_ECC160_MESSAGE_SIZE, message, GRD_ECC160_DIGEST_SIZE, digest);
        ErrorHandling(handle, status, 1);

        //        printf("Verifing digitally sign a message: ");
        //        status = GrdVerifyDigest(GRD_ECC160_PUBLIC_KEY_SIZE, publicKey, GRD_ECC160_MESSAGE_SIZE, message, GRD_ECC160_DIGEST_SIZE, digest);
        //        ErrorHandling(handle, status, 1);
    }

    //------------------------------------------------------------------------------
    // Get rest of the feature life time (valid for time limited feature only)
    //------------------------------------------------------------------------------
    {
        grd_time_t remainingTime = 0;
        printf("Get rest of the feature life time: ");
        status = GrdFeatureGetTimeLimit(handle, &remainingTime);
        ErrorHandling(handle, status, 0);
        if (status == GRD_OK)
            printf("Rest of feature life time in seconds: %d\n", remainingTime);
    }

    //------------------------------------------------------------------------------
    // Get current date and time (valid for time limited feature only)
    //------------------------------------------------------------------------------
    {
        grd_time_t currentTime = 0;
        status = GrdGetRealTime(handle, &currentTime);
        ErrorHandling(handle, status, 0);
        if (status == GRD_OK)
            printf("Current datetime in seconds: %d\n", currentTime);
    }

    //------------------------------------------------------------------------------
    // Working with memory (valid for feature with memory only)
    //------------------------------------------------------------------------------
    if (argc > 2)
    {
        grd_uint32 memoryId = atoi(argv[2]);
        grd_uint32 memSize;
        grd_uint32 password = 0; // no password
        grd_uint32 offset = 0;
        grd_uint8* memBuf = NULL;

        printf("Get memory size: ");
        status = GrdMemoryGetSize(handle, memoryId, &memSize);
        ErrorHandling(handle, status, 0);
        if (status != GRD_OK)
            memSize = 4; // 4 bytes by default
        printf("Memory size: %d\n", memSize);

        printf("Read memory: ");
        memBuf = (grd_uint8*)malloc((size_t)memSize);
        status = GrdMemoryRead(handle, memoryId, password, memSize, offset, memBuf);
        free(memBuf);
        ErrorHandling(handle, status, 0);
    }

    //------------------------------------------------------------------------------
    // Logout from feature
    //------------------------------------------------------------------------------
    printf("Logout from the feature with id %lu: ", featureId);
    status = GrdFeatureLogout(handle);
    ErrorHandling(handle, status, 1);

    PrintText("\nALL SUCCESSFULLY DONE!\n");

    return 0;
}

//----------------------------------------------------------------------------------------------
// Prints operation result and forces program termination on error
//----------------------------------------------------------------------------------------------
void PrintError(const char* Text)
{
#ifdef WIN32
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), FOREGROUND_RED);
    printf("%s\n", Text);
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_BLUE);
#elif defined(__linux) || defined(__linux__)
    printf("\033[1;31m");
    printf("%s\n", Text);
    printf("\033[0m");
#else
    printf("%s\n", Text);
#endif
}

//----------------------------------------------------------------------------------------------
// Prints operation result and forces program termination on error
//----------------------------------------------------------------------------------------------
void PrintText(const char* Text)
{
#ifdef WIN32
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), FOREGROUND_GREEN);
    printf("%s\n", Text);
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_BLUE);
#elif defined(__linux) || defined(__linux__)
    printf("\033[1;32m");
    printf("%s\n", Text);
    printf("\033[0m");
#else
    printf("%s\n", Text);
#endif
}

//----------------------------------------------------------------------------------------------
// Prints operation result and forces program termination on error
//----------------------------------------------------------------------------------------------
int PrintCode(int nErrorCode)
{
    // Buffer for error string
    if (nErrorCode == GRD_OK)
        PrintText("success");
    else
    {
        char szErrorMsg[256];
        sprintf(szErrorMsg, "ERROR! %d ", nErrorCode);
        PrintError(szErrorMsg);
    }
    return nErrorCode;
}

//----------------------------------------------------------------------------------------------
// Handle errors
// Prints operation result, logout from feature
//----------------------------------------------------------------------------------------------
int ErrorHandling(GrdHandle handle, int nRet, int exitProgram)
{
    PrintCode(nRet);

    if (nRet != GRD_OK && exitProgram)
    {
        printf("Logout from feature: ");
        nRet = GrdFeatureLogout(handle);
        PrintCode(nRet);

        // Terminate application
        PrintError("Application terminated by exit()");
        exit(nRet);
    }
    return nRet;
}
//------------------------------------------------------------------------------
