unit fruActivation;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes,
  Vcl.Graphics, Vcl.Controls, Vcl.Forms, Vcl.Dialogs, Vcl.ComCtrls, Vcl.StdCtrls,
  Vcl.ExtCtrls,
  fruBaseFrame, grdlic_delphi, uHelper, System.ImageList, Vcl.ImgList;

type
  Tfrm_Activation = class(TBaseFrame)
    pgc_Main: TPageControl;
    ts_Online: TTabSheet;
    ts_Offline: TTabSheet;
    lbl_SerialNumber: TLabel;
    edt_SerialNumber: TEdit;
    btn_Activate: TButton;
    grpActivation: TGroupBox;
    lbl_PrepareActivationRequest: TLabel;
    lbl_SendActivateRequest: TLabel;
    lbl_InstallLicense: TLabel;
    lbl_RequestFile: TLabel;
    lbl_LicenseFile: TLabel;
    btn_InstallLicense: TButton;
    btn_CreateActivationRequest: TButton;
    btn_ObtainLicense: TButton;
    grpUpdate: TGroupBox;
    Label2: TLabel;
    Label3: TLabel;
    Label4: TLabel;
    Label5: TLabel;
    Label6: TLabel;
    lbl_SerialNumber_Upd: TLabel;
    btn_InstallUpdate: TButton;
    btn_CreateUpdateRequest: TButton;
    btn_ObtainUpdate: TButton;
    edt_LicenseId: TEdit;
    grp_OnlineActivation: TGroupBox;
    pnl_Activation: TPanel;
    pnl_Update: TPanel;
    Label7: TLabel;
    lbl_NewLicenseId: TLabel;
    Edit2: TEdit;
    edt_NewLicenseId: TEdit;
    grp_Common: TGroupBox;
    lbl_FirstName: TLabel;
    lbl_LastName: TLabel;
    lbl_CompanyName: TLabel;
    lbl_Phone: TLabel;
    lbl_Email: TLabel;
    edt_Email: TEdit;
    edt_FirstName: TEdit;
    edt_LastName: TEdit;
    edt_CompanyName: TEdit;
    edt_Phone: TEdit;
    cb_CustomerRequired: TCheckBox;
    dlg_Open: TOpenDialog;
    dlg_Save: TSaveDialog;
    edt_LicenseActivationRequestFile: TButtonedEdit;
    iml_Buttons: TImageList;
    edt_LicenseActivationResponseFile: TButtonedEdit;
    lbl_CustomerInfo: TLabel;
    grp_OnlineUpdate: TGroupBox;
    btn_CheckUpdate: TButton;
    lbl_UpdateAvialable: TLabel;
    btn_Update: TButton;
    edt_LicenseUpdateRequestFile: TButtonedEdit;
    edt_LicenseUpdateResponseFile: TButtonedEdit;
    procedure btn_ActivateClick(Sender: TObject);
    procedure btn_CheckUpdateClick(Sender: TObject);
    procedure btn_CreateActivationRequestClick(Sender: TObject);
    procedure btn_CreateUpdateRequestClick(Sender: TObject);
    procedure btn_InstallLicenseClick(Sender: TObject);
    procedure btn_InstallUpdateClick(Sender: TObject);
    procedure btn_ObtainLicenseClick(Sender: TObject);
    procedure btn_ObtainUpdateClick(Sender: TObject);
    procedure btn_UpdateClick(Sender: TObject);
    procedure edt_LicenseActivationRequestFileRightButtonClick(Sender: TObject);
    procedure edt_LicenseActivationResponseFileRightButtonClick(Sender: TObject);
    procedure edt_LicenseUpdateRequestFileRightButtonClick(Sender: TObject);
    procedure edt_LicenseUpdateResponseFileRightButtonClick(Sender: TObject);
  private
    { Private declarations }
    FLicenseManager: TLicenseManager;
  protected

  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

implementation

{$R *.dfm}

procedure Tfrm_Activation.btn_ActivateClick(Sender: TObject);
var
  CustomerInfo: PCustomerInfo;
  Sn: String;
  NewLicenseID: Cardinal;
begin
  if cb_CustomerRequired.Checked then
  begin
    new(CustomerInfo);
    CustomerInfo.FirstName:=   edt_FirstName.Text;
    CustomerInfo.LastName:=    edt_LastName.Text;
    CustomerInfo.Email:=       edt_Email.Text;
    CustomerInfo.CompanyName:= edt_CompanyName.Text;
    CustomerInfo.Phone:=       edt_Phone.Text;
  end
  else
  begin
    CustomerInfo:= nil;
  end;

  sn:= edt_SerialNumber.Text;

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;

  NewLicenseID:= FLicenseManager.LicenseActivate(Sn, CustomerInfo);
  SendLog('Activate', FLicenseManager.Error);

  if Assigned(CustomerInfo) then
    Dispose(CustomerInfo);

  if FLicenseManager.Error <> 0 then
  begin
    Exit;
  end;

  edt_NewLicenseId.Text:= IntToHex(NewLicenseID, 8);
end;

{ Tfrm_Activation }

constructor Tfrm_Activation.Create(AOwner: TComponent);
begin
  inherited;
  FLicenseManager:= TLicenseManager.Create();
  lbl_UpdateAvialable.Caption:= '';
  lbl_UpdateAvialable.Font.Color:= clBlack;
  Description:= '   .';
end;

destructor Tfrm_Activation.Destroy;
begin
  if Assigned(FLicenseManager) then
    FLicenseManager.Free;
  inherited;
end;

procedure Tfrm_Activation.btn_CheckUpdateClick(Sender: TObject);
var
  LicenseId: Cardinal;
begin
  LicenseId:= StrToUInt('$' + edt_LicenseId.Text);
  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;

  FLicenseManager.LicenseCheckUpdateIsAvailable(LicenseId);
  SendLog('GrdLicenseCheckUpdateIsAvailable', FLicenseManager.Error);
  if (FLicenseManager.Error = 0) then
  begin
    lbl_UpdateAvialable.Caption:= ' ';
    lbl_UpdateAvialable.Font.Color:= clGreen;
  end
  else
  begin
    lbl_UpdateAvialable.Caption:= '    ';
    lbl_UpdateAvialable.Font.Color:= clRed;
  end;
end;

procedure Tfrm_Activation.btn_CreateActivationRequestClick(Sender: TObject);
var
  PathToFile: string;
  Request: TBytes;
begin
  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseCreateActivationRequest(Request);
  SendLog('GrdLicenseCreateActivationRequest', FLicenseManager.Error);

  if FLicenseManager.Error <> 0 then
  begin
    Exit;
  end;

  dlg_Save.DefaultExt:= '.request';
  dlg_Save.InitialDir := GetCurrentDir;
  dlg_Save.FileName:= LocalCimputerName + '.activation';
  dlg_Save.Filter:= 'request files|*.request|all files|*.*';
  if dlg_Save.Execute() = True then
    PathToFile:= dlg_Save.FileName
  else
    PathToFile:= dlg_Save.FileName + '.tmp';

  BufferToFile(PathToFile, Request);
  edt_LicenseActivationRequestFile.Text:= PathToFile;
  SetLength(Request, 0);
end;

procedure Tfrm_Activation.btn_CreateUpdateRequestClick(Sender: TObject);
var
  PathToFile: string;
  Request: TBytes;
  LicenseId: Cardinal;
begin
  LicenseId:= StrToUInt('$' + edt_LicenseId.Text);

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseCreateUpdateRequest(LicenseId, Request);
  SendLog('GrdLicenseCreateUpdateRequest', FLicenseManager.Error);

  if FLicenseManager.Error <> 0 then
    Exit;

  dlg_Save.DefaultExt:= '.request';
  dlg_Save.InitialDir := GetCurrentDir;
  dlg_Save.FileName:= LocalCimputerName + '-' + edt_LicenseId.Text + '.update';
  dlg_Save.Filter:= 'request files|*.request|all files|*.*';
  if dlg_Save.Execute() = True then
    PathToFile:= dlg_Save.FileName
  else
    PathToFile:= dlg_Save.FileName + '.tmp';

  BufferToFile(PathToFile, Request);
  edt_LicenseUpdateRequestFile.Text:= PathToFile;
  SetLength(Request, 0);
end;

procedure Tfrm_Activation.btn_InstallLicenseClick(Sender: TObject);
var
  PathToResponseFile: string;
  Response: TBytes;
begin
  if not FileExists(edt_LicenseActivationResponseFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.license';
    dlg_Open.InitialDir := GetCurrentDir;
    dlg_Open.Filter:= 'license files|*.license|all files|*.*';
    if dlg_Open.Execute() = False then
      PathToResponseFile:= dlg_Open.FileName
    else
      Exit;
  end
  else
    PathToResponseFile:= edt_LicenseActivationResponseFile.Text;

  FileToBuffer(PathToResponseFile, Response);

  FLicenseManager.LicenseInstall(Response);
  SendLog('GrdLicenseInstall', FLicenseManager.Error);
  SetLength(Response, 0);
end;

procedure Tfrm_Activation.btn_InstallUpdateClick(Sender: TObject);
var
  PathToResponseFile: string;
  Response: TBytes;
begin
  if not FileExists(edt_LicenseUpdateResponseFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.license';
    dlg_Open.InitialDir := GetCurrentDir;
    dlg_Open.Filter:= 'license files|*.license|all files|*.*';
    if dlg_Open.Execute() then
      PathToResponseFile:= dlg_Open.FileName
    else
      Exit;
  end
  else
    PathToResponseFile:= edt_LicenseUpdateResponseFile.Text;

  FileToBuffer(PathToResponseFile, Response);

  FLicenseManager.LicenseInstall(Response);
  SendLog('GrdLicenseInstall', FLicenseManager.Error);
  SetLength(Response, 0);
end;

procedure Tfrm_Activation.btn_ObtainLicenseClick(Sender: TObject);
var
  PathToRequestFile: string;
  Request: TBytes;
  PathToResponseFile: string;
  Response: TBytes;
  CustomerInfo: PCustomerInfo;
begin
  if not FileExists(edt_LicenseActivationRequestFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.request';
    dlg_Open.InitialDir := GetCurrentDir;
    dlg_Open.FileName:= LocalCimputerName + '.activation';
    dlg_Open.Filter:= 'request files|*.request|all files|*.*';
    if dlg_Open.Execute() = False then
      Exit
    else
      PathToRequestFile:= dlg_Open.FileName;
  end
  else
  begin
    PathToRequestFile:= edt_LicenseActivationRequestFile.Text;
  end;

  FileToBuffer(PathToRequestFile, Request);

  if cb_CustomerRequired.Checked then
  begin
    new(CustomerInfo);
    CustomerInfo.FirstName:=   edt_FirstName.Text;
    CustomerInfo.LastName:=    edt_LastName.Text;
    CustomerInfo.Email:=       edt_Email.Text;
    CustomerInfo.CompanyName:= edt_CompanyName.Text;
    CustomerInfo.Phone:=       edt_Phone.Text;
  end
  else
  begin
    CustomerInfo:= nil;
  end;

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseSendActivationRequest(edt_SerialNumber.Text, Request, Response, CustomerInfo);
  SendLog('GrdLicenseSendActivationRequest', FLicenseManager.Error);
  if FLicenseManager.Error <> 0 then
  begin
    Exit;
  end;

  dlg_Save.DefaultExt:= '.license';
  dlg_Save.InitialDir := GetCurrentDir;
  dlg_Save.FileName:= LocalCimputerName + '.activation';
  dlg_Save.Filter:= 'license files|*.license|all files|*.*';
  if dlg_Save.Execute() = True then
    PathToResponseFile:= dlg_Save.FileName
  else
    PathToResponseFile:= dlg_Save.FileName + '.tmp';

  BufferToFile(PathToResponseFile, Response);
  edt_LicenseActivationResponseFile.Text:= PathToResponseFile;
  SetLength(Request, 0);
  SetLength(Response, 0);
end;

procedure Tfrm_Activation.btn_ObtainUpdateClick(Sender: TObject);
var
  PathToRequestFile: string;
  Request: TBytes;
  PathToResponseFile: string;
  Response: TBytes;
begin
  if not FileExists(edt_LicenseUpdateRequestFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.request';
    dlg_Open.InitialDir := GetCurrentDir;
    dlg_Open.FileName:= LocalCimputerName + '.update';
    dlg_Open.Filter:= 'request files|*.request|all files|*.*';
    if dlg_Open.Execute() = False then
      Exit
    else
      PathToRequestFile:= dlg_Open.FileName;
  end
  else
  begin
    PathToRequestFile:= edt_LicenseUpdateRequestFile.Text;
  end;

  FileToBuffer(PathToRequestFile, Request);

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseSendUpdateRequest(Request, Response);
  SendLog('GrdLicenseSendUpdateRequest', FLicenseManager.Error);
  if FLicenseManager.Error <> 0 then
  begin
    Exit;
  end;

  dlg_Save.DefaultExt:= '.license';
  dlg_Save.InitialDir := GetCurrentDir;
  dlg_Save.FileName:= LocalCimputerName + '.update';
  dlg_Save.Filter:= 'license files|*.license|all files|*.*';
  if dlg_Save.Execute() = True then
    PathToResponseFile:= dlg_Save.FileName
  else
    PathToResponseFile:= dlg_Save.FileName + '.tmp';

  BufferToFile(PathToResponseFile, Response);
  edt_LicenseUpdateResponseFile.Text:= PathToResponseFile;
  SetLength(Request, 0);
  SetLength(Response, 0);
end;

procedure Tfrm_Activation.btn_UpdateClick(Sender: TObject);
var
  LicenseId: Cardinal;
begin
  LicenseId:= StrToUInt('$' + edt_LicenseId.Text);
  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;

  FLicenseManager.LicenseUpdate(LicenseId);
  SendLog('GrdLicenseUpdate', FLicenseManager.Error);
end;

procedure Tfrm_Activation.edt_LicenseActivationRequestFileRightButtonClick(Sender: TObject);
begin
  dlg_Open.DefaultExt:= '.request';
  dlg_Open.InitialDir := GetCurrentDir;
  dlg_Open.FileName:= LocalCimputerName + '.activation';
  dlg_Open.Filter:= 'request files|*.request|all files|*.*';
  if dlg_Open.Execute() = True then
    edt_LicenseActivationRequestFile.Text:= dlg_Open.FileName
  else
end;

procedure Tfrm_Activation.edt_LicenseActivationResponseFileRightButtonClick(Sender:
    TObject);
begin
  dlg_Open.DefaultExt:= '.license';
  dlg_Open.InitialDir := GetCurrentDir;
  dlg_Open.FileName:= LocalCimputerName + '.activation';
  dlg_Open.Filter:= 'license files|*.license|all files|*.*';
  if dlg_Open.Execute() = True then
    edt_LicenseActivationResponseFile.Text:= dlg_Open.FileName;
end;

procedure Tfrm_Activation.edt_LicenseUpdateRequestFileRightButtonClick(Sender:
    TObject);
begin
  dlg_Open.DefaultExt:= '.request';
  dlg_Open.InitialDir := GetCurrentDir;
  dlg_Open.FileName:= LocalCimputerName + '.update';
  dlg_Open.Filter:= 'request files|*.request|all files|*.*';
  if dlg_Open.Execute() = True then
    edt_LicenseUpdateRequestFile.Text:= dlg_Open.FileName;
end;

procedure Tfrm_Activation.edt_LicenseUpdateResponseFileRightButtonClick(Sender:
    TObject);
begin
  dlg_Open.DefaultExt:= '.license';
  dlg_Open.InitialDir := GetCurrentDir;
  dlg_Open.FileName:= LocalCimputerName + '.update';
  dlg_Open.Filter:= 'license files|*.license|all files|*.*';
  if dlg_Open.Execute() = True then
    edt_LicenseUpdateResponseFile.Text:= dlg_Open.FileName;
end;

end.
