unit fruDetach;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, fruBaseFrame, grdlic_delphi,
  Vcl.StdCtrls, uHelper, System.ImageList, Vcl.ImgList, Vcl.ExtCtrls;

type
  Tfrm_Detach = class(TBaseFrame)
    btn_DetachLicense: TButton;
    edt_HostName: TEdit;
    lbl_HostName: TLabel;
    lbl_GetFingerprint: TLabel;
    lbl_FingerPrintFile: TLabel;
    edt_FingerPrintFile: TButtonedEdit;
    btn_GetFingerPrint: TButton;
    dlg_Open: TOpenDialog;
    dlg_Save: TSaveDialog;
    iml_Buttons: TImageList;
    edt_LicenseDetactResponseFile: TButtonedEdit;
    edt_LicenseId: TEdit;
    lbl_License: TLabel;
    edt_ProductNumber: TEdit;
    edt_FeatureNumber: TEdit;
    lbl_Product: TLabel;
    lbl_Feature: TLabel;
    edt_LicenseCount: TEdit;
    lbl_LicenseCount: TLabel;
    lbl_InstallLicense: TLabel;
    btn_InstallLicense: TButton;
    lbl_SendActivateRequest: TLabel;
    lbl_LicenseDetachResponseFile: TLabel;
    lbl_Duration: TLabel;
    edt_Duration: TEdit;
    grp_Detuch: TGroupBox;
    grp_Return: TGroupBox;
    btn_ReturnToPool: TButton;
    edt_LicenseIdToReturn: TEdit;
    lbl_LicenseToRetirn: TLabel;
    edt_LicenseToReturnFile: TButtonedEdit;
    lbl_ReturnFile: TLabel;
    lbl_1: TLabel;
    lbl_Step2Return: TLabel;
    btn_ReturnInstall: TButton;
    cb_IsRemoteComputer: TCheckBox;
    procedure btn_DetachLicenseClick(Sender: TObject);
    procedure btn_GetFingerPrintClick(Sender: TObject);
    procedure btn_InstallLicenseClick(Sender: TObject);
    procedure btn_ReturnInstallClick(Sender: TObject);
    procedure btn_ReturnToPoolClick(Sender: TObject);
    procedure edt_FingerPrintFileRightButtonClick(Sender: TObject);
    procedure edt_LicenseDetactResponseFileRightButtonClick(Sender: TObject);
    procedure edt_LicenseToReturnFileRightButtonClick(Sender: TObject);
  private
    { Private declarations }
    FLicenseManager: TLicenseManager;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  frm_Detach: Tfrm_Detach;

implementation

{$R *.dfm}

{ Tfrm_Detach }

constructor Tfrm_Detach.Create(AOwner: TComponent);
begin
  inherited;
  FLicenseManager:= TLicenseManager.Create();
  Description:= ' .';
end;

destructor Tfrm_Detach.Destroy;
begin
  if Assigned(FLicenseManager) then
    FLicenseManager.Free;
  inherited;
end;

procedure Tfrm_Detach.btn_DetachLicenseClick(Sender: TObject);
var
  Visibility: TVisibility;
  Fingerprint: AnsiString;
  PathToFingerPrintFile, PathToFile: String;
  FingerPrintBuffer: TBytes;
  FeatureList: TProductFeaturePairs;
  recipientLicense: TBytes;
  Duration: Integer;
  Resource: Word;
begin
  Visibility.LicenseId:= StrToUInt('$' + edt_LicenseId.Text);
  Visibility.RemoteMode:= rmLocal;
  Visibility.DongleModel:= dmDl;
  Visibility.ProductNumber:= StrToUInt(edt_ProductNumber.Text);
  Visibility.NetworkResourceToConsume:= StrToUInt(edt_LicenseCount.Text);;
  Visibility.BroadcastSearch:= false;
  Visibility.HostName:='localhost';

  FLicenseManager.Visibility:= Visibility;
  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;

  SetLength(FeatureList, 1);
  FeatureList[0].productNumber:= StrToUInt(edt_ProductNumber.Text);
  FeatureList[0].featureNumber:= StrToUInt(edt_FeatureNumber.Text);

  if not FileExists(edt_FingerPrintFile.Text) then
  begin
    dlg_Open.FileName:= '';
    dlg_Open.DefaultExt:= '.fingerprint';
    dlg_Open.InitialDir := GetCurrentDir;
    dlg_Open.Filter:= 'fingerprint files|*.fingerprint|all files|*.*';
    if not dlg_Open.Execute then
      Exit
    else
      PathToFingerPrintFile:= dlg_Open.FileName;
  end
  else
  begin
    PathToFingerPrintFile:= edt_FingerPrintFile.Text;
  end;

  FileToBuffer(PathToFingerPrintFile, FingerPrintBuffer);
  SetLength(FingerPrint, Length(FingerPrintBuffer));
  Move(Pointer(FingerPrintBuffer)^, Pointer(FingerPrint)^, Length(FingerPrintBuffer));

  Duration:= StrToInt(edt_Duration.Text);
  Resource:= StrToInt(edt_LicenseCount.Text);

  FLicenseManager.LicenseDetach(FeatureList, Duration, Resource, PAnsiChar(FingerPrint), recipientLicense);
  SendLog('GrdLicenseDetach', FLicenseManager.Error);
  if FLicenseManager.Error <> 0 then
  begin
    Exit;
  end;

  dlg_Save.FileName:= '';
  dlg_Save.DefaultExt:= '.license';
  dlg_Save.InitialDir := GetCurrentDir;
  dlg_Save.Filter:= 'license files|*.license|all files|*.*';
  if dlg_Save.Execute then
    PathToFile:= dlg_Save.FileName
  else
    Exit;

  BufferToFile(PathToFile, recipientLicense);
  edt_LicenseDetactResponseFile.Text:= PathToFile;
end;

procedure Tfrm_Detach.btn_GetFingerPrintClick(Sender: TObject);
var
  Visibility: TVisibility;
  Fingerprint: PAnsiChar;
  PathToFile: String;
  Buffer: TBytes;
begin
  Visibility.LicenseId:= 0;
  if cb_IsRemoteComputer.Checked then
    Visibility.RemoteMode:= rmRemote
  else
    Visibility.RemoteMode:= rmLocal;

  Visibility.DongleModel:= dmDl;
  Visibility.ProductNumber:= 0;
  Visibility.NetworkResourceToConsume:= 1;
  Visibility.BroadcastSearch:= false;
  Visibility.HostName:= AnsiString(edt_HostName.Text);

  FLicenseManager.Visibility:= Visibility;
  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;

  FLicenseManager.GetHostFingerprint(Fingerprint);
  SendLog('GetHostFingerprint', FLicenseManager.Error);
  if FLicenseManager.Error <> 0 then
    Exit;

  dlg_Save.DefaultExt:= '.fingerprint';
  dlg_Save.InitialDir := GetCurrentDir;
  dlg_Save.FileName:= edt_HostName.Text;
  dlg_Save.Filter:= 'fingerprint files|*.fingerprint|all files|*.*';
  if dlg_Save.Execute then
    PathToFile:= dlg_Save.FileName
  else
    Exit;

  SetLength(Buffer, Length(Fingerprint));
  Move(Pointer(Fingerprint)^, Pointer(Buffer)^, Length(Fingerprint));

  BufferToFile(PathToFile, Buffer);
  edt_FingerPrintFile.Text:= PathToFile;

  SetLength(Buffer, 0);
end;

procedure Tfrm_Detach.btn_InstallLicenseClick(Sender: TObject);
var
  PathToResponseFile: string;
  Response: TBytes;
begin
  if not FileExists(edt_LicenseDetactResponseFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.license';
    dlg_Open.InitialDir:= GetCurrentDir;
    dlg_Open.FileName:= '';
    dlg_Open.Filter:= 'license files|*.license|all files|*.*';
    if dlg_Open.Execute then
      PathToResponseFile:= dlg_Open.FileName
    else
      Exit;
  end
  else
    PathToResponseFile:= edt_LicenseDetactResponseFile.Text;

  FileToBuffer(PathToResponseFile, Response);

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseInstall(Response);
  SendLog('GrdLicenseInstall', FLicenseManager.Error);
  SetLength(Response, 0);
end;

procedure Tfrm_Detach.btn_ReturnInstallClick(Sender: TObject);
var
  PathToResponseFile: string;
  Response: TBytes;
begin
  if not FileExists(edt_LicenseToReturnFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.license';
    dlg_Open.InitialDir:= GetCurrentDir;
    dlg_Open.FileName:= '';
    dlg_Open.Filter:= 'license files|*.license|all files|*.*';
    if dlg_Open.Execute then
      PathToResponseFile:= dlg_Open.FileName
    else
      Exit;
  end
  else
    PathToResponseFile:= edt_LicenseToReturnFile.Text;

  FileToBuffer(PathToResponseFile, Response);

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseInstall(Response);
  SendLog('GrdLicenseInstall', FLicenseManager.Error);
  SetLength(Response, 0);
end;

procedure Tfrm_Detach.btn_ReturnToPoolClick(Sender: TObject);
var
  LicenseId: Cardinal;
  PathToResponseFile: String;
  Response: TBytes;
begin
  LicenseId:= StrToUint('$'+ edt_LicenseIdToReturn.Text);

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseReturnToPool(LicenseId, Response);
  SendLog('LicenseReturnToPool', FLicenseManager.Error);
  if FLicenseManager.Error <> 0 then
    Exit;

  dlg_Open.DefaultExt:= '.license';
  dlg_Open.InitialDir:= GetCurrentDir;
  dlg_Open.FileName:= '';
  dlg_Save.Filter:= 'detach files|*.detach|all files|*.*';
  if dlg_Save.Execute() = True then
    PathToResponseFile:= dlg_Save.FileName
  else
    Exit;

  BufferToFile(PathToResponseFile, Response);
  edt_LicenseToReturnFile.Text:= PathToResponseFile;
  SetLength(Response, 0);
end;

procedure Tfrm_Detach.edt_FingerPrintFileRightButtonClick(Sender: TObject);
begin
  dlg_Open.DefaultExt:= '.fingerprint';
  dlg_Open.InitialDir := GetCurrentDir;
  dlg_Open.FileName:= '';
  dlg_Open.Filter:= 'fingerprint files|*.fingerprint|all files|*.*';
  if dlg_Open.Execute then
    edt_FingerPrintFile.Text:= dlg_Open.FileName;
end;

procedure Tfrm_Detach.edt_LicenseDetactResponseFileRightButtonClick(Sender:
    TObject);
begin
    dlg_Open.DefaultExt:= '.license';
    dlg_Open.InitialDir:= GetCurrentDir;
    dlg_Open.FileName:= '';
    dlg_Open.Filter:= 'license files|*.license|all files|*.*';
    if dlg_Open.Execute then
      edt_LicenseDetactResponseFile.Text:= dlg_Open.FileName;
end;

procedure Tfrm_Detach.edt_LicenseToReturnFileRightButtonClick(Sender: TObject);
begin
    dlg_Open.DefaultExt:= '.license';
    dlg_Open.InitialDir:= GetCurrentDir;
    dlg_Open.FileName:= '';
    dlg_Open.Filter:= 'license files|*.license|all files|*.*';
    if dlg_Open.Execute then
      edt_LicenseToReturnFile.Text:= dlg_Open.FileName;
end;

end.
