unit fruLicenses;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, fruBaseFrame, grdlic_delphi,
  Vcl.StdCtrls, Vcl.ComCtrls, Vcl.ExtCtrls, System.JSON;

type
  TNodeInfo = record
    LicenseId:    Cardinal;
    ProductNuber: Cardinal;
    FeatureNuber: Cardinal;
  end;
  PNodeInfo = ^TNodeInfo;

type
  Tfrm_Licenses = class(TBaseFrame)
    grp_RemoteMode: TGroupBox;
    cb_LocalMode: TCheckBox;
    cb_RemoteMode: TCheckBox;
    lbl_HostHame: TLabel;
    edt_HostName: TEdit;
    btn_GetLicenseList: TButton;
    tv_Licenses: TTreeView;
    mmo_Info: TMemo;
    pnl_Left: TPanel;
    spl_1: TSplitter;
    btn_GetSessions: TButton;
    procedure btn_GetLicenseListClick(Sender: TObject);
    procedure btn_GetSessionsClick(Sender: TObject);
    procedure tv_LicensesClick(Sender: TObject);
  private
    { Private declarations }
    FLicenseManager: TLicenseManager;
    FLicenseList: TLicenseList;
    FSessionList: TSessionList;
    FLisMode: Integer; // 0: License, 1: Sessions
    procedure FillLicenses;
    procedure FillSessions;
    procedure ShowInfo(LicenseId, ProductNuber, FeatureNuber, Level: Cardinal);
    procedure DoGetJsonData(Sender: TObject; const AParam: PAnsiChar);
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;



var
  frm_Licenses: Tfrm_Licenses;

implementation

uses
  uHelper, System.DateUtils;

{$R *.dfm}

{ Tfrm_Licenses }

constructor Tfrm_Licenses.Create(AOwner: TComponent);
begin
  inherited;
  FLicenseManager:= TLicenseManager.Create();
  FLicenseManager.OnGetJsonData:= DoGetJsonData;
  Description:= ' .';
end;

destructor Tfrm_Licenses.Destroy;
begin
  if Assigned(FLicenseManager) then
    FLicenseManager.Free;
  inherited;
end;

procedure Tfrm_Licenses.DoGetJsonData(Sender: TObject; const AParam: PAnsiChar);
var
  FileName: string;
  str: string;
  jLicenseList: TJSONObject;
  stl: TStringList;
begin
  jLicenseList:= TJSONObject.Create;
  jLicenseList.Parse(TEncoding.UTF8.GetBytes(string(AParam)),0);
  str:= jLicenseList.ToString;
  stl:= TStringList.Create();
  try
    stl.Add(str);

    FileName:= 'Data.json';
    stl.SaveToFile(FileName);
  finally
    FreeAndNil(stl);
  end;
end;

procedure Tfrm_Licenses.FillLicenses;
var
  NodeLicense, NodeProduct: TTreeNode;
  l,p,f: Integer;
  License: TLicenseInfo;
  Dongle: TDongleInfo;
  Product: TProduct;
  Feature: TFeature;
  licInfo, prodtInfo, featInfo: string;
  NodeInfo: PNodeInfo;
begin
  tv_Licenses.Items.Clear;

  for l:= 0 to FLicenseList.licenseCount - 1 do
  begin
    License:= FLicenseList.licenses[l].licenseInfo;
    Dongle:=  FLicenseList.licenses[l].dongleInfo;

    if License.licenseId = 0 then
      Continue;

    licInfo:= Format('%-8.8s', [ModelToStr(Dongle.dongleModel, Dongle.typeFlags)]);
    licInfo:= licInfo + ' ' + IntToHex(License.licenseId, 8);
    if FLicenseList.licenses[l].flagsAsFields.remote then
    begin
      licInfo:= licInfo + ' host: ' + FLicenseList.licenses[l].lm.hostName;
    end;

    NodeLicense:= tv_Licenses.Items.Add(nil, licInfo);

    for p:= 0 to License.productsCount - 1 do
    begin
      Product:= License.products[p];
      prodtInfo:= Product.name + ' (' + IntToStr(Product.number) + ')';

      NodeProduct:= tv_Licenses.Items.AddChild(NodeLicense, prodtInfo);//, NodeInfo);

      for f:= 0 to Product.featuresCount - 1 do
      begin
        Feature:= Product.features[f];
        featInfo:=  Format('%-20.20s', [Feature.name]) +  ' (' + IntToStr(Feature.number) + ')';

        NodeInfo:= new(PNodeInfo);
        NodeInfo.LicenseId:= License.licenseId;
        NodeInfo.ProductNuber:= Product.number;
        NodeInfo.FeatureNuber:= Feature.number;
        tv_Licenses.Items.AddChildObject(NodeProduct, featInfo, NodeInfo);
      end; // Features
    end;   // Products
  end;     // Licenses

  tv_Licenses.FullExpand;
end;

procedure Tfrm_Licenses.FillSessions;
var
  NodeSession: TTreeNode;
  i: Integer;
  Session: TSession;
  sessInfo: string;
begin
  tv_Licenses.Items.Clear;

  for i:= 0 to FSessionList.sessionCount - 1 do
  begin
    Session:= FSessionList.sessions[i];

    if Session.sessionId = 0 then
      Continue;

    sessInfo:= 'Session # ' + Session.sessionIdText;

    NodeSession:= tv_Licenses.Items.Add(nil,   sessInfo);

    tv_Licenses.Items.AddChild(NodeSession, 'sessionId: ' + Session.sessionId.ToString);
    tv_Licenses.Items.AddChild(NodeSession, 'sessionIdText: ' + Session.sessionIdText);

    tv_Licenses.Items.AddChild(NodeSession, 'licenseId: ' + Session.licenseId.ToHexString);
    tv_Licenses.Items.AddChild(NodeSession, 'featureNumber: ' + Session.featureNumber.ToString);

    tv_Licenses.Items.AddChild(NodeSession, 'host.fqdn: ' + Session.host.fqdn);
    tv_Licenses.Items.AddChild(NodeSession, 'host.hostId: ' + Session.host.hostId.ToString);
    tv_Licenses.Items.AddChild(NodeSession, 'host.ip: ' + Session.host.ip);
    tv_Licenses.Items.AddChild(NodeSession, 'host.name: ' + Session.host.name);

    tv_Licenses.Items.AddChild(NodeSession, 'issueTime: ' + Session.issueTime.ToString);
    tv_Licenses.Items.AddChild(NodeSession, 'issueTimeText: ' + Session.issueTimeText);

    tv_Licenses.Items.AddChild(NodeSession, 'processId: ' + Session.processId.ToString);
    tv_Licenses.Items.AddChild(NodeSession, 'processName: ' + Session.processName);
    tv_Licenses.Items.AddChild(NodeSession, 'processPath: ' + Session.processPath);
    tv_Licenses.Items.AddChild(NodeSession, 'productNumber: ' + Session.productNumber.ToString);

    tv_Licenses.Items.AddChild(NodeSession, 'user.id: ' + Session.user.id.ToString);
    tv_Licenses.Items.AddChild(NodeSession, 'user.name: ' + Session.user.name);

  end;     // Sessions

  tv_Licenses.FullExpand;
end;

procedure Tfrm_Licenses.ShowInfo(LicenseId, ProductNuber, FeatureNuber,
  Level: Cardinal);
var
  l,p,f: Integer;
  License: TLicenseInfo;
  Product: TProduct;
  Feature: TFeature;
const
  IntFmt = '%-25.25s: %d';
  StrFmt = '%-25.25s: %s';

procedure AddInt(AParam: string; AValue: Cardinal);
begin
  mmo_Info.Lines.Add(Format(IntFmt, [AParam, AValue]));
end;
procedure AddStr(AParam: string; AValue: string);
begin
  mmo_Info.Lines.Add(Format(StrFmt, [AParam, AValue]));
end;
procedure AddDate(AParam: string; AValue: Cardinal);
var
  Date: TDateTime;
begin
  Date:= UnixToDateTime(AValue);
  mmo_Info.Lines.Add(Format('%-25.25s: ', [AParam])  + FormatDateTime('dd.mm.yyyy hh:mm:ss', Date ) );
end;
begin
  mmo_Info.Clear;

  for l:= 0 to FLicenseList.licenseCount - 1 do
  begin
    License:= FLicenseList.licenses[l].licenseInfo;
    if LicenseId <> License.licenseId then
      Continue;

    if level = 0 then
    begin
      mmo_Info.Lines.Add('  ');
      AddStr('licenseId',           IntToHex(License.licenseId, 8));
      AddInt('currentUpdate',       License.currentUpdate);
      AddStr('vendorPublicCode',    IntToHex(License.vendorPublicCode));
      AddStr('vendorCompanyName',   License.vendorCompanyName);
      AddStr('customerFirstName',   License.customerFirstName);
      AddStr('customerLastName',    License.customerLastName);
      AddStr('customerEmail',       License.customerEmail);
      AddStr('customerCompanyName', License.customerCompanyName); // Fix 2023.01.30 - Kulikov
      break;
    end;

    for p:= 0 to License.productsCount - 1 do
    begin
      Product:= License.products[p];
      if ProductNuber <> Product.number then
        Continue;

      for f:= 0 to Product.featuresCount - 1 do
      begin
        Feature:= Product.features[f];
        if FeatureNuber <> Feature.number then
          Continue;

        if level = 2 then
        begin
          mmo_Info.Lines.Add('  ');
          AddInt('number',                 Feature.number);
          AddStr('name',                   Feature.name);
          AddInt('flags',                  Feature.flags);
          AddInt('remoteMode',             Feature.remoteMode);
          AddInt('consumptionMode',        Feature.consumptionMode);
          AddDate('validFromDate',         Feature.validFromDate);
          AddDate('validUpToDate',         Feature.validUpToDate);
          AddInt('restOfLifeTime',         Feature.restOfLifeTime);
          AddInt('maxRunCounter',          Feature.maxRunCounter);
          AddInt('maxConcurrentResource',  Feature.maxConcurrentResource);
          AddInt('floatingResource',       Feature.floatingResource);
          AddInt('reservedResource',       Feature.reservedResource);
          AddInt('detachedResource',       Feature.detachedResource);
          AddInt('sessionsCount',          Feature.sessionsCount);
          AddInt('currentRunCounterValue', Feature.currentRunCounterValue);
          AddInt('licenseType',            Feature.licenseType);
          break;
        end;
      end;

    end;
  end;
end;

procedure Tfrm_Licenses.btn_GetLicenseListClick(Sender: TObject);
var
  Visibility: TVisibility;
  RemoteMode: Integer;
begin
  Visibility.licenseId:= $FFFFFFFF;

  RemoteMode:= 0;
  if cb_LocalMode.Checked then
    RemoteMode:= RemoteMode + Integer(rmLocal);
  if cb_RemoteMode.Checked then
    RemoteMode:= RemoteMode + Integer(rmRemote);
  Visibility.remoteMode:= TRemoteMode(RemoteMode);
  Visibility.BroadcastSearch:= true;
  Visibility.HostName:= AnsiString(edt_HostName.Text);
  if Visibility.HostName <> '' then
    Visibility.BroadcastSearch:= false;
  Visibility.dongleModel:= dmAll;
  Visibility.productNumber:= $FFFFFFFF;
  Visibility.NetworkResourceToConsume:= 1;

  FLicenseManager.Visibility:= Visibility;
  FLicenseList:= FLicenseManager.GetLicenseInfo;
  SendLog('GrdGetLicenseInfo', FLicenseManager.Error);

  if FLicenseManager.Error <> 0 then
    Exit;

  FLisMode:= 0;
  FillLicenses;
end;

procedure Tfrm_Licenses.btn_GetSessionsClick(Sender: TObject);
var
  Visibility: TVisibility;
  RemoteMode: Integer;
begin
  Visibility.licenseId:= $FFFFFFFF;

  RemoteMode:= 0;
  if cb_LocalMode.Checked then
    RemoteMode:= RemoteMode + Integer(rmLocal);
  if cb_RemoteMode.Checked then
    RemoteMode:= RemoteMode + Integer(rmRemote);
  Visibility.remoteMode:= TRemoteMode(RemoteMode);
  Visibility.BroadcastSearch:= true;
  Visibility.HostName:= AnsiString(edt_HostName.Text);
  if Visibility.HostName <> '' then
    Visibility.BroadcastSearch:= false;
  Visibility.dongleModel:= dmAll;
  Visibility.productNumber:= $FFFFFFFF;
  Visibility.NetworkResourceToConsume:= 1;

  FLicenseManager.Visibility:= Visibility;
  FSessionList:= FLicenseManager.GetSessionInfo;
  SendLog('GrdGetSessionInfo', FLicenseManager.Error);

  if FLicenseManager.Error <> 0 then
    Exit;

  FLisMode:= 1;
  FillSessions;
end;

procedure Tfrm_Licenses.tv_LicensesClick(Sender: TObject);
var
  Node: TTreeNode;
  Info: PNodeInfo;
  Level: Integer;
begin
  if FLisMode = 1 then
    exit;

  Node:= tv_Licenses.Selected;
  if Node = nil then
    Exit;

  Level:= Node.Level;
  Info:= new(PNodeInfo);
  case Level of
    0: Info:= PNodeInfo(Node.getFirstChild.getFirstChild.Data);
    1: Info:= PNodeInfo(Node.getFirstChild.Data);
    2: Info:= PNodeInfo(Node.Data);
  end;

  ShowInfo(Info.LicenseId, Info.ProductNuber, Info.FeatureNuber, Level);
end;

end.
