unit fruRehost;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, fruBaseFrame, grdlic_delphi,
  System.ImageList, Vcl.ImgList, Vcl.StdCtrls, Vcl.ExtCtrls;

type
  Tfrm_Rehost = class(TBaseFrame)
    btn_GrdLicenseExtractForRehost: TButton;
    edt_LicenseId: TEdit;
    lbl_SerialNumber_Upd: TLabel;
    dlg_Save: TSaveDialog;
    dlg_Open: TOpenDialog;
    iml_Buttons: TImageList;
    Label2: TLabel;
    Label5: TLabel;
    edt_LicenseRehostFile: TButtonedEdit;
    lbl_ApplyRehost: TLabel;
    btn_OnlineLicenseRehost: TButton;
    grp_Common: TGroupBox;
    grp_OnlineRehost: TGroupBox;
    grp_OfflineRehost: TGroupBox;
    lbl_SendActivateRequest: TLabel;
    btn_ObtainLicense: TButton;
    lbl_LicenseFile: TLabel;
    edt_LicenseRehostResponseFile: TButtonedEdit;
    lbl_GetFingerprint: TLabel;
    lbl_FingerPrintFile: TLabel;
    edt_FingerPrintFile: TButtonedEdit;
    btn_GetFingerPrint: TButton;
    lbl_HostName: TLabel;
    edt_HostName: TEdit;
    lbl_InstallLicense: TLabel;
    btn_InstallLicense: TButton;
    procedure btn_GetFingerPrintClick(Sender: TObject);
    procedure btn_GrdLicenseExtractForRehostClick(Sender: TObject);
    procedure btn_InstallLicenseClick(Sender: TObject);
    procedure btn_ObtainLicenseClick(Sender: TObject);
    procedure btn_OnlineLicenseRehostClick(Sender: TObject);
    procedure edt_FingerPrintFileRightButtonClick(Sender: TObject);
    procedure edt_LicenseRehostFileRightButtonClick(Sender: TObject);
    procedure edt_LicenseRehostResponseFileRightButtonClick(Sender: TObject);
  private
    { Private declarations }
    FLicenseManager: TLicenseManager;
  public
    { Public declarations }
    constructor Create(AOwner: TComponent); override;
    destructor Destroy; override;
  end;

var
  frm_Rehost: Tfrm_Rehost;

implementation

uses
  uHelper;

{$R *.dfm}

procedure Tfrm_Rehost.btn_GrdLicenseExtractForRehostClick(Sender: TObject);
var
  PathToFile: string;
  Request: TBytes;
  LicenseID: Cardinal;
begin
  LicenseId:= StrToUInt('$' + edt_LicenseId.Text);

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseExtractForRehost(LicenseId, Request);
  SendLog('GrdLicenseExtractForRehost', FLicenseManager.Error);
  if FLicenseManager.Error <> 0 then
    Exit;

  dlg_Save.DefaultExt:= '.rehost';
  dlg_Save.InitialDir := GetCurrentDir;
  dlg_Save.FileName:= LocalCimputerName + '-' + edt_LicenseId.Text;
  dlg_Save.Filter:= 'rehost files|*.rehost|all files|*.*';
  if dlg_Save.Execute() = True then
    PathToFile:= dlg_Save.FileName
  else
    PathToFile:= dlg_Save.FileName + '.tmp';

  BufferToFile(PathToFile, Request);
  edt_LicenseRehostFile.Text:= PathToFile;
end;

constructor Tfrm_Rehost.Create(AOwner: TComponent);
begin
  inherited;
  FLicenseManager:= TLicenseManager.Create();
  Description:= ' .';
end;

destructor Tfrm_Rehost.Destroy;
begin
  if Assigned(FLicenseManager) then
    FLicenseManager.Free;
  inherited;
end;

procedure Tfrm_Rehost.btn_GetFingerPrintClick(Sender: TObject);
var
  Visibility: TVisibility;
  Fingerprint: PAnsiChar;
  PathToFile: String;
  Buffer: TBytes;
begin
  Visibility.LicenseId:= 0;
  Visibility.RemoteMode:= rmRemote;
  Visibility.DongleModel:= dmDl;
  Visibility.ProductNumber:= 0;
  Visibility.NetworkResourceToConsume:= 1;

  Visibility.BroadcastSearch:= false;
  Visibility.HostName:= AnsiString(edt_HostName.Text);

  FLicenseManager.Visibility:= Visibility;
  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;

  FLicenseManager.GetHostFingerprint(Fingerprint);
  SendLog('GetHostFingerprint', FLicenseManager.Error);
  if FLicenseManager.Error <> 0 then
    Exit;

  dlg_Save.DefaultExt:= '.fingerprint';
  dlg_Save.InitialDir := GetCurrentDir;
  dlg_Save.FileName:= edt_HostName.Text;
  dlg_Save.Filter:= 'fingerprint files|*.fingerprint|all files|*.*';
  if dlg_Save.Execute then
    PathToFile:= dlg_Save.FileName
  else
    Exit;

  SetLength(Buffer, Length(Fingerprint));
  Move(Pointer(Fingerprint)^, Pointer(Buffer)^, Length(Fingerprint));

  BufferToFile(PathToFile, Buffer);
  edt_FingerPrintFile.Text:= PathToFile;

  SetLength(Buffer, 0);
end;

procedure Tfrm_Rehost.btn_InstallLicenseClick(Sender: TObject);
var
  PathToResponseFile: string;
  Response: TBytes;
begin
  if not FileExists(edt_LicenseRehostResponseFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.license';
    dlg_Open.InitialDir:= GetCurrentDir;
    dlg_Open.FileName:= '';
    dlg_Open.Filter:= 'license files|*.license|all files|*.*';
    if dlg_Open.Execute then
      PathToResponseFile:= dlg_Open.FileName
    else
      Exit;
  end
  else
    PathToResponseFile:= edt_LicenseRehostResponseFile.Text;

  FileToBuffer(PathToResponseFile, Response);

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseInstall(Response);
  SendLog('GrdLicenseInstall', FLicenseManager.Error);
  SetLength(Response, 0);
end;

procedure Tfrm_Rehost.btn_ObtainLicenseClick(Sender: TObject);
var
  PathToDonorLicenseFile: string;
  DonorLicense: TBytes;
  PathToFingerPrintFile: string;
  Buffer: TBytes;
  FingerPrint: AnsiString;
  PathTorecipientLicenseFile: string;
  recipientLicense: TBytes;
begin
  if not FileExists(edt_LicenseRehostFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.rehost';
    dlg_Open.InitialDir := GetCurrentDir;
    dlg_Open.Filter:= 'rehost files|*.rehost|all files|*.*';
    if dlg_Open.Execute then
      PathToDonorLicenseFile:= dlg_Open.FileName
    else
      Exit;
  end
  else
    PathToDonorLicenseFile:= edt_LicenseRehostFile.Text;

  FileToBuffer(PathToDonorLicenseFile, DonorLicense);

  if not FileExists(edt_FingerPrintFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.fingerprint';
    dlg_Open.InitialDir:= GetCurrentDir;
    dlg_Open.FileName:= '';
    dlg_Open.Filter:= 'fingerprint files|*.fingerprint|all files|*.*';
    if dlg_Open.Execute then
      PathToFingerPrintFile:= dlg_Open.FileName
    else
      Exit;
  end
  else
    PathToFingerPrintFile:= edt_FingerPrintFile.Text;

  FileToBuffer(PathToFingerPrintFile, Buffer);
  SetLength(FingerPrint, Length(Buffer));
  Move(Pointer(Buffer)^, Pointer(FingerPrint)^, Length(Buffer));

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseSendRehostRequest(DonorLicense, PAnsiChar(FingerPrint), recipientLicense);
  SendLog('GrdLicenseSendRehostRequest', FLicenseManager.Error);
  if FLicenseManager.Error <> 0 then
    Exit;

  dlg_Save.DefaultExt:= '.license';
  dlg_Save.InitialDir:= GetCurrentDir;
  dlg_Open.FileName:= '';
  dlg_Save.Filter:= 'license files|*.license|all files|*.*';
  if dlg_Save.Execute then
    PathTorecipientLicenseFile:= dlg_Save.FileName
  else
    PathTorecipientLicenseFile:= dlg_Save.FileName + '.tmp';

  BufferToFile(PathTorecipientLicenseFile, recipientLicense);
  edt_LicenseRehostResponseFile.Text:= PathTorecipientLicenseFile;
end;

procedure Tfrm_Rehost.btn_OnlineLicenseRehostClick(Sender: TObject);
var
  PathToResponseFile: string;
  Response: TBytes;
begin
  if not FileExists(edt_LicenseRehostFile.Text) then
  begin
    dlg_Open.DefaultExt:= '.rehost';
    dlg_Open.InitialDir := GetCurrentDir;
    dlg_Open.Filter:= 'rehost files|*.rehost|all files|*.*';
    if not dlg_Open.Execute then
      Exit
    else
      PathToResponseFile:= dlg_Open.FileName;
  end
  else
    PathToResponseFile:= edt_LicenseRehostFile.Text;

  FileToBuffer(PathToResponseFile, Response);

  FLicenseManager.Host:= GetHost;
  FLicenseManager.Port:= GetPort;
  FLicenseManager.LicenseRehost(Response);
  SendLog('GrdLicenseRehost', FLicenseManager.Error);
  if FLicenseManager.Error <> 0 then
    Exit;
  SetLength(Response, 0);
end;

procedure Tfrm_Rehost.edt_FingerPrintFileRightButtonClick(Sender: TObject);
begin
  dlg_Open.DefaultExt:= '.fingerprint';
  dlg_Open.InitialDir := GetCurrentDir;
  dlg_Open.FileName:= '';
  dlg_Open.Filter:= 'fingerprint files|*.fingerprint|all files|*.*';
  if dlg_Open.Execute then
    edt_FingerPrintFile.Text:= dlg_Open.FileName;
end;

procedure Tfrm_Rehost.edt_LicenseRehostFileRightButtonClick(Sender: TObject);
begin
  dlg_Open.DefaultExt:= '.rehost';
  dlg_Open.InitialDir := GetCurrentDir;
  dlg_Open.Filter:= 'rehost files|*.rehost|all files|*.*';
  if dlg_Open.Execute then
    edt_LicenseRehostFile.Text:= dlg_Open.FileName;
end;

procedure Tfrm_Rehost.edt_LicenseRehostResponseFileRightButtonClick(Sender:
    TObject);
begin
  dlg_Open.DefaultExt:= '.license';
  dlg_Open.InitialDir:= GetCurrentDir;
  dlg_Open.FileName:= '';
  dlg_Open.Filter:= 'license files|*.license|all files|*.*';
  if dlg_Open.Execute then
    edt_LicenseRehostResponseFile.Text:= dlg_Open.FileName;
end;

end.
