unit fuMainForm;

interface

uses
  Winapi.Windows, Winapi.Messages, System.SysUtils, System.Variants, System.Classes, Vcl.Graphics,
  Vcl.Controls, Vcl.Forms, Vcl.Dialogs, System.ImageList, Vcl.ImgList,
  Vcl.CategoryButtons, Vcl.ExtCtrls, Vcl.ButtonGroup, Vcl.Imaging.pngimage,
  Vcl.ToolWin, Vcl.ActnMan, Vcl.ActnCtrls, System.Actions, Vcl.ActnList,
  Vcl.PlatformDefaultStyleActnCtrls, Vcl.StdCtrls, fruBaseFrame, fruParams, fruActivation, fruRehost, fruDetach, fruLicenses, fruExample;

type
  TfmMainForm = class(TForm)
    pnl_Actions: TPanel;
    iml_Frames: TImageList;
    img_Frame_0: TImage;
    img_Frame_1: TImage;
    img_Frame_2: TImage;
    img_Frame_3: TImage;
    img_Frame_4: TImage;
    pnl_Frames: TPanel;
    pnl_Log: TPanel;
    spl_1: TSplitter;
    mmo_Log: TMemo;
    pnl_Top: TPanel;
    img_Frame_5: TImage;
    procedure FormCreate(Sender: TObject);
    procedure act_ExitExecute(Sender: TObject);
  private
    { Private declarations }
    FCurrentFrameIndex: Integer;
    Frame_0: Tfrm_Params;
    Frame_1: Tfrm_Activation;
    Frame_2: Tfrm_Rehost;
    Frame_3: Tfrm_Detach;
    Frame_4: Tfrm_Licenses;
    Frame_5: Tfrm_Example;
    procedure OnImageClick(Sender: TObject);
    procedure DefaultIcons;
    procedure HotIcon(Sender: TObject);
    procedure ChangeFrame(AFrameIndex: Integer);
    procedure GetLogEvent(const AFunction: string; AResult: Integer);
  public
    { Public declarations }
  end;

var
  fmMainForm: TfmMainForm;

implementation

uses
  grdlic_delphi;

{$R *.dfm}
const
  FORM_CAPTION = '   Guardant Feature API.';

procedure TfmMainForm.FormCreate(Sender: TObject);
var
  s: string;
begin
  FCurrentFrameIndex:= -1;
  img_Frame_0.OnClick:=  OnImageClick;
  img_Frame_1.OnClick:=  OnImageClick;
  img_Frame_2.OnClick:=  OnImageClick;
  img_Frame_3.OnClick:=  OnImageClick;
  img_Frame_4.OnClick:=  OnImageClick;
  img_Frame_5.OnClick:=  OnImageClick;

  Frame_0:= Tfrm_Params.Create(Self);
  Frame_0.Parent:= pnl_Frames;
  Frame_0.Align:= alClient;
  Frame_0.Hide;

  Frame_1:= Tfrm_Activation.Create(Self);
  Frame_1.Parent:= pnl_Frames;
  Frame_1.Align:= alClient;
  Frame_1.Hide;
  Frame_1.OnLog:= GetLogEvent;

  Frame_2:= Tfrm_Rehost.Create(Self);
  Frame_2.Parent:= pnl_Frames;
  Frame_2.Align:= alClient;
  Frame_2.Hide;
  Frame_2.OnLog:= GetLogEvent;

  Frame_3:= Tfrm_Detach.Create(Self);
  Frame_3.Parent:= pnl_Frames;
  Frame_3.Align:= alClient;
  Frame_3.Hide;
  Frame_3.OnLog:= GetLogEvent;

  Frame_4:= Tfrm_Licenses.Create(Self);
  Frame_4.Parent:= pnl_Frames;
  Frame_4.Align:= alClient;
  Frame_4.Hide;
  Frame_4.OnLog:= GetLogEvent;

  Frame_5:= Tfrm_Example.Create(Self);
  Frame_5.Parent:= pnl_Frames;
  Frame_5.Align:= alClient;
  Frame_5.Hide;
  Frame_5.OnLog:= GetLogEvent;

  OnImageClick(img_Frame_0);

  s:= Format('%-35.35s | %-9.9s | %s', ['', '', ' ']);
  mmo_Log.Lines.Add(s);
end;

procedure TfmMainForm.GetLogEvent(const AFunction: string; AResult: Integer);
var
  s: string;
begin
  s:= Format('%-35.35s | %9d | %s', [AFunction, AResult, TLicenseManager.ErrorMessage(AResult)]);

  mmo_Log.Lines.Add(s);
end;

procedure TfmMainForm.ChangeFrame(AFrameIndex: Integer);
var
  Frame: TBaseFrame;
  i: Integer;
begin
  for i:= 0 to Self.ComponentCount - 1 do
  Begin
    if Self.Components[i] is TBaseFrame then
    begin
      Frame:= (Self.Components[i] as TBaseFrame);
      if Frame.Tag = AFrameIndex then
      begin
        Self.Caption:= FORM_CAPTION + ' ' + (Frame as TBaseFrame).Description;
        Frame.Show;
      end
      else
        Frame.Hide;
    end;
  End;
end;

procedure TfmMainForm.DefaultIcons;
var
  Icon: TIcon;
  i: Integer;
  Image: TImage;
begin
  Icon:= TIcon.Create;

  try
    for i:= 0 to Self.ComponentCount - 1 do
    begin
      if Self.Components[i] is TImage then
      begin
        Image:= Self.Components[i] as TImage;
        if Image.Tag = FCurrentFrameIndex then
          Continue;

        iml_Frames.GetIcon(Image.Tag * 2, Icon);
        Image.Picture.Icon.Assign(Icon);
      end;
    end;
  finally
    Icon.Free;
  end;
end;

procedure TfmMainForm.HotIcon(Sender: TObject);
var
  Icon: TIcon;
  Image: TImage;
begin
  if not (Sender is TImage) then
    exit;
  Image:= Sender as TImage;

  Icon:= TIcon.Create;
  try
    if Image.Tag = FCurrentFrameIndex then
    begin
      iml_Frames.GetIcon((Image.Tag*2) + 1, Icon);
      Image.Picture.Icon.Assign(Icon);
    end;
  finally
    Icon.Free;
  end;
end;

procedure TfmMainForm.act_ExitExecute(Sender: TObject);
begin
  Close;
end;

procedure TfmMainForm.OnImageClick(Sender: TObject);
begin
  FCurrentFrameIndex:= (Sender as TImage).Tag;
  HotIcon(Sender);
  DefaultIcons;
  ChangeFrame(FCurrentFrameIndex);
end;

end.
