unit uHelper;

interface

uses
  System.SysUtils, System.IniFiles, Vcl.Dialogs, Vcl.Forms, Winapi.Windows;

type
  TExampleFeature = record
    Initialised:   Boolean;
    FeatureNumber: Cardinal;
    LicenseID:     Cardinal;
    ProductNumber: Cardinal;
    RemoteMode:    Cardinal;
    PublicCode:    Cardinal;
    ReadCode:      Cardinal;
    WriteCode:     Cardinal;
  end;

type
  TLogEvent = procedure(const AFunction: string; AResult: Integer) of object;

  function ArrayToStr(Buffer: array of byte): string;
  function ValToInt(Value: string): Cardinal;
  function GetHost(): string;
  function GetPort(): Cardinal;
  procedure WriteParam(ASection, AIdent, AValue: string);
  function ReadParam(ASection, AIdent, ADefault: string): string;
  function LocalCimputerName(): string;
  function ModelToStr(AModel, AType: Cardinal): string;

implementation

uses
  grdlic;

function ArrayToStr(Buffer: array of byte): string;
const
  Hex: array[0..15] of char =
    ('0','1','2','3','4','5','6','7','8','9','A','B','C','D','E','F');
var
  i, len: integer;
begin
  len:= Length(Buffer);
  SetLength(Result, len*2);

  for i := 0 to len - 1 do
  begin
    Result[i*2+1]:= Hex[Buffer[(i)] shr 4];
    Result[i*2+2]:= Hex[Buffer[(i)] and $0F];
  end;
end;

function ValToInt(Value: string): Cardinal;
var
  ErrPos: Integer;
begin
  Result:= 0;
  if Value = '' then
    Exit;

  // Is Int
  Val(Trim(Value), Result, ErrPos);
  if ErrPos = 0 then
  begin
    Exit;
  end;

  // Is hex
  Val('$' + Trim(Value), Result, ErrPos);
  if ErrPos <> 0 then
    ShowMessage('Incorrect value: ' + Value + ' at position ' + IntToStr(ErrPos - 1));
end;

function GetHost(): string;
var
  IniFile: TCustomIniFile;
  FilePath: string;
begin
  //Result:= 'getlicense.guardant.ru';
  FilePath:= ExtractFilePath(Application.ExeName) + 'params.ini';
  IniFile:= TIniFile.Create(FilePath);
  try
    Result:= IniFile.ReadString('SERVER', 'host', 'getlicense.guardant.ru');
  finally
    IniFile.Free;
  end;
end;

function GetPort(): Cardinal;
var
  IniFile: TCustomIniFile;
  FilePath: string;
begin
  //Result:= 443;
  FilePath:= ExtractFilePath(Application.ExeName) + 'params.ini';
  IniFile:= TIniFile.Create(FilePath);
  try
    Result:= StrToUint(IniFile.ReadString ('SERVER', 'port', '443'));
  finally
    IniFile.Free;
  end;
end;

procedure WriteParam(ASection, AIdent, AValue: string);
var
  IniFile: TCustomIniFile;
  FilePath: string;
begin
  FilePath:= ExtractFilePath(Application.ExeName) + 'params.ini';
  IniFile:= TIniFile.Create(FilePath);
  try
    IniFile.WriteString(ASection, AIdent, AValue);
  finally
    IniFile.Free;
  end;
end;

function ReadParam(ASection, AIdent, ADefault: string): string;
var
  IniFile: TCustomIniFile;
  FilePath: string;
begin
  FilePath:= ExtractFilePath(Application.ExeName) + 'params.ini';
  IniFile:= TIniFile.Create(FilePath);
  try
    Result:= IniFile.ReadString(ASection, AIdent, ADefault).Trim;
  finally
    IniFile.Free;
  end;
end;

function LocalCimputerName(): string;
var
  Buffer: array[0..MAX_COMPUTERNAME_LENGTH + 1] of Char;
  Size: Cardinal;
begin
  Size := MAX_COMPUTERNAME_LENGTH + 1;
  GetComputerName(@Buffer, Size);
  Result := StrPas(Buffer);
end;

function ModelToStr(AModel, AType: Cardinal): string;
var
  TimeFlag: string;
  NetFlag: string;
begin
  Result:= '';
  TimeFlag:= '';
  NetFlag:= '';

  if (AType and 2) = 2 then
    TimeFlag:= ' Time';
  if (AType and 1) = 1 then
    NetFlag:= ' Net';

  case AModel of
    GRD_DONGLE_MODEL_SIGN:
    begin
      Result:= 'Sign';
      if (AType and 2) = 2 then
        Result:= 'Time';
      Result:= Result + NetFlag;
    end;
    GRD_DONGLE_MODEL_CODE:
      Result:= 'Code' + TimeFlag + NetFlag;
    GRD_DONGLE_MODEL_DL:
      Result:= 'DL'+ NetFlag;
  end;
end;
end.
