package JavaDemo;
/*
 * JavaDemo.java
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
import grdlic.*;

public class JavaDemo
{
    public static void main(String[] args) throws Exception
    {
        System.out.println("------------------------------------------------------------------------------");
        System.out.println("      Guardant Software Licensing Kit example for Java");
        System.out.println("          ( C ) 2020 Aktiv Co. All rights reserved");

        // --------------------------------------------------------------------------------------------
        // Load native library grdlic_java
        // --------------------------------------------------------------------------------------------

        System.out.println("------------------------------------------------------------------------------");
        String libName = (System.getProperty("sun.arch.data.model").equals("32")?"x86/":"x86_64/") + "grdlic_java";
        System.out.print("Load native sdk library " + libName + " : ");
        try
        {
            System.loadLibrary(libName);
            ErrorHandling(GrdStatus.OK);
        }
        catch (UnsatisfiedLinkError e)
        {
            // Terminate application
            System.out.println("Library not found");
            System.out.println(e);
            System.exit(-1);
        }

        GrdInt majorVersion = new GrdInt();
        GrdInt minorVersion = new GrdInt();
        int apiVersion = api.GrdGetApiVersion(majorVersion, minorVersion);
        System.out.format("Grdlic Api version is: %d (%d.%d)\n", apiVersion, majorVersion.getValue(), minorVersion.getValue());

        GrdVendorCodes codes = new GrdVendorCodes(0x519175b7, 0x51917645, 0x51917603); // DEMO codes
        String visibility = "{\"dongleModel\": 0x480, \"remoteMode\": 3}";
        System.out.print("Get licenses info: ");
        GrdString licensesListJson = new GrdString();
        GrdStatus status = api.GrdGetLicenseInfo(visibility, null, licensesListJson);
        ErrorHandling(status);
        System.out.println("Found licenses:\n" + licensesListJson.getValue());

        // next you need to choose some feature to work with (i.e. by parsing licensesListJson), but we
        // simply choose arbitrary feature number 1 (notice: the feature with this number may not exists
        // in your license
        int featureNumber = 1;
        System.out.format("\nChosen feature number is %d:\n", featureNumber);
        Feature feature = new Feature(featureNumber);
        System.out.print("Try to login feature: ");
        status = feature.Login(codes, visibility);
        ErrorHandling(status);

        System.out.print("Call check feature: ");
        status = feature.Check(null);
        ErrorHandling(status);

        System.out.print("Get feature info: ");
        GrdString outFeatureInfo = new GrdString();
        status = feature.GetInfo(outFeatureInfo);
        ErrorHandling(status);
        System.out.format("Feature info:\n%s\n\n", outFeatureInfo.getValue());

        byte[] data = new byte[48];
        System.out.format("Data before encryption: %s\n", Arrays.toString(data));
        System.out.print("Encrypt data: ");
        status = feature.Encrypt(data, GrdFeatureEncryptMode.GRD_EM_ECB, null);
        ErrorHandling(status);
        System.out.format("Data after encryption: %s\n", Arrays.toString(data));

        System.out.print("Decrypt data: ");
        status = feature.Decrypt(data, GrdFeatureEncryptMode.GRD_EM_ECB, null);
        ErrorHandling(status);
        System.out.format("Data after decryption: %s\n", Arrays.toString(data));

        System.out.print("Sign message: ");
        byte[] message = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09}; // of GrdEcc160.GRD_ECC160_MESSAGE_SIZE length
        byte[] digest = new byte[GrdEcc160.GRD_ECC160_DIGEST_SIZE];
        status = feature.Sign(message, digest);
        ErrorHandling(status);
        System.out.format("Message digest content: %s\n", Arrays.toString(digest));

//        System.out.print("Verify digest: ");
//        byte[] somePublicKey = {0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x00, 0x01,
//                                0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09, 0x00, 0x01, 0x02, 0x03, 0x04, 0x05, 0x06, 0x07, 0x08, 0x09}; // of GrdEcc160.GRD_ECC160_PUBLIC_KEY_SIZE length
//        status = api.GrdVerifyDigest(somePublicKey, message, digest);
//        ErrorHandling(status);

//        System.out.print("Get feature time limit: ");
//        GrdInt remainingTime = new GrdInt();
//        status = feature.GetTimeLimit(remainingTime);
//        ErrorHandling(status);
//        System.out.println("Feature remaining time is : " + SecondsToDateString(remainingTime.getValue()));

//        System.out.print("Get run counter value: ");
//        GrdInt runCounter = new GrdInt();
//        status = feature.GetRunCounter(runCounter);
//        ErrorHandling(status);
//        System.out.format("Run counter value is: %d\n", runCounter.getValue());

        System.out.print("Get current time: ");
        GrdInt currentTime = new GrdInt();
        status = feature.GetRealTime(currentTime);
        ErrorHandling(status);
        System.out.println("Current time is :\n" + SecondsToDateString(currentTime.getValue()));

        System.out.print("Get max concurrent net resource: ");
        GrdInt maxResource = new GrdInt();
        status = feature.GetMaxConcurrentResource(maxResource);
        ErrorHandling(status);
        System.out.format("Max concurrent net resource is: %d\n", maxResource.getValue());

//        System.out.print("Read memory: ");
//        int memoryId = 777; // set your own memory ID
//        int memSize = 48; // set your own memory size
//        int password = 0; // set your own access password
//        byte[] memory = new byte[memSize];
//        status = feature.MemoryRead(memoryId, password, 0, memory);
//        ErrorHandling(status);

//        System.out.print("Write memory: ");
//        status = feature.MemoryWrite(memoryId, password, GrdMemoryWriteMode.GRD_WRITE_MODE_MOV, 0, memory);
//        ErrorHandling(status);

        System.out.print("Logout feature: ");
        status = feature.Logout();
        ErrorHandling(status);

        System.out.print("Blink dongle LED: ");
        int dongleId = 0x3C716771; // set your own memory size
        status = api.GrdLedBlink(dongleId);
        ErrorHandling(status);

        System.out.print("Set dongle driver type (need dongle re-plug after operation): ");
        status = api.GrdSetDriverType(codes, dongleId, GrdDriverType.GRD_DRIVER_TYPE_WINUSB);
        ErrorHandling(status);
    }

    private static void ErrorHandling(GrdStatus status)
    {
        if (!status.equals(GrdStatus.OK))
        {
            System.out.print("Error! ");
            System.out.println(status.getValue());
            throw new GrdlicException("Grdlic API error");
        }
        else
            System.out.println("Success");
    }

    private static String SecondsToDateString(int seconds)
    {
        Date date = new Date(1000* (long)seconds);
        SimpleDateFormat sdf = new SimpleDateFormat("dd MMM yyyy hh:mm:ss zzz", Locale.ENGLISH);
        sdf.setTimeZone(TimeZone.getTimeZone("UTC"));
        return sdf.format(date);
    }
}

class GrdlicException extends RuntimeException
{
    private static final long serialVersionUID = 1L;

    public GrdlicException(String msg)
    {
        super(msg);
    }
};
