//------------------------------------------------------------------------------
// (C) 1997 - 2019 Aktiv Co. All rights reserved.
//  Guardant License activation API example C
//------------------------------------------------------------------------------
// This example demonstrates Guardant Licensing API usage.
// This code sample is for API demonstration only!
// Please do not use this code in real projects!
//------------------------------------------------------------------------------
#include <stdio.h>
#include <stdlib.h>
#include <string.h>
#ifdef WIN32
#include <windows.h>
#endif
#include <grdlic.h>

void PrintError(const char* Text);
void PrintText(const char* Text);
int PrintCode(int nErrorCode);
int ErrorHandling(int nRet);

//----------------------------------------------------------------------------------
//  Main function
//----------------------------------------------------------------------------------
int main(int argc, char** argv)
{
    int status = GRD_OK;
    GrdCustomerInfo customerInfo = {0};
    grd_int32 licenseId = 0;
    const char* serial = argv[1];
    const char* host = argv[2];
    const char* portStr = argv[3];
    if (argc < 4)
    {
        printf("Usage:"
               "\tdemo <serial> <host> <port>\n");
        exit(-1);
    }

    //------------------------------------------------------------------------------
    // Set customer info
    //------------------------------------------------------------------------------
    strcpy(customerInfo.companyName, "demo");
    strcpy(customerInfo.email, "demo@customer.ru");
    strcpy(customerInfo.phone, "+7(777)777-77-77");

    //------------------------------------------------------------------------------
    // Activate License
    //------------------------------------------------------------------------------
    printf("Activate license: ");
    status = GrdLicenseActivate(serial, host, atoi(portStr), &customerInfo, &licenseId);
    ErrorHandling(status);

    //------------------------------------------------------------------------------
    // Get license info
    //------------------------------------------------------------------------------
    {
        const char* visibilityFmt = "{\"licenseId\": %d, \"dongleModel\": 0x400, \"remoteMode\": 1}";
        size_t size = snprintf(NULL, 0, visibilityFmt, licenseId) + 1;   // Extra space for '\0'
        char* visibilityJson = (char*)malloc(size);
        memset(visibilityJson, 0, size);
        snprintf(visibilityJson, size, visibilityFmt, licenseId);

        char* licenseInfoList = NULL;
        printf("Retrieve information about license: ");
        status = GrdGetLicenseInfo(visibilityJson, NULL, &licenseInfoList);
        ErrorHandling(status);

        printf("Information about license:\n%s\n", licenseInfoList);
        GrdFree(licenseInfoList);
    }

    //------------------------------------------------------------------------------
    // Get license info after activation
    //------------------------------------------------------------------------------
    printf("Check license update after initial activation: ");
    status = GrdLicenseCheckUpdateIsAvailable(licenseId, host, atoi(portStr));
    if (status == GRD_NO_UPDATES_AVAILABLE)
    {
        PrintText("no updates available. \n\nSUCCESSFULLY DONE!\n");
        exit(0);
    }
    ErrorHandling(status);


    //------------------------------------------------------------------------------
    // License update
    //------------------------------------------------------------------------------
    printf("Update license: ");
    status = GrdLicenseUpdate(licenseId, host, atoi(portStr));
    ErrorHandling(status);

    //------------------------------------------------------------------------------
    // Get license info after license update
    //------------------------------------------------------------------------------
    {
        const char* visibilityFmt = "{\"licenseId\": %d, \"dongleModel\": 0x400, \"remoteMode\": 1}";
        size_t size = snprintf(NULL, 0, visibilityFmt, licenseId) + 1;   // Extra space for '\0'
        char* visibilityJson = (char*)malloc(size);
        memset(visibilityJson, 0, size);
        snprintf(visibilityJson, size, visibilityFmt, licenseId);
        printf("Retrieve information about license: ");
        char* licenseInfoList = NULL;
        status = GrdGetLicenseInfo(visibilityJson, NULL, &licenseInfoList);
        ErrorHandling(status);

        printf("Information about license:\n%s\n", licenseInfoList);
        GrdFree(licenseInfoList);
    }

    //------------------------------------------------------------------------------
    // Check license update one more time
    //------------------------------------------------------------------------------
    printf("Check license update after update [must return GRD_NO_UPDATES_AVAILABLE]: ");
    status = GrdLicenseCheckUpdateIsAvailable(licenseId, host, atoi(portStr));
    if (status == GRD_NO_UPDATES_AVAILABLE)
    {
        PrintText("no updates available. \n\nSUCCESSFULLY DONE!\n");
        exit(0);
    }
    ErrorHandling(status);

    PrintText("\n\nWARNING: updates still available!\n");

    return 1;
}

//----------------------------------------------------------------------------------------------
// Prints operation result and forces program termination on error
//----------------------------------------------------------------------------------------------
void PrintError(const char* Text)
{
#ifdef WIN32
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), FOREGROUND_RED);
    printf("%s\n", Text);
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_BLUE);
#elif defined(__linux) || defined(__linux__)
    printf("\033[1;31m");
    printf("%s\n", Text);
    printf("\033[0m");
#else
    printf("%s\n", Text);
#endif
}

//----------------------------------------------------------------------------------------------
// Prints operation result and forces program termination on error
//----------------------------------------------------------------------------------------------
void PrintText(const char* Text)
{
#ifdef WIN32
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), FOREGROUND_GREEN);
    printf("%s\n", Text);
    SetConsoleTextAttribute(GetStdHandle(STD_OUTPUT_HANDLE), FOREGROUND_GREEN | FOREGROUND_RED | FOREGROUND_BLUE);
#elif defined(__linux) || defined(__linux__)
    printf("\033[1;32m");
    printf("%s\n", Text);
    printf("\033[0m");
#else
    printf("%s\n", Text);
#endif
}

//----------------------------------------------------------------------------------------------
// Prints operation result and forces program termination on error
//----------------------------------------------------------------------------------------------
int PrintCode(int nErrorCode)
{
    // Buffer for error string
    if (nErrorCode == GRD_OK)
        PrintText("success");
    else
    {
        char szErrorMsg[256];
        sprintf(szErrorMsg, "ERROR! %d ", nErrorCode);
        PrintError(szErrorMsg);
    }
    return nErrorCode;
}

//----------------------------------------------------------------------------------------------
// Handle errors
// Prints operation result, logout from feature
//----------------------------------------------------------------------------------------------
int ErrorHandling(int nRet)
{
    PrintCode(nRet);

    if (nRet != GRD_OK)
    {
        // Terminate application
        PrintError("Application terminated by exit()");
        exit(nRet);
    }
    return nRet;
}
//------------------------------------------------------------------------------
