package JavaActivationDemo;
/*
 * JavaActivationDemo.java
 * To change this template, choose Tools | Template Manager
 * and open the template in the editor.
 */

import java.io.*;
import java.util.*;
import java.text.SimpleDateFormat;
import grdlic.*;

public class JavaActivationDemo
{
    public static void main(String[] args) throws Exception
    {
        if (args.length < 1)
        {
            System.out.println("Usage:\n demo serial_number_string");
            System.exit(-1);
        }

        System.out.println("------------------------------------------------------------------------------");
        System.out.println("      Guardant Software Licensing Kit example for Java");
        System.out.println("          ( C ) 2020 Aktiv Co. All rights reserved");

        // --------------------------------------------------------------------------------------------
        // Load native library grdlic_java
        // --------------------------------------------------------------------------------------------

        System.out.println("------------------------------------------------------------------------------");
        String libName = (System.getProperty("sun.arch.data.model").equals("32")?"x86/":"x86_64/") + "grdlic_java";
        System.out.print("Load native sdk library " + libName + " : ");
        try
        {
            System.loadLibrary(libName);
            ErrorHandling(GrdStatus.OK);
        }
        catch (UnsatisfiedLinkError e)
        {
            // Terminate application
            System.out.println("Library not found");
            System.out.println(e);
            System.exit(-1);
        }

        // setup host name and port for activations&updates
        LicenseActivator la = new LicenseActivator("getlicense.guardant.ru", 443);

        String serial = args[0];
        System.out.format("Check serial number %s: ", serial);
        GrdStatus status = la.CheckSerialNumberFormat(serial);
        ErrorHandling(status);

        System.out.print("Activate license: ");
        GrdInt licenseId = new GrdInt();
        GrdCustomerInfo customerInfo = new GrdCustomerInfo("Ivan", "Ivanov", "invanov@gmail.com", "LLC Ivanov", "");
        status = la.Activate(serial, customerInfo, licenseId);
        ErrorHandling(status);
        System.out.format("License ID is %d\n", licenseId.getValue());

        System.out.print("Check license update is available: ");
        status = la.CheckUpdateIsAvailable(licenseId.getValue());
        ErrorHandling(status);

        System.out.print("Update license: ");
        status = la.Update(licenseId.getValue());
        ErrorHandling(status);

        System.out.print("Check license is not banned: ");
        status = la.CheckIsNotBanned(licenseId.getValue());
        ErrorHandling(status);

        System.out.print("Remove license: ");
        status = la.Remove(licenseId.getValue());
        ErrorHandling(status);


/*      OFFLINE ACTIVATION      */
//        System.out.print("Create activation request: ");
//        GrdByteArray request = new GrdByteArray();
//        status = la.CreateActivationRequest(request);
//        ErrorHandling(status);

//        System.out.print("Send activation request: ");
//        GrdByteArray response = new GrdByteArray();
//        status = la.SendActivationRequest(serial, customerInfo, request.mValue, response);
//        ErrorHandling(status);

//        System.out.print("Install license: ");
//        status = la.Install(response.mValue);
//        ErrorHandling(status);

//        System.out.print("Create update request: ");
//        GrdByteArray updateRequest = new GrdByteArray();
//        status = la.CreateUpdateRequest(licenseId.getValue(), updateRequest);
//        ErrorHandling(status);

//        System.out.print("Send update request: ");
//        GrdByteArray updateResponse = new GrdByteArray();
//        status = la.SendUpdateRequest(updateRequest.mValue, updateResponse);
//        ErrorHandling(status);

//        System.out.print("Install update license: ");
//        status = la.Install(updateResponse.mValue);
//        ErrorHandling(status);
    }

    private static void ErrorHandling(GrdStatus status)
    {
        if (status.equals(GrdStatus.NO_UPDATES_AVAILABLE))
            System.out.println("No updates available");
        else if (!status.equals(GrdStatus.OK))
        {
            System.out.print("Error! ");
            System.out.println(status.getValue());
            throw new GrdlicException("Grdlic API error");
        }
        else
            System.out.println("Success");
    }
}

class GrdlicException extends RuntimeException
{
    private static final long serialVersionUID = 1L;

    public GrdlicException(String msg)
    {
        super(msg);
    }
};
